/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef LIB_APSH_EXTRA_H
#define LIB_APSH_EXTRA_H

#ifdef APSH_EXTRA_GLIB
#include <glib.h>
#endif

#ifdef APSH_EXTRA_JSON
#include <json-c/json.h>
#endif

#ifdef __cplusplus
extern "C" {
#define NOEXCEPT noexcept
#else
#define NOEXCEPT
#endif

#define st_stat() status_t

#ifdef APSH_EXTRA_GLIB

GSList* apsh_get_va_pages(
    apsh_instance_t apsh,
    addr_t pt) NOEXCEPT;

GSList* apsh_get_nested_va_pages(
    apsh_instance_t apsh,
    addr_t npt,
    page_mode_t npm,
    addr_t pt,
    page_mode_t pm) NOEXCEPT;

#endif

#ifdef APSH_EXTRA_JSON

os_t apsh_init_profile(
    apsh_instance_t apsh,
    apsh_config_t config_mode,
    void *config,
    addr_t EPT) NOEXCEPT;

json_object* apsh_get_kernel_json(
    apsh_instance_t apsh) NOEXCEPT;


st_stat() apsh_get_symbol_addr_from_json(
    apsh_instance_t apsh,
    json_object* json,
    const char* symbol,
    addr_t* addr) NOEXCEPT;


st_stat() apsh_get_struct_size_from_json(
    apsh_instance_t apsh,
    json_object* json,
    const char* struct_name,
    size_t* size) NOEXCEPT;

st_stat() apsh_get_struct_member_offset_from_json(
    apsh_instance_t apsh,
    json_object* json,
    const char* struct_name,
    const char* struct_member,
    addr_t* offset) NOEXCEPT;

st_stat() apsh_get_bitfield_offset_and_size_from_json(apsh_instance_t apsh, json_object *json,
        const char *struct_name,
        const char *struct_member,
        addr_t *offset, size_t *start_bit,
        size_t *end_bit);

st_stat() apsh_get_symbol_addr_from_apsh_json(
    apsh_instance_t apsh,
    const char* symbol,
    addr_t* addr) NOEXCEPT;

st_stat() apsh_get_struct_size_from_apsh_json(
    apsh_instance_t apsh,
    const char* struct_name,
    size_t* size) NOEXCEPT;

st_stat() apsh_get_struct_member_offset_from_apsh_json(
    apsh_instance_t apsh,
    const char* struct_name,
    const char* struct_member,
    addr_t* offset) NOEXCEPT;

st_stat() apsh_get_bitfield_offset_and_size_from_apsh_json(apsh_instance_t apsh,
        const char *struct_name,
        const char *struct_member,
        addr_t *offset, size_t *start_bit,
        size_t *end_bit);
#endif

#ifdef __cplusplus
}
#endif

#endif
