/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef LIBAPSH_PAGE_H
#define LIBAPSH_PAGE_H

#include "apsh_const.h"
#include "apsh_x86_regs.h"

#ifdef __cplusplus
extern "C" {
#define NOEXCEPT noexcept
#else
#define NOEXCEPT
#endif

typedef struct arm_registers {
    uint64_t ttbr0;
    uint64_t ttbr1;
    uint64_t ttbcr;
    uint64_t pc;
    uint32_t cpsr;
    uint32_t _pad;
} arm_registers_t;

typedef struct registers {
    union {
        x86_registers_t x86;
        arm_registers_t arm;
    };
} registers_t;

typedef struct xsave_area {
    char fpu_sse[512];
    uint64_t xstate_bv;         /* Updated by XRSTOR */
    uint64_t xcomp_bv;          /* Updated by XRSTOR{C,S} */
} xsave_area_t;

typedef uint64_t addr_t;

typedef int32_t apsh_pid_t;

typedef struct page_info {
    addr_t APSH_FUNC_DEFINE(vaddr);       
    addr_t APSH_FUNC_DEFINE(paddr);       
    addr_t APSH_FUNC_DEFINE(naddr);       

    page_size_t size;   
    page_size_t nsize;  

    addr_t APSH_FUNC_DEFINE(pt);          
    page_mode_t pm;     

    addr_t APSH_FUNC_DEFINE(npt);         
    page_mode_t npm;    

    union {
        struct {
            addr_t APSH_FUNC_DEFINE(pte_location);
            addr_t APSH_FUNC_DEFINE(pte_value);
            addr_t APSH_FUNC_DEFINE(pgd_location);
            addr_t APSH_FUNC_DEFINE(pgd_value);
        } x86_legacy;

        struct {
            addr_t APSH_FUNC_DEFINE(pte_location);
            addr_t APSH_FUNC_DEFINE(pte_value);
            addr_t APSH_FUNC_DEFINE(pgd_location);
            addr_t APSH_FUNC_DEFINE(pgd_value);
            addr_t APSH_FUNC_DEFINE(pdpe_location);
            addr_t APSH_FUNC_DEFINE(pdpe_value);
        } x86_pae;

        struct {
            addr_t APSH_FUNC_DEFINE(pte_location);
            addr_t APSH_FUNC_DEFINE(pte_value);
            addr_t APSH_FUNC_DEFINE(pgd_location);
            addr_t APSH_FUNC_DEFINE(pgd_value);
            addr_t APSH_FUNC_DEFINE(pdpte_location);
            addr_t APSH_FUNC_DEFINE(pdpte_value);
            addr_t APSH_FUNC_DEFINE(pml4e_location);
            addr_t APSH_FUNC_DEFINE(pml4e_value);
        } x86_ia32e;

        struct {
            uint32_t fld_location;
            uint32_t fld_value;
            uint32_t sld_location;
            uint32_t sld_value;
        } arm_aarch32;

        struct {
            uint64_t zld_location;
            uint64_t zld_value;
            uint64_t fld_location;
            uint64_t fld_value;
            uint64_t sld_location;
            uint64_t sld_value;
            uint64_t tld_location;
            uint64_t tld_value;
        } arm_aarch64;
    };
} page_info_t;


#ifdef __cplusplus
}
#endif

#endif /* LIBAPSH_H */
