/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <stdlib.h>

#include "arch/amd64.h"
#include "arch/arch_interface.h"
#include "arch/arm_aarch32.h"
#include "arch/arm_aarch64.h"
#include "arch/ept.h"
#include "arch/intel.h"
#include "driver/driver_wrapper.h"
#include "apsh_instance.h"

void
arch_init_lookup_tables(apsh_instance_t apsh)
{
	apsh->arch_interface.lookup[APSH_PM_LEGACY] = v2p_nopae;
	apsh->arch_interface.lookup[APSH_PM_PAE] = v2p_pae;
	apsh->arch_interface.lookup[APSH_PM_IA32E] = v2p_ia32e;
	apsh->arch_interface.lookup[APSH_PM_AARCH32] = v2p_aarch32;
	apsh->arch_interface.lookup[APSH_PM_AARCH64] = v2p_aarch64;
	apsh->arch_interface.lookup[APSH_PM_EPT_4L] = v2p_ept_4l;

	apsh->arch_interface.get_pages[APSH_PM_LEGACY] = get_pages_nopae;
	apsh->arch_interface.get_pages[APSH_PM_PAE] = get_pages_pae;
	apsh->arch_interface.get_pages[APSH_PM_IA32E] = get_pages_ia32e;
	apsh->arch_interface.get_pages[APSH_PM_EPT_4L] = get_pages_ept_4l;
}

status_t
arch_init(apsh_instance_t apsh)
{
	if (apsh->page_mode != APSH_PM_UNKNOWN)
		return APSH_S;

	return APSH_S;
}
