/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef ARCH_INTERFACE_H_
#define ARCH_INTERFACE_H_

#include "apsh_instance.h"

typedef status_t (*arch_lookup_t)
(apsh_instance_t apsh,
 addr_t npt,
 page_mode_t npm,
 addr_t pt,
 addr_t addr,
 page_info_t *info);
typedef GSList* (*arch_get_pages_t)
(apsh_instance_t apsh,
 addr_t npt,
 page_mode_t npm,
 addr_t dtb);

typedef struct arch_interface {
    arch_lookup_t lookup[APSH_PM_EPT_5L + 1];
    arch_get_pages_t get_pages[APSH_PM_EPT_5L + 1];
} arch_interface_t;

status_t arch_init(apsh_instance_t apsh);

static inline bool valid_npm(page_mode_t npm)
{
    return npm == APSH_PM_EPT_4L;
}

static inline bool valid_pm(page_mode_t pm)
{
    return pm >= APSH_PM_LEGACY && pm < APSH_PM_EPT_5L;
}

static inline
page_mode_t get_page_mode_x86(reg_t cr0, reg_t cr4, reg_t efer)
{
    if (!APSH_GET_BIT(cr0, 31))
        return APSH_PM_NONE;

    if (!APSH_GET_BIT(cr4, 5))
        return APSH_PM_LEGACY;

    if (!APSH_GET_BIT(efer, 8))
        return APSH_PM_PAE;

    return APSH_PM_IA32E;
}

void arch_init_lookup_tables(apsh_instance_t apsh);

#endif /* ARCH_INTERFACE_H_ */
