/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef CACHE_H
#define CACHE_H

struct key_128;
typedef struct key_128 *key_128_t;

#define NOOP

gboolean key_128_equals(gconstpointer key1, gconstpointer key2);

#ifdef ENABLE_ADDRESS_CACHE

void sym_cache_set(apsh_instance_t apsh, addr_t base_addr, apsh_pid_t pid, const char *sym, addr_t va);
void sym_cache_flush(apsh_instance_t apsh);
void sym_cache_init(apsh_instance_t apsh);
void sym_cache_destroy(apsh_instance_t apsh);
status_t sym_cache_get(apsh_instance_t apsh, addr_t base_addr, apsh_pid_t pid, const char *sym, addr_t *va);
status_t sym_cache_del(apsh_instance_t apsh, addr_t base_addr, apsh_pid_t pid, char *sym);

void v2p_cache_set(apsh_instance_t apsh, addr_t va, addr_t pt, addr_t npt, addr_t pa);
void v2p_cache_flush(apsh_instance_t apsh, addr_t pt, addr_t npt);
void v2p_cache_init(apsh_instance_t apsh);
void v2p_cache_destroy(apsh_instance_t apsh);
status_t v2p_cache_get(apsh_instance_t apsh, addr_t va, addr_t pt, addr_t npt, addr_t *pa);
status_t v2p_cache_del(apsh_instance_t apsh, addr_t va, addr_t np, addr_t npt);


void pid_cache_set(apsh_instance_t apsh, apsh_pid_t pid, addr_t dtb);
void pid_cache_flush(apsh_instance_t apsh);
void pid_cache_init(apsh_instance_t apsh);
void pid_cache_destroy(apsh_instance_t apsh);
status_t pid_cache_get(apsh_instance_t apsh, apsh_pid_t pid, addr_t *dtb);
status_t pid_cache_del(apsh_instance_t apsh, apsh_pid_t pid);

void rva_cache_set(apsh_instance_t apsh, addr_t base_addr, addr_t dtb, addr_t rva, char *sym);
void rva_cache_flush(apsh_instance_t apsh);
void rva_cache_init(apsh_instance_t apsh);
void rva_cache_destroy(apsh_instance_t apsh);
status_t rva_cache_get(apsh_instance_t apsh, addr_t base_addr, addr_t dtb, addr_t rva, char **sym);
status_t rva_cache_del(apsh_instance_t apsh, addr_t base_addr, addr_t dtb, addr_t rva);

#else
#define v2p_cache_init(...)     NOOP
#define v2p_cache_destroy(...)  NOOP
#define v2p_cache_set(...)      NOOP
#define v2p_cache_flush(...)    NOOP
#define v2p_cache_get(...) APSH_F
#define v2p_cache_del(...) APSH_F

#define pid_cache_init(...)     NOOP
#define pid_cache_destroy(...)  NOOP
#define pid_cache_set(...)      NOOP
#define pid_cache_flush(...)    NOOP
#define pid_cache_get(...) APSH_F
#define pid_cache_del(...) APSH_F

#define rva_cache_init(...)     NOOP
#define rva_cache_destroy(...)  NOOP
#define rva_cache_set(...)      NOOP
#define rva_cache_flush(...)    NOOP
#define rva_cache_get(...) APSH_F
#define rva_cache_del(...) APSH_F

#define sym_cache_init(...)     NOOP
#define sym_cache_destroy(...)  NOOP
#define sym_cache_set(...)      NOOP
#define sym_cache_flush(...)    NOOP
#define sym_cache_get(...) APSH_F
#define sym_cache_del(...) APSH_F

#endif

struct key_128 {
    uint64_t low;
    uint64_t high;
};
uint64_t hash128to64(uint64_t low, uint64_t high);
guint key_128_hash(gconstpointer key);

#endif
