/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <stdlib.h>
#include <string.h>

#include "apsh_instance.h"
#include "driver/driver_interface.h"

#ifdef ENABLE_FILE
#include "driver/handle_files/file.h"
#endif

status_t driver_init_mode(const char *name,
						  uint64_t domainid,
						  uint64_t init_flags,
						  apsh_init_data_t *init_data,
						  apsh_mode_t *mode)
{
	unsigned long count = 0;

	if (file_test(domainid, name, init_flags, init_data) == APSH_S) {
		debug_print(APSH_DEBUG_DRIVER, "--found file\n");
		*mode = APSH_FILE;
		count++;
	}

	/* if we didn't see exactly one system, report error */
	if (count == 0) {
		print_error("Could not find a live guest VM or file to use.\n");
		print_error("Opening a live guest VM requires root access.\n");
		return APSH_F;
	} else if (count > 1) {
		print_error
		("Found more than one VMM or file to use,\nplease specify what you want instead of using APSH_AUTO.\n");
		return APSH_F;
	} else { /* count == 1 */
		return APSH_S;
	}
}

status_t driver_init(apsh_instance_t apsh,
					 uint32_t init_flags,
					 apsh_init_data_t *init_data)
{
	status_t rc = APSH_F;

	if (apsh->driver.initialized) {
		print_error("Driver is already initialized.\n");
		return rc;
	}

	memset(&apsh->driver, 0, sizeof(driver_interface_t));

	switch (apsh->mode) {
	case APSH_FILE:
		rc = driver_file_setup(apsh);
		break;
	default:
		break;
	};

	if (rc == APSH_S && apsh->driver.init_ptr)
		rc = apsh->driver.init_ptr(apsh, init_flags, init_data);

	return rc;
}

status_t driver_init_apsh(apsh_instance_t apsh,
						 uint32_t init_flags,
						 apsh_init_data_t *init_data)
{
	status_t rc = APSH_F;

	if (apsh->driver.init_apsh_ptr)
		rc = apsh->driver.init_apsh_ptr(apsh, init_flags, init_data);

	return rc;
}
