/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DRIVER_INTERFACE_H
#define DRIVER_INTERFACE_H

#include "apsh_instance.h"

typedef struct driver_interface {
	status_t (*init_ptr) (
		apsh_instance_t,
		uint32_t init_flags,
		apsh_init_data_t *init_data);
	status_t (*init_apsh_ptr) (
		apsh_instance_t,
		uint32_t init_flags,
		apsh_init_data_t *init_data);
	void (*destroy_ptr) (
		apsh_instance_t);
	status_t (*get_name_ptr) (
		apsh_instance_t,
		char **);
	void (*set_name_ptr) (
		apsh_instance_t,
		const char *);
	status_t (*get_memsize_ptr) (
		apsh_instance_t,
		uint64_t *,
		addr_t *);
	void *(*read_page_ptr) (
		apsh_instance_t,
		addr_t);
	void *(*read_memory_ptr) (
		apsh_instance_t,
		addr_t,
		uint32_t);
	/* Driver-specific data storage. */
	void* driver_data;

	/* Set to true once driver is initialized. */
	bool initialized;

} driver_interface_t;

status_t driver_init_mode(
	const char *name,
	uint64_t domainid,
	uint64_t init_flags,
	apsh_init_data_t *init_data,
	apsh_mode_t *mode);

status_t driver_init(
	apsh_instance_t apsh,
	uint32_t init_flags,
	apsh_init_data_t *init_data);

status_t driver_init_apsh(
	apsh_instance_t apsh,
	uint32_t init_flags,
	apsh_init_data_t *init_data);

#endif /* DRIVER_INTERFACE_H */

