/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef FILE_DRIVER_H
#define FILE_DRIVER_H
status_t file_init(apsh_instance_t apsh,uint32_t init_flags,apsh_init_data_t *init_data);
status_t file_init_apsh(apsh_instance_t apsh,uint32_t init_flags,apsh_init_data_t *init_data);
void file_destroy(apsh_instance_t apsh);
void *file_get_memory(apsh_instance_t apsh,addr_t paddr,uint32_t length);
status_t file_get_name();
void file_set_name();
status_t file_get_memsize(apsh_instance_t apsh,uint64_t *allocated_ram_size,addr_t *maximum_physical_address);
void *file_read_page(apsh_instance_t apsh,addr_t page);
status_t file_write(apsh_instance_t apsh,addr_t paddr,void *buf, uint32_t length);
status_t file_test();
static inline status_t driver_file_setup(apsh_instance_t apsh){
    driver_interface_t driver = {0};
    driver.initialized = true;
    driver.init_ptr = &file_init;
    driver.init_apsh_ptr = &file_init_apsh;
    driver.destroy_ptr = &file_destroy;
    driver.get_name_ptr = &file_get_name;
    driver.set_name_ptr = &file_set_name;
    driver.get_memsize_ptr = &file_get_memsize;
    driver.read_page_ptr = &file_read_page;
    driver.read_memory_ptr = &file_get_memory;
    apsh->driver = driver;
    return APSH_S;
}
#endif
