/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef OS_LINUX_H_
#define OS_LINUX_H_

#include "apsh_instance.h"

struct linux_instance {
    char *sysmap; /**< system map file for domain's running kernel */

    addr_t tasks_offset; /**< task_struct->tasks */

    addr_t mm_offset; /**< task_struct->mm */

    addr_t pid_offset; /**< task_struct->pid */

    addr_t pgd_offset; /**< mm_struct->pgd */

    addr_t name_offset; /**< task_struct->comm */

    addr_t kaslr_offset; /**< offset generated at boot time for KASLR */

    addr_t init_task_fixed; /**< Rekall's location for init task, ignoring KASLR */

    addr_t swapper_pg_dir; /**< swapper_pg_dir for file mode initR */

    addr_t kernel_pgt; /**< kernel_pgt for file mode init */

    addr_t boundary; /**< boundary for file mode init */

    addr_t phys_start; /**< phys_start for file mode init */

    addr_t virt_start; /**< virt_start for file mode init */

};
typedef struct linux_instance *linux_instance_t;

status_t refind_kpgd(apsh_instance_t instance);

status_t linux_init(apsh_instance_t instance, GHashTable *config);

status_t linux_get_offset(apsh_instance_t apsh, const char* offset_name, addr_t *offset);

status_t linux_get_kernel_struct_offset(apsh_instance_t apsh,
                                        const char* symbol, const char* member, addr_t *addr);

status_t linux_symbol_to_address(apsh_instance_t instance,
                                 const char *symbol, addr_t *__unused, addr_t *address);

char* linux_system_map_address_to_symbol(apsh_instance_t apsh,
        addr_t address, const access_context_t *ctx);

status_t linux_pid_to_pgd(apsh_instance_t apsh, apsh_pid_t pid, addr_t *pgd);

status_t linux_pgd_to_pid(apsh_instance_t apsh, addr_t pgd, apsh_pid_t *pid);

status_t linux_teardown(apsh_instance_t apsh);

#endif /* OS_LINUX_H_ */
