/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef OS_INTERFACE_H_
#define OS_INTERFACE_H_

#include "apsh_instance.h"
#include "os/windows/windows.h"
#include "os/linux/linux.h"

#define st_stat() status_t
#define acc access_context_t
#define add_system(x) os_##x

typedef st_stat() (*add_system(get_offset_t))(apsh_instance_t apsh,
                                    const char* offset_name, addr_t *offset);

typedef st_stat() (*add_system(get_kernel_struct_offset_t))(apsh_instance_t apsh,
        const char* symbol, const char* member, addr_t *addr);

typedef st_stat() (*add_system(pgd_to_pid_t))(apsh_instance_t apsh, addr_t pgd, apsh_pid_t *pid);

typedef st_stat() (*add_system(pid_to_pgd_t))(apsh_instance_t apsh, apsh_pid_t pid, addr_t *dtb);

typedef st_stat() (*add_system(kernel_symbol_to_address_t))(apsh_instance_t instance,
        const char *symbol, addr_t *kernel_base_vaddr, addr_t *address);

typedef st_stat() (*add_system(user_symbol_to_rva_t))(apsh_instance_t instance,
        const acc *ctx, const char *symbol, addr_t *rva);

typedef char* (*add_system(address_to_symbol_t))(apsh_instance_t apsh, addr_t address,
                                        const acc *ctx);

typedef char* (*add_system(address_to_symbol_kaslr_t))(apsh_instance_t apsh, addr_t address,
        const acc *ctx);

typedef unicode_string_t* (*add_system(read_unicode_struct_t))(apsh_instance_t apsh,
        const acc *ctx);

typedef unicode_string_t* (*add_system(read_unicode_struct_pm_t))(apsh_instance_t apsh,
        const acc *ctx, page_mode_t page_mode);

typedef st_stat() (*add_system(teardown_t))(apsh_instance_t apsh);

typedef struct add_system(interface) {
    add_system(get_kernel_struct_offset_t) os_get_kernel_struct_offset;
    add_system(get_offset_t) os_get_offset;
    add_system(pgd_to_pid_t) os_pgd_to_pid;
    add_system(pid_to_pgd_t) os_pid_to_pgd;
    add_system(kernel_symbol_to_address_t) os_ksym2v;
    add_system(user_symbol_to_rva_t) os_usym2rva;
    add_system(address_to_symbol_t) os_v2sym;
    add_system(address_to_symbol_kaslr_t) os_v2ksym;
    add_system(read_unicode_struct_t) os_read_unicode_struct;
    add_system(read_unicode_struct_pm_t) os_read_unicode_struct_pm;
    add_system(teardown_t) os_teardown;
} *os_interface_t;

st_stat() os_destroy(apsh_instance_t apsh);

#endif /* OS_INTERFACE_H_ */
