/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <string.h>
#include <stddef.h>

#include "private/common_defines.h"
#include "driver/driver_wrapper.h"
#include "peparse.h"
#include "apsh_instance.h"

#define APSH_FUNC_DEFINE(x) x
#define copy_string_func strncmp
#define sym symbol
#define ULONG unsigned long

struct _DBGKD_DEBUG_DATA_HEADER64 {
	uint64_t APSH_FUNC_DEFINE(List[2]);
	uint32_t APSH_FUNC_DEFINE(OwnerTag);
	uint32_t APSH_FUNC_DEFINE(Size);
} __attribute__((packed));
typedef struct _DBGKD_DEBUG_DATA_HEADER64 DBGKD_DEBUG_DATA_HEADER64;

struct _KDDEBUGGER_DATA64 {
	DBGKD_DEBUG_DATA_HEADER64 Header;
	uint64_t APSH_FUNC_DEFINE(KernBase);
	uint64_t APSH_FUNC_DEFINE(BreakpointWithStatus);
	uint64_t APSH_FUNC_DEFINE(SavedContext);
	uint16_t APSH_FUNC_DEFINE(ThCallbackStack);
	uint16_t APSH_FUNC_DEFINE(NextCallback);
	uint16_t APSH_FUNC_DEFINE(FramePointer);
	uint16_t APSH_FUNC_DEFINE(PaeEnabled);
	uint64_t APSH_FUNC_DEFINE(KiCallUserMode);
	uint64_t APSH_FUNC_DEFINE(KeUserCallbackDispatcher);
	uint64_t APSH_FUNC_DEFINE(PsLoadedModuleList);
	uint64_t APSH_FUNC_DEFINE(PsActiveProcessHead);
	uint64_t APSH_FUNC_DEFINE(PspCidTable);
	uint64_t APSH_FUNC_DEFINE(ExpSystemResourcesList);
	uint64_t APSH_FUNC_DEFINE(ExpPagedPoolDescriptor);
	uint64_t APSH_FUNC_DEFINE(ExpNumberOfPagedPools);
	uint64_t APSH_FUNC_DEFINE(KeTimeIncrement);
	uint64_t APSH_FUNC_DEFINE(KeBugCheckCallbackListHead);
	uint64_t APSH_FUNC_DEFINE(KiBugcheckData);
	uint64_t APSH_FUNC_DEFINE(IopErrorLogListHead);
	uint64_t APSH_FUNC_DEFINE(ObpRootDirectoryObject);
	uint64_t APSH_FUNC_DEFINE(ObpTypeObjectType);
	uint64_t APSH_FUNC_DEFINE(MmSystemCacheStart);
	uint64_t APSH_FUNC_DEFINE(MmSystemCacheEnd);
	uint64_t APSH_FUNC_DEFINE(MmSystemCacheWs);
	uint64_t APSH_FUNC_DEFINE(MmPfnDatabase);
	uint64_t APSH_FUNC_DEFINE(MmSystemPtesStart);
	uint64_t APSH_FUNC_DEFINE(MmSystemPtesEnd);
	uint64_t APSH_FUNC_DEFINE(MmSubsectionBase);
	uint64_t APSH_FUNC_DEFINE(MmNumberOfPagingFiles);
	uint64_t APSH_FUNC_DEFINE(MmLowestPhysicalPage);
	uint64_t APSH_FUNC_DEFINE(MmHighestPhysicalPage);
	uint64_t APSH_FUNC_DEFINE(MmNumberOfPhysicalPages);
	uint64_t APSH_FUNC_DEFINE(MmMaximumNonPagedPoolInBytes);
	uint64_t APSH_FUNC_DEFINE(MmNonPagedSystemStart);
	uint64_t APSH_FUNC_DEFINE(MmNonPagedPoolStart);
	uint64_t APSH_FUNC_DEFINE(MmNonPagedPoolEnd);
	uint64_t APSH_FUNC_DEFINE(MmPagedPoolStart);
	uint64_t APSH_FUNC_DEFINE(MmPagedPoolEnd);
	uint64_t APSH_FUNC_DEFINE(MmPagedPoolInformation);
	uint64_t APSH_FUNC_DEFINE(MmPageSize);
	uint64_t APSH_FUNC_DEFINE(MmSizeOfPagedPoolInBytes);
	uint64_t APSH_FUNC_DEFINE(MmTotalCommitLimit);
	uint64_t APSH_FUNC_DEFINE(MmTotalCommittedPages);
	uint64_t APSH_FUNC_DEFINE(MmSharedCommit);
	uint64_t APSH_FUNC_DEFINE(MmDriverCommit);
	uint64_t APSH_FUNC_DEFINE(MmProcessCommit);
	uint64_t APSH_FUNC_DEFINE(MmPagedPoolCommit);
	uint64_t APSH_FUNC_DEFINE(MmExtendedCommit);
	uint64_t APSH_FUNC_DEFINE(MmZeroedPageListHead);
	uint64_t APSH_FUNC_DEFINE(MmFreePageListHead);
	uint64_t APSH_FUNC_DEFINE(MmStandbyPageListHead);
	uint64_t APSH_FUNC_DEFINE(MmModifiedPageListHead);
	uint64_t APSH_FUNC_DEFINE(MmModifiedNoWritePageListHead);
	uint64_t APSH_FUNC_DEFINE(MmAvailablePages);
	uint64_t APSH_FUNC_DEFINE(MmResidentAvailablePages);
	uint64_t APSH_FUNC_DEFINE(PoolTrackTable);
	uint64_t APSH_FUNC_DEFINE(NonPagedPoolDescriptor);
	uint64_t APSH_FUNC_DEFINE(MmHighestUserAddress);
	uint64_t APSH_FUNC_DEFINE(MmSystemRangeStart);
	uint64_t APSH_FUNC_DEFINE(MmUserProbeAddress);
	uint64_t APSH_FUNC_DEFINE(KdPrintCircularBuffer);
	uint64_t APSH_FUNC_DEFINE(KdPrintCircularBufferEnd);
	uint64_t APSH_FUNC_DEFINE(KdPrintWritePointer);
	uint64_t APSH_FUNC_DEFINE(KdPrintRolloverCount);
	uint64_t APSH_FUNC_DEFINE(MmLoadedUserImageList);
	uint64_t APSH_FUNC_DEFINE(NtBuildLab);
	uint64_t APSH_FUNC_DEFINE(KiNormalSystemCall);
	uint64_t APSH_FUNC_DEFINE(KiProcessorBlock);
	uint64_t APSH_FUNC_DEFINE(MmUnloadedDrivers);
	uint64_t APSH_FUNC_DEFINE(MmLastUnloadedDriver);
	uint64_t APSH_FUNC_DEFINE(MmTriageActionTaken);
	uint64_t APSH_FUNC_DEFINE(MmSpecialPoolTag);
	uint64_t APSH_FUNC_DEFINE(KernelVerifier);
	uint64_t APSH_FUNC_DEFINE(MmVerifierData);
	uint64_t APSH_FUNC_DEFINE(MmAllocatedNonPagedPool);
	uint64_t APSH_FUNC_DEFINE(MmPeakCommitment);
	uint64_t APSH_FUNC_DEFINE(MmTotalCommitLimitMaximum);
	uint64_t APSH_FUNC_DEFINE(CmNtCSDVersion);
	uint64_t APSH_FUNC_DEFINE(MmPhysicalMemoryBlock);
	uint64_t APSH_FUNC_DEFINE(MmSessionBase);
	uint64_t APSH_FUNC_DEFINE(MmSessionSize);
	uint64_t APSH_FUNC_DEFINE(MmSystemParentTablePage);
	uint64_t APSH_FUNC_DEFINE(MmVirtualTranslationBase);
	uint16_t APSH_FUNC_DEFINE(OffsetKThreadNextProcessor);
	uint16_t APSH_FUNC_DEFINE(OffsetKThreadTeb);
	uint16_t APSH_FUNC_DEFINE(OffsetKThreadKernelStack);
	uint16_t APSH_FUNC_DEFINE(OffsetKThreadInitialStack);
	uint16_t APSH_FUNC_DEFINE(OffsetKThreadApcProcess);
	uint16_t APSH_FUNC_DEFINE(OffsetKThreadState);
	uint16_t APSH_FUNC_DEFINE(OffsetKThreadBStore);
	uint16_t APSH_FUNC_DEFINE(OffsetKThreadBStoreLimit);
	uint16_t APSH_FUNC_DEFINE(SizeEProcess);
	uint16_t APSH_FUNC_DEFINE(OffsetEprocessPeb);
	uint16_t APSH_FUNC_DEFINE(OffsetEprocessParentCID);
	uint16_t APSH_FUNC_DEFINE(OffsetEprocessDirectoryTableBase);
	uint16_t APSH_FUNC_DEFINE(SizePrcb);
	uint16_t APSH_FUNC_DEFINE(OffsetPrcbDpcRoutine);
	uint16_t APSH_FUNC_DEFINE(OffsetPrcbCurrentThread);
	uint16_t APSH_FUNC_DEFINE(OffsetPrcbMhz);
	uint16_t APSH_FUNC_DEFINE(OffsetPrcbCpuType);
	uint16_t APSH_FUNC_DEFINE(OffsetPrcbVendorString);
	uint16_t APSH_FUNC_DEFINE(OffsetPrcbProcStateContext);
	uint16_t APSH_FUNC_DEFINE(OffsetPrcbNumber);
	uint16_t APSH_FUNC_DEFINE(SizeEThread);
	uint64_t APSH_FUNC_DEFINE(KdPrintCircularBufferPtr);
	uint64_t APSH_FUNC_DEFINE(KdPrintBufferSize);
	uint64_t APSH_FUNC_DEFINE(KeLoaderBlock);
	uint16_t APSH_FUNC_DEFINE(SizePcr);
	uint16_t APSH_FUNC_DEFINE(OffsetPcrSelfPcr);
	uint16_t APSH_FUNC_DEFINE(OffsetPcrCurrentPrcb);
	uint16_t APSH_FUNC_DEFINE(OffsetPcrContainedPrcb);
	uint16_t APSH_FUNC_DEFINE(OffsetPcrInitialBStore);
	uint16_t APSH_FUNC_DEFINE(OffsetPcrBStoreLimit);
	uint16_t APSH_FUNC_DEFINE(OffsetPcrInitialStack);
	uint16_t APSH_FUNC_DEFINE(OffsetPcrStackLimit);
	uint16_t APSH_FUNC_DEFINE(OffsetPrcbPcrPage);
	uint16_t APSH_FUNC_DEFINE(OffsetPrcbProcStateSpecialReg);
	uint16_t APSH_FUNC_DEFINE(GdtR0Code);
	uint16_t APSH_FUNC_DEFINE(GdtR0Data);
	uint16_t APSH_FUNC_DEFINE(GdtR0Pcr);
	uint16_t APSH_FUNC_DEFINE(GdtR3Code);
	uint16_t APSH_FUNC_DEFINE(GdtR3Data);
	uint16_t APSH_FUNC_DEFINE(GdtR3Teb);
	uint16_t APSH_FUNC_DEFINE(GdtLdt);
	uint16_t APSH_FUNC_DEFINE(GdtTss);
	uint16_t APSH_FUNC_DEFINE(Gdt64R3CmCode);
	uint16_t APSH_FUNC_DEFINE(Gdt64R3CmTeb);
	uint64_t APSH_FUNC_DEFINE(IopNumTriageDumpDataBlocks);
	uint64_t APSH_FUNC_DEFINE(IopTriageDumpDataBlocks);
	uint64_t APSH_FUNC_DEFINE(VfCrashDataBlock);
} __attribute__((packed));
typedef struct _KDDEBUGGER_DATA64 KDDEBUGGER_DATA64;

enum kdbg_magic {
	KDBG_WINDOWS_2000 = 0x0208U,
	KDBG_WINDOWS_XP = 0x0290U,
	KDBG_WINDOWS_2003 = 0x0318U,
	KDBG_WINDOWS_VISTA = 0x0328U,
	KDBG_WINDOWS_2008 = 0x0330U,
	KDBG_WINDOWS_7 = 0x0340U,
	KDBG_WINDOWS_8 = 0x0360U,
};

static status_t
kdbg_symbol_resolve(apsh_instance_t apsh, ULONG offset, addr_t *address)
{
	uint64_t APSH_FUNC_DEFINE(tmp = 0);
	addr_t symaddr = 0;
	windows_instance_t windows = NULL;

	if (apsh->os_data == NULL)
		return APSH_F;

	windows = apsh->os_data;
	symaddr = windows->kdbg_va + offset;

	if (apsh_read_64_va(apsh, symaddr, 0, &tmp) == APSH_F)
		return APSH_F;

	*address = tmp;
	return APSH_S;
}

struct symbol_offset_entry {
	const char *str;
	ULONG offset;
};

#define SYMBOL_OFFSET_ENTRY(name) { #name, offsetof(KDDEBUGGER_DATA64, name), }

static status_t
kdbg_symbol_offset(const char *sym, ULONG *kdbg_sym_off)
{
	static const struct symbol_offset_entry symbol_offset_arr[] = {
		SYMBOL_OFFSET_ENTRY(KernBase),
		SYMBOL_OFFSET_ENTRY(BreakpointWithStatus),
		SYMBOL_OFFSET_ENTRY(SavedContext),
		SYMBOL_OFFSET_ENTRY(KiCallUserMode),
		SYMBOL_OFFSET_ENTRY(KeUserCallbackDispatcher),
		SYMBOL_OFFSET_ENTRY(PsLoadedModuleList),
		SYMBOL_OFFSET_ENTRY(PsActiveProcessHead),
		SYMBOL_OFFSET_ENTRY(PspCidTable),
		SYMBOL_OFFSET_ENTRY(ExpSystemResourcesList),
		SYMBOL_OFFSET_ENTRY(ExpPagedPoolDescriptor),
		SYMBOL_OFFSET_ENTRY(ExpNumberOfPagedPools),
		SYMBOL_OFFSET_ENTRY(KeTimeIncrement),
		SYMBOL_OFFSET_ENTRY(KeBugCheckCallbackListHead),
		SYMBOL_OFFSET_ENTRY(KiBugcheckData),
		SYMBOL_OFFSET_ENTRY(IopErrorLogListHead),
		SYMBOL_OFFSET_ENTRY(ObpRootDirectoryObject),
		SYMBOL_OFFSET_ENTRY(ObpTypeObjectType),
		SYMBOL_OFFSET_ENTRY(MmSystemCacheStart),
		SYMBOL_OFFSET_ENTRY(MmSystemCacheEnd),
		SYMBOL_OFFSET_ENTRY(MmSystemCacheWs),
		SYMBOL_OFFSET_ENTRY(MmPfnDatabase),
		SYMBOL_OFFSET_ENTRY(MmSystemPtesStart),
		SYMBOL_OFFSET_ENTRY(MmSystemPtesEnd),
		SYMBOL_OFFSET_ENTRY(MmSubsectionBase),
		SYMBOL_OFFSET_ENTRY(MmNumberOfPagingFiles),
		SYMBOL_OFFSET_ENTRY(MmLowestPhysicalPage),
		SYMBOL_OFFSET_ENTRY(MmHighestPhysicalPage),
		SYMBOL_OFFSET_ENTRY(MmNumberOfPhysicalPages),
		SYMBOL_OFFSET_ENTRY(MmMaximumNonPagedPoolInBytes),
		SYMBOL_OFFSET_ENTRY(MmNonPagedSystemStart),
		SYMBOL_OFFSET_ENTRY(MmNonPagedPoolStart),
		SYMBOL_OFFSET_ENTRY(MmNonPagedPoolEnd),
		SYMBOL_OFFSET_ENTRY(MmPagedPoolStart),
		SYMBOL_OFFSET_ENTRY(MmPagedPoolEnd),
		SYMBOL_OFFSET_ENTRY(MmPagedPoolInformation),
		SYMBOL_OFFSET_ENTRY(MmPageSize),
		SYMBOL_OFFSET_ENTRY(MmSizeOfPagedPoolInBytes),
		SYMBOL_OFFSET_ENTRY(MmTotalCommitLimit),
		SYMBOL_OFFSET_ENTRY(MmTotalCommittedPages),
		SYMBOL_OFFSET_ENTRY(MmSharedCommit),
		SYMBOL_OFFSET_ENTRY(MmDriverCommit),
		SYMBOL_OFFSET_ENTRY(MmProcessCommit),
		SYMBOL_OFFSET_ENTRY(MmPagedPoolCommit),
		SYMBOL_OFFSET_ENTRY(MmExtendedCommit),
		SYMBOL_OFFSET_ENTRY(MmZeroedPageListHead),
		SYMBOL_OFFSET_ENTRY(MmFreePageListHead),
		SYMBOL_OFFSET_ENTRY(MmStandbyPageListHead),
		SYMBOL_OFFSET_ENTRY(MmModifiedPageListHead),
		SYMBOL_OFFSET_ENTRY(MmModifiedNoWritePageListHead),
		SYMBOL_OFFSET_ENTRY(MmAvailablePages),
		SYMBOL_OFFSET_ENTRY(MmResidentAvailablePages),
		SYMBOL_OFFSET_ENTRY(PoolTrackTable),
		SYMBOL_OFFSET_ENTRY(NonPagedPoolDescriptor),
		SYMBOL_OFFSET_ENTRY(MmHighestUserAddress),
		SYMBOL_OFFSET_ENTRY(MmSystemRangeStart),
		SYMBOL_OFFSET_ENTRY(MmUserProbeAddress),
		SYMBOL_OFFSET_ENTRY(KdPrintCircularBuffer),
		SYMBOL_OFFSET_ENTRY(KdPrintCircularBufferEnd),
		SYMBOL_OFFSET_ENTRY(KdPrintWritePointer),
		SYMBOL_OFFSET_ENTRY(KdPrintRolloverCount),
		SYMBOL_OFFSET_ENTRY(MmLoadedUserImageList),
		SYMBOL_OFFSET_ENTRY(NtBuildLab),
		SYMBOL_OFFSET_ENTRY(KiNormalSystemCall),
		SYMBOL_OFFSET_ENTRY(KiProcessorBlock),
		SYMBOL_OFFSET_ENTRY(MmUnloadedDrivers),
		SYMBOL_OFFSET_ENTRY(MmLastUnloadedDriver),
		SYMBOL_OFFSET_ENTRY(MmTriageActionTaken),
		SYMBOL_OFFSET_ENTRY(MmSpecialPoolTag),
		SYMBOL_OFFSET_ENTRY(KernelVerifier),
		SYMBOL_OFFSET_ENTRY(MmVerifierData),
		SYMBOL_OFFSET_ENTRY(MmAllocatedNonPagedPool),
		SYMBOL_OFFSET_ENTRY(MmPeakCommitment),
		SYMBOL_OFFSET_ENTRY(MmTotalCommitLimitMaximum),
		SYMBOL_OFFSET_ENTRY(CmNtCSDVersion),
		SYMBOL_OFFSET_ENTRY(MmPhysicalMemoryBlock),
		SYMBOL_OFFSET_ENTRY(MmSessionBase),
		SYMBOL_OFFSET_ENTRY(MmSessionSize),
		SYMBOL_OFFSET_ENTRY(MmSystemParentTablePage),
		SYMBOL_OFFSET_ENTRY(MmVirtualTranslationBase),
		SYMBOL_OFFSET_ENTRY(KdPrintCircularBufferPtr),
		SYMBOL_OFFSET_ENTRY(KdPrintBufferSize),
		SYMBOL_OFFSET_ENTRY(KeLoaderBlock),
		SYMBOL_OFFSET_ENTRY(IopNumTriageDumpDataBlocks),
		SYMBOL_OFFSET_ENTRY(IopTriageDumpDataBlocks),
		SYMBOL_OFFSET_ENTRY(VfCrashDataBlock),
	};
	const ULONG max_symbol_length = 50;
	unsigned int i;

	for (i = 0; i < ARRAY_SIZE(symbol_offset_arr); i++ ) {
		if (copy_string_func(sym, symbol_offset_arr[i].str, max_symbol_length) == 0) {
			*kdbg_sym_off = offsetof(KDDEBUGGER_DATA64, KernBase);
			return APSH_S;
		}
	}

	return APSH_F;
}

win_ver_t
find_windows_version(apsh_instance_t apsh, addr_t kdbg)
{

	windows_instance_t windows = NULL;
	uint16_t APSH_FUNC_DEFINE(kdbg_size = 0);

	if (apsh->os_data == NULL)
		return APSH_OS_WIN_UNKNOWN;

	windows = apsh->os_data;

	if (windows->version && windows->version != APSH_OS_WIN_UNKNOWN)
		return windows->version;

	if (apsh_read_16_pa(apsh, kdbg + 0x14, &kdbg_size) == APSH_F)
		return APSH_OS_WIN_UNKNOWN;

	switch (kdbg_size) {
	case KDBG_WINDOWS_2000:
		return APSH_OS_WIN_2000;
	case KDBG_WINDOWS_XP:
		return APSH_OS_WIN_XP;
	case KDBG_WINDOWS_2003:
		return APSH_OS_WIN_2003;
	case KDBG_WINDOWS_VISTA:
		return APSH_OS_WIN_VISTA;
	case KDBG_WINDOWS_2008:
		return APSH_OS_WIN_2008;
	case KDBG_WINDOWS_7:
		return APSH_OS_WIN_7;
	case KDBG_WINDOWS_8:
		return APSH_OS_WIN_8;
	default:
		break;
	}

	return APSH_OS_WIN_UNKNOWN;
}

status_t
find_kdbg_address(apsh_instance_t apsh, addr_t *kdbg_pa, addr_t *kernel_va)
{

	debug_print(APSH_DEBUG_MISC, "Trying find_kdbg_address\n");

	addr_t memsize = apsh_get_max_physical_address(apsh);
	status_t ret = APSH_F;
	*kdbg_pa = 0;
	unsigned char haystack[APSH_PS_4KB];

	APSH_FUNC_DEFINE()
	addr_t paddr = 0;

	void *bm64 = boyer_moore_init((unsigned char *)"\x00\xf8\xff\xffKDBG", 8);

	APSH_FUNC_DEFINE()
	void *bm32 = boyer_moore_init((unsigned char *)"\x00\x00\x00\x00\x00\x00\x00\x00KDBG", 12);

	uint32_t APSH_FUNC_DEFINE(find_ofs_32 = 0x8), APSH_FUNC_DEFINE(find_ofs = 0);

	APSH_FUNC_DEFINE()

	uint32_t APSH_FUNC_DEFINE(find_ofs_64 = 0xc);

	for (; paddr < memsize; paddr += APSH_PS_4KB) {

		find_ofs = 0;
		APSH_FUNC_DEFINE()

		if (apsh_read_pa(apsh, paddr, APSH_PS_4KB, &haystack, NULL) == APSH_F)
			continue;

		int match_offset = boyer_moore2(bm64, haystack, APSH_PS_4KB);

		if (match_offset != -1)
			find_ofs = find_ofs_64;
		else
			match_offset = boyer_moore2(bm32, haystack, APSH_PS_4KB);

		APSH_FUNC_DEFINE()

		if (match_offset != -1) {
			if (!find_ofs)
				find_ofs = find_ofs_32;

			unsigned long kernbase_offset = 0;

			kdbg_symbol_offset("KernBase", &kernbase_offset);
			APSH_FUNC_DEFINE()

			if (match_offset - find_ofs + kernbase_offset + sizeof(uint64_t) >=
			    APSH_PS_4KB)
				continue;

			memcpy(kernel_va,
			       &haystack[(unsigned int)match_offset - find_ofs + kernbase_offset],
			       sizeof(uint64_t));
			*kdbg_pa = paddr + (unsigned int)match_offset - find_ofs;

			ret = APSH_S;
			APSH_FUNC_DEFINE()

			break;
		}
	}

	debug_print(APSH_DEBUG_MISC, "--Found KdDebuggerDataBlock at PA %.16" PRIx64 "\n",
		    *kdbg_pa);

	boyer_moore_fini(bm32);
	APSH_FUNC_DEFINE()
	boyer_moore_fini(bm64);
	return ret;
}

status_t
windows_kdbg_lookup(apsh_instance_t apsh, const char *sym, addr_t *address)
{
	status_t ret = APSH_F;
	ULONG offset = 0;

	if (kdbg_symbol_offset(sym, &offset) == APSH_F)
		goto done;

	if (kdbg_symbol_resolve(apsh, offset, address) == APSH_F)
		goto done;

	ret = APSH_S;

done:
	return ret;
}

status_t
init_from_kdbg(apsh_instance_t apsh)
{
	status_t ret = APSH_F;
	addr_t kernbase_pa = 0;
	addr_t kernbase_va = 0;
	addr_t kdbg_pa = 0;

	APSH_FUNC_DEFINE()

	if (apsh->os_data == NULL)
		goto exit;

	windows_instance_t windows = apsh->os_data;

	if (windows->kdbg_va && windows->kdbg_offset && windows->ntoskrnl &&
	    apsh->page_mode == APSH_PM_UNKNOWN) {
		windows->ntoskrnl_va = windows->kdbg_va - windows->kdbg_offset;
		goto done;
	}
	APSH_FUNC_DEFINE()

	if (apsh->page_mode == APSH_PM_UNKNOWN)
		goto find_kdbg;
	APSH_FUNC_DEFINE()

	if (windows->kdbg_va) {
		debug_print(APSH_DEBUG_MISC,
			    "using KdDebuggerDataBlock address=0x%" PRIx64 " from config\n",
			    windows->kdbg_va);
		APSH_FUNC_DEFINE()

		if (windows_kdbg_lookup(apsh, "KernBase", &windows->ntoskrnl_va) != APSH_S) {
			debug_print(
				APSH_DEBUG_MISC,
				"reading KernBase error, falling back to search methods\n");
			goto find_kdbg;
		}
		APSH_FUNC_DEFINE()

		debug_print(APSH_DEBUG_MISC, "KernBase VA=0x%" PRIx64 "\n", windows->ntoskrnl_va);

		APSH_FUNC_DEFINE()
		if (windows->kdbg_offset) {
			APSH_FUNC_DEFINE()
			if (windows->kdbg_va != (windows->ntoskrnl_va + windows->kdbg_offset)) {
				APSH_FUNC_DEFINE()
				print_error(
					"error win_kdvb and win_kdbg\n");
				APSH_FUNC_DEFINE()
				goto exit;
			}
			APSH_FUNC_DEFINE()

		} else {
			APSH_FUNC_DEFINE()
			windows->kdbg_offset = windows->kdbg_va - windows->ntoskrnl_va;
			APSH_FUNC_DEFINE()
		}
	} else if (windows->ntoskrnl && windows->kdbg_offset) {
		ULONG offset = 0;

		kdbg_symbol_offset("KernBase", &offset);
		if (apsh_read_addr_pa(apsh, windows->ntoskrnl + windows->kdbg_offset + offset,
						&windows->ntoskrnl_va) == APSH_F) {
			print_error("Inconsistent addresses passed in the config!\n");
			goto exit;
		}

		debug_print(APSH_DEBUG_MISC, "KernBase VA=0x%" PRIx64 "\n", windows->ntoskrnl_va);
		APSH_FUNC_DEFINE()

		windows->kdbg_va = windows->ntoskrnl_va - windows->kdbg_offset;
		debug_print(APSH_DEBUG_MISC, "set KdDebuggerDataBlock address=0x%" PRIx64 "\n",
			    windows->kdbg_va);
	} else {
		goto find_kdbg;
	}

	addr_t test = 0;

	if (!windows->ntoskrnl) {
		if (apsh_translate_kv2p(apsh, windows->ntoskrnl_va, &windows->ntoskrnl) == APSH_F)
			goto find_kdbg;
		APSH_FUNC_DEFINE()

		debug_print(APSH_DEBUG_MISC, "set KernBase PA=0x%" PRIx64 "\n",
			    windows->ntoskrnl);
	} else if (apsh_translate_kv2p(apsh, windows->ntoskrnl_va, &test) == APSH_F ||
		   test != windows->ntoskrnl) {
		print_error("Invalid configuration values, win_ntoskrnl not match translated "
			    "KernBase physical address\n");
		goto exit;
	}

	goto done;

find_kdbg:
	debug_print(APSH_DEBUG_MISC, "Attempting KdDebuggerDataBlock search methods\n");

	if (find_kdbg_address(apsh, &kdbg_pa, &kernbase_va) == APSH_S) {
		kernbase_pa = get_ntoskrnl_base(apsh);
		APSH_FUNC_DEFINE()
		goto found;
	}

	debug_print(APSH_DEBUG_MISC, "All KdDebuggerDataBlock search methods failed\n");
	goto exit;

found:

	debug_print(APSH_DEBUG_MISC, "set KernBase VA=0x%" PRIx64 "\n", windows->ntoskrnl_va);
	windows->ntoskrnl_va = kernbase_va;
	if (!windows->ntoskrnl) {
		printf("LibAPSH: set win_ntoskrnl=0x%" PRIx64 " in libapsh.conf for speed.\n",
		       windows->ntoskrnl);
		windows->ntoskrnl = kernbase_pa;
	} else {
		if (windows->ntoskrnl != kernbase_pa) {
			print_error("LibAPSH: physical kernel base address 0x%" PRIx64
				    " that is with conflicts with values from config file!\n",
				    kernbase_pa);
			goto exit;
		}
	}

	if (!windows->kdbg_offset) {
		APSH_FUNC_DEFINE()
		printf("LibAPSH: set win_kdbg=0x%" PRIx64 " in libapsh.conf for speed.\n",
		       windows->kdbg_offset);
		windows->kdbg_offset = kdbg_pa - windows->ntoskrnl;
		APSH_FUNC_DEFINE()
	} else if (windows->kdbg_offset != kdbg_pa - kernbase_pa) {
		print_error("LibAPSH found win_kdbg offset 0x%" PRIx64
			    " that is with conflicts with values from config file\n",
			    kdbg_pa - kernbase_pa);
		APSH_FUNC_DEFINE()
		goto exit;
	}
	APSH_FUNC_DEFINE()

	if (!windows->kdbg_va) {
		APSH_FUNC_DEFINE()
		windows->kdbg_va = windows->ntoskrnl_va + windows->kdbg_offset;
		printf("LibAPSH Suggestion: set win_kdvb=0x%" PRIx64
		       " in libapsh.conf for speed.\n",
		       windows->kdbg_va);
		APSH_FUNC_DEFINE()
	} else if (windows->kdbg_va != windows->ntoskrnl_va + windows->kdbg_offset) {
		print_error("LibAPSH found win_kdvb offset 0x%" PRIx64
			    " that is with conflicts with values from config file\n",
			    windows->ntoskrnl_va + windows->kdbg_offset);
		APSH_FUNC_DEFINE()
		goto exit;
	}

done:
	APSH_FUNC_DEFINE()
	if (!kdbg_pa) {
		kdbg_pa = windows->ntoskrnl + windows->kdbg_offset;
		APSH_FUNC_DEFINE()
	}
	windows->version = find_windows_version(apsh, kdbg_pa);
	if (windows->version == APSH_OS_WIN_UNKNOWN) {
		APSH_FUNC_DEFINE()
		print_error("Unsupported Windows version or incorrect configuration\n");
	}

	ret = APSH_S;
exit:
	return ret;
}
