/*
 * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef OS_WINDOWS_H_
#define OS_WINDOWS_H_

#include "apsh_instance.h"

#define IMAGE_DEBUG_TYPE_CODEVIEW 2
#define IMAGE_DEBUG_TYPE_MISC 4
#define RSDS 0x53445352 // SDSR
#define MAX_HEADER_SIZE 1024
#define PDB_FILENAME_LENGTH 12

struct image_debug_directory {
	uint32_t characteristics;
	uint32_t time_date_stamp;
	uint16_t major_version;
	uint16_t minor_version;
	uint32_t type;
	uint32_t size_of_data;
	uint32_t address_of_raw_data;
	uint32_t pointer_to_raw_data;
} __attribute__((packed));

struct guid;

struct cv_info_pdb70;

struct windows_instance {
	addr_t ntoskrnl;
	addr_t ntoskrnl_va;
	addr_t kdbg_va;
	addr_t sysproc;
	uint64_t tasks_offset;
	uint64_t pdbase_offset;
	uint64_t pid_offset;
	uint64_t kpcr_offset;
	uint64_t kdbg_offset;
	uint64_t pname_offset;
	uint16_t build;
	win_ver_t version;
	uint16_t major;
	uint16_t minor;
};
typedef struct windows_instance *windows_instance_t;

struct host_info {
	char pdb_guid[34];
	char *kernel_name;
};

status_t windows_init(apsh_instance_t instance, GHashTable *config, addr_t EPT);
status_t get_windows_profile(apsh_instance_t apsh, addr_t kernel_base_p, uint8_t *pe);
addr_t get_ntoskrnl_base(apsh_instance_t apsh);

status_t windows_teardown(apsh_instance_t apsh);

struct guid {
	uint32_t data1;
	uint16_t data2;
	uint16_t data3;
	uint8_t data4[8];
} __attribute__((packed));

struct cv_info_pdb70 {
	uint32_t cv_signature;
	struct guid signature;
	uint32_t age;
	uint8_t pdb_file_name[];
} __attribute__((packed));

typedef int (*check_magic_func)(uint32_t);
int find_pname_offset(apsh_instance_t apsh, check_magic_func check);
addr_t windows_find_eprocess(apsh_instance_t instance, const char *name);
addr_t eprocess_list_search(apsh_instance_t apsh, addr_t list_head, int offset, size_t len,
			    void *value);
addr_t windows_find_eprocess_list_pid(apsh_instance_t apsh, apsh_pid_t pid);
addr_t windows_find_eprocess_list_pgd(apsh_instance_t apsh, addr_t pgd);

status_t windows_pid_to_pgd(apsh_instance_t apsh, apsh_pid_t pid, addr_t *dtb);
status_t windows_pgd_to_pid(apsh_instance_t apsh, addr_t pgd, apsh_pid_t *pid);

status_t windows_kernel_symbol_to_address(apsh_instance_t apsh, const char *symbol,
					  addr_t *kernel_base_address, addr_t *address);

status_t windows_get_kernel_struct_offset(apsh_instance_t apsh, const char *symbol,
					  const char *member, addr_t *addr);

status_t windows_export_to_rva(apsh_instance_t apsh, const access_context_t *ctx, const char *symbol,
			       addr_t *rva);
char *windows_rva_to_export(apsh_instance_t apsh, addr_t rva, const access_context_t *ctx);

status_t init_from_kdbg(apsh_instance_t apsh);
status_t windows_kdbg_lookup(apsh_instance_t apsh, const char *symbol, addr_t *address);

unicode_string_t *windows_read_unicode_struct(apsh_instance_t apsh, const access_context_t *ctx);

unicode_string_t *windows_read_unicode_struct_pm(apsh_instance_t apsh, const access_context_t *ctx,
						 page_mode_t page_mode);



#endif
