/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <stdlib.h>
#include "plugin_context.h"

#include "doca_log.h"
DOCA_LOG_REGISTER(APSH::PLUGIN_CTX);


void
plugin_ctx_destroy(struct plugin_context *ctx)
{
	if (ctx != NULL) {
		(*ctx->destroy_ptr)(ctx->context);
		free(ctx);
	}
}

int
plugin_ctx_insert(GHashTable *plugin_table, enum apsh_plugin_contexts plugin_key, void *context, void (*destroy_ptr)(void *))
{
	struct plugin_context *plugin_ctx = calloc(sizeof(*plugin_ctx), 1);

	if (plugin_ctx == NULL)
		return 0;

	plugin_ctx->context = context;
	plugin_ctx->destroy_ptr = destroy_ptr;

	if (!g_hash_table_insert(plugin_table, GINT_TO_POINTER(plugin_key), plugin_ctx)) {
		DOCA_DLOG_ERR("failed to store plugin context");
		free(plugin_ctx);
		return 0;
	}

	return 1;
}

void *
plugin_ctx_lookup(GHashTable *plugin_table, enum apsh_plugin_contexts plugin_key)
{
	struct plugin_context *ctx = g_hash_table_lookup(plugin_table, GINT_TO_POINTER(plugin_key));

	if (ctx)
		return ctx->context;
	return NULL;
}
