/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef lib_doca_mema

#define lib_doca_mema
#define PAGE_SIZE 4096
#define HUGE_PAGE_SIZE 2097152

#include "mem_pool.h"
#include "mema.h"
#include <stdbool.h>

/**
 *
 */

struct doca_mema_channel {
	//	struct doca_mema_context *sctx;

	struct mema_device *sdev;          /** */
	struct ibv_context *ib_ctx;        /** */
	struct ibv_pd *pd;                 /** */
	struct ibv_cq_ex *cq_ex;           /** */
	struct ibv_qp *qp;                 /** */
	struct ibv_qp_ex *qp_ex;           /** */
	struct mlx5dv_qp_ex *mqp_ex;       /** */

	uint32_t hp_num;                   /** */
	uint32_t dma_rkey;                 /** */
	char *dma_membase;                 /** 8k */
	char *hp_dma_membase;              /** 2MB is the limit */

	// Profile file: memory region
	char *mem_regions_path;            /** */
	memp_t *mmem_config;               /** */

	struct ibv_mr
		*mr;
	struct ibv_mr *hp_mr;              /** the Huge page MR will be used for those cases when
	                                     * we need multiple outstanding 4k pages */

	// uint32_t                crossed_vhca_mkey;
	// struct snap_cross_mkey *mkey;

	uint16_t vport_vhca_id;            /** */
	uint64_t mod_allowed_mask;         /** */

	uint32_t last_cmd;                 /** */
	bool enabled;                      /** */
};

/**
 *
 */

struct mema_req {
	uint8_t opcode;                    /** */
	uint32_t list_size;                /** */
	int64_t *remote_phy_addr_list;     /** */
	char **local_phy_addr_list;        /** */
	uint32_t *len_list;                /** */
};

enum {
	MEMA_OP_USE_HUGE_PAGES = 0xF1,
};

extern int doca_mema_create_channel(struct doca_mema_channel **mdev, char *dev_name,
				    uint16_t pcie_function_id);
extern int doca_mema_destroy_channel(struct doca_mema_channel **mdev);

extern char *doca_mema_chnl_dma_read(struct doca_mema_channel *mdev, uint64_t phy_addr,
				     uint32_t len);

extern int doca_mema_chnl_dma_read_pointers_list(struct doca_mema_channel *mdev,
						 struct mema_req *phyaddr_lst);
extern int doca_mema_chnl_dma_read_pages_list(struct doca_mema_channel *mdev,
					      struct mema_req *phyaddr_lst);

#endif
