/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef _MEM_POOL
#define _MEM_POOL

#include <fcntl.h>
#include <inttypes.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>

enum huge_page { HUGE_PAGE_32MB = 33554432, HUGE_PAGE_2MB = 2097152 };
enum page_size { PAGE_32MB = 33554432, PAGE_8KB = 8192 };

#define check_rc(cond, msg) \
	({ \
		if (cond) { \
			DOCA_DLOG_INFO("\tERR: %s err %d\n", msg, err); \
			if (err) \
				DOCA_DLOG_ERR("\tPERROR"); \
			goto bail; \
		} \
	})

typedef struct mem_region_t {
	uint64_t start_address;
	uint64_t length;
} mr_t;

typedef struct machine_memory_profile_t {

	/* mrs: valid memory regions map: start_addres[i] has
	 *  the start address of memory region i
	 *  length[i] has the length of memory region i
	 */
	mr_t *mrs;
	int mrs_count;
} memp_t;

memp_t *get_mem(void);

memp_t *create_machine_mem_regions(const char *mem_region_path);
int destroy_pf_mem_regions(memp_t *mem_p);

int pf_mem_is_valid(memp_t *mem_p, uint64_t phy_addr, uint32_t len);

int virt2phy(const void *virt_addr, char **ret_phy_adrs);
int alloc_hp_buf(int *hp_num, char **ret_buf, enum huge_page hp_size);

#endif
