/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef mema
#define mema

#include <infiniband/mlx5dv.h>
#include <infiniband/verbs.h>

struct mema_cross_mkey {
	struct mlx5dv_devx_obj *devx_obj;
	uint32_t mkey;
};

struct mema_device_attr {
	int pcie_function_id;
	uint32_t flags; /* Use enum mema_device_attr_flags */
};

struct mema_device {
	uint32_t phy_ports_cnt_ex;
	uint16_t pcie_function_id;
	uint16_t vport_vhca_id;
	struct mema_cross_mkey *mkey;
	uint32_t introspection_mkey;
	bool enabled;
};

struct ibv_context *mema_open(struct ibv_device *ibdev);
struct mema_device *mema_open_device(struct ibv_context *ib_ctx, struct mema_device_attr *attr);

void mema_close_device(struct mema_device *sdev);

int mema_get_introspection_mkey(struct ibv_context *context, uint16_t pcie_function_id, uint16_t vport, uint32_t *mkey);
struct mema_cross_mkey *mema_create_cross_mkey(struct ibv_pd *pd, struct mema_device *target_sdev);
int mema_destroy_cross_mkey(struct mema_cross_mkey *mkey);

#endif
