/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __LINUX_HASHTEST_H__
#define __LINUX_HASHTEST_H__
#include "vmm.h"
#include "pslist.h"

#define MAX_FILENAME 256

struct file_entry;
struct page_permission;

typedef struct _LINUX_HASHTEST_LIST {
	struct file_entry *file_hashes;
	struct page_permission *page_permissions;
	uint32_t pid;
	char *hash_file;
	GSList *results_list;
} linux_hashtest_t;


/**
 * linux_hashtest
 * Attests a linux process by comparing its memory pages to a pre-computed hash file values
 * @param[in]  apsh  apsh instance for host/guest
 * @param[in]  process a process to attest, if set NULL, then internally get process list, and search for pid
 * @param[in]  pid pid of process to attest
 * @param[in]  results_list list to be filled with GHashTables. if set NULL resutls will be printed out
 * @return APSH status_t defined in APSH
 */

status_t linux_hashtest(apsh_instance_t apsh, linux_processes_list_t *process, uint32_t pid, const char *hash_file, linux_hashtest_t **results_list);
status_t linux_hashtest_refresh(apsh_instance_t apsh, linux_processes_list_t *process, linux_hashtest_t **results_list);
void linux_free_hashtest_results(linux_hashtest_t *results_list);

#endif /* __LINUX_HASHTEST_H__ */
