/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __LINUX_LIBS_H__
#define __LINUX_LIBS_H__

#include "jsonout.h"
#include "pslist.h"
#include "vmm.h"

typedef struct LINUX_LIBS {
	char *lib_name;
	addr_t pid;
	addr_t address;
	char *path;
	GHashTable *data;
	struct LINUX_LIBS *next;
} linux_libs_t;

/**
 * linux_libs
 * Lists the processes's memory pages present in memory
 * @param[in]  apsh  apsh instance for host/guest
 * @param[in]  process list or if set NULL, then pslist internally get process list
 * @param[out] json_obj results stored in json_object
 * @param[in]  print out on console
 * @return APSH status_t defined in APSH
 */
status_t linux_libs(apsh_instance_t apsh, linux_processes_list_t *process_list, addr_t pid, json_object **json_obj,
		    bool printout);

/**
 * linux_get_libs
 * Gets a list of libs for desire process/processes
 * @param[in]  apsh  apsh instance for host/guest
 * @param[in]  process list or if set NULL, then pslist internally get process list
 * @param[in]  pid pid for filtering process you want, 0 for all process
 * @param[out] libs_list results stored in libs_list
 * @return APSH status_t defined in APSH
 */
status_t linux_get_libs(apsh_instance_t apsh, linux_processes_list_t *process_list, addr_t pid, linux_libs_t **libs_list);

/**
 * free libs list
 * @param[in]  libs_list  pointer to libs list
 */
void linux_free_libs_list(linux_libs_t *libs_list);

#endif /* __LINUX_LIBS_H__ */
