/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __LINUX_PSLIST_H__
#define __LINUX_PSLIST_H__

#include "vmm.h"

typedef struct _LINUX_PROCESSES_LIST
{
    uint32_t pid;
    uint32_t ppid;
    int32_t uid;
    int32_t gid;
    char *name;
    char *args;
    addr_t address;
    addr_t exit_time;
    uint64_t state;
    uint64_t cpu_time;
    GHashTable* hash;
    struct _LINUX_PROCESSES_LIST *next;
} linux_processes_list_t;

/**
 * pslist
 * Lists the processes present in a particular linux memory image.
 * @param[in]  apsh  apsh instance for host/guest
 * @param[in]  process list or if set NULL, then pslist internally get process list
 * @param[out] json_obj results stored in json_object
 * @param[in]  print out on console
 * @return APSH status_t defined in APSH
 */
status_t linux_pslist(apsh_instance_t apsh, linux_processes_list_t *process_list, json_object **json_obj, bool printout);

/**
 * linux_pslist_processes_list
 * get processes list
 * @param[in]  apsh  apsh instance for host/guest
 * @param[out] process list
 * @return APSH status_t defined in APSH
 */
status_t linux_pslist_processes_list(apsh_instance_t apsh, linux_processes_list_t **pprocess_list);

/**
 * free process list
 */
void linux_free_process_list(linux_processes_list_t *process_list);

/**
 * get process from process list by given pid
 * @param[in]  process list
 * @param[in]  process id
 * @return pointer to demanded process, or NULL
*/
linux_processes_list_t *linux_pslist_get_process(linux_processes_list_t *process_list, uint32_t pid);


#endif /* __LINUX_PSLIST_H__ */
