/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __LINUX_THREADLIST_H__
#define __LINUX_THREADLIST_H__
#include "vmm.h"
#include "pslist.h"

typedef struct _LINUX_THREAD_LIST
{
    uint32_t pid;
    uint32_t tid;
    char *proc_name;
    char *thread_name;
    addr_t address;
    uint64_t state;
    GHashTable* hash;
    struct _LINUX_THREAD_LIST *next;
} linux_thread_list_t;


/**
 * linux_threadlist
 * Lists the threads present for a process with specified pid or all processes if pid==0.
 * @param[in]  apsh  apsh instance for host/guest
 * @param[in]  process_list list or processes to retrieve their threads or if set NULL, then pslist internally get process list
 * @param[in]  pid of the process to retrieve its threads or if set NULL, then pslist internally get process list
 * @param[out] json_obj results stored in json_object
 * @param[in]  printout out on console
 * @return APSH status_t defined in APSH
 */
status_t linux_threadlist(apsh_instance_t apsh, linux_processes_list_t *process_list, uint32_t pid, json_object **json_obj, bool printout);


/**
 * linux_get_threadlist
 * Returns a thread list for process with specified pid or all processes if pid==0..
 * @param[in]  apsh apsh instance for host/guest
 * @param[in]  process_list list or processes to retrieve their threads or if set NULL, then pslist internally get process list
 * @param[in]  pid of the process to retrieve its threads or if set 0 then retrieve all threads of all processes, then pslist internally get process list
 * @param[out] thread_list results stored in linux_thread_list_t
 * @return APSH status_t defined in APSH
 */
status_t linux_get_threadlist(apsh_instance_t apsh, linux_processes_list_t *process_list, uint32_t pid, linux_thread_list_t** thread_list);



/**
 * Frees a thread list
 * @param[in] thread_list thread list to be freed
 */
void linux_free_thread_list(linux_thread_list_t *thread_list);



#endif /* __LINUX_THREADLIST_H__ */
