/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __LINUX_VMA_H__
#define __LINUX_VMA_H__

#include "vmm.h"
#include "jsonout.h"
#include "pslist.h"

static bool offset_loaded_vad = false;

typedef struct LINUX_VMA
{
    char* process_name;
    addr_t pid;
    addr_t offset;
    char* protection;
    addr_t start;
    addr_t end;
    char* path;
    GHashTable* data;
    bool path_alloc;
    struct LINUX_VMA *next;
} linux_vma_t;

/**
 * linux_vma
 * Lists the processes's memory pages present in memory
 * @param[in]  apsh  apsh instance for host/guest
 * @param[in]  process list or if set NULL, then pslist internally get process list
 * @param[out] json_obj results stored in json_object
 * @param[in]  print out on console
 * @return APSH status_t defined in APSH
 */
status_t linux_vma(apsh_instance_t apsh, linux_processes_list_t *process_list, addr_t pid, json_object **json_obj, bool printout);

/**
 * linux_get_vma
 * Gets a list of vma for desire process/processes
 * @param[in]  apsh  apsh instance for host/guest
 * @param[in]  process list or if set NULL, then pslist internally get process list
 * @param[in]  pid pid for filtering process you want, 0 for all process
 * @param[out] vma_list results stored in vma_list
 * @return APSH status_t defined in APSH
 */
status_t linux_get_vma(apsh_instance_t apsh, linux_processes_list_t *process_list, addr_t pid, linux_vma_t **vma_list);

/**
 * free vma list
 * @param[in]  vma_list  pointer to vma list
 */
void linux_free_vma_list(linux_vma_t *vma_list);

#endif /* __LINUX_VMA_H__ */
