/*
 * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __WINDOWS_DLLLIST_H__
#define __WINDOWS_DLLLIST_H__

#include "pslist.h"
#include "vmm.h"

struct windows_export_info {
	addr_t dll_base;
	uint32_t dll_size;
	GHashTable *exports;
} __attribute__((packed));

typedef struct WINDOWS_LIBS {
	addr_t pid;
	char *image_name;
	unicode_string_t *dll_base_name;
	unicode_string_t *dll_full_name;
	uint32_t image_size;
	GHashTable *data;
	struct WINDOWS_LIBS *next;
} windows_libs_t;

status_t windows_store_dll_functions(apsh_instance_t apsh, addr_t dll_base, uint32_t dll_size, apsh_pid_t pid, char *full_path);
status_t windows_dlllist(apsh_instance_t apsh, windows_processes_list_t *process_list, uint32_t pid, json_object **json_obj,
		 bool printout, bool monitor_mode);
status_t windows_save_user_exports(apsh_instance_t apsh, windows_processes_list_t *process_list);
status_t windows_get_dlls(apsh_instance_t apsh, windows_processes_list_t *process_list, windows_libs_t **libs_list);
void windows_free_libs_list(windows_libs_t *libs_list);
#endif /* __WINDOWS_DLLLIST_H__ */
