/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __WINDOWS_ENVARS_H__
#define __WINDOWS_ENVARS_H__

#include "vmm.h"
#include "pslist.h"

typedef struct _WINDOWS_ENVARS_LIST {
	addr_t pid;
	char *procname;
	char *variable;
	char *value;
	uint64_t environment;
	GHashTable *data;
	struct _WINDOWS_ENVARS_LIST *next;
} windows_envars_t;

/**
 * windows_envars
 *
 * @param[in]  apsh
 * @param[in]  process list or if set NULL, then pstree will internally get process list
 * @param[in]  pid pid of process to get envars of (or 0 if we want to get envars of all processes)
 * @param[out] results_list list of envars data are stored in results_list
 * @return APSH status_t defined in APSH
 */
status_t windows_envars(apsh_instance_t apsh, windows_processes_list_t *process_list, uint32_t pid, windows_envars_t **results_list);

/**
 * free envars list
 * @param[in]  envars_list pointer to envars list
 */
void windows_free_envars_list(windows_envars_t *envars_list);

#endif /* __WINDOWS_ENVARS_H__ */
