/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include "private/common_defines.h"
#include "handles.h"
#include "doca_log.h"
#include "../../core/APSH/apsh_instance.h"
#include "../../core/APSH/plugin_context.h"
#include "../../doca_apsh_attr.h"

DOCA_LOG_REGISTER(APSH::Windows::HANDLES);

static const char* getTypeName(uint8_t type_index) {
	static const char* const pc_type_map[] =
	{
		"",                                 /* i0 */
		"",                                 /* i1 */
		"Type",                             /*i2 ptre40f3deccbc0 name*/
		"Directory",                        /*i3 ptre40f3decc4e0 name*/
		"SymbolicLink",                     /*i4 ptre40f3decc640 name*/
		"Token",                            /*i5 ptre40f3decc7a0 name*/
		"Job",                              /*i6 ptre40f3decc900 name*/
		"Process",                          /*i7 ptre40f3decca60 name*/
		"Thread",                           /*i8 ptre40f3decebc0 name*/
		"Partition",                        /*i9 ptre40f3decf140 name*/
		"UserApcReserve",                   /*i10 ptre40f3decf400 name*/
		"IoCompletionReserve",              /*i11 ptre40f3dece380 name*/
		"ActivityReference",                /*i12 ptre40f3decf2a0 name*/
		"PsSiloContextPaged",               /*i13 ptre40f3decf820 name*/
		"PsSiloContextNonPaged",            /*i14 ptre40f3dece220 name*/
		"DebugObject",                      /*i15 ptre40f3dece900 name*/
		"Event",                            /*i16 ptre40f3decf560 name*/
		"Mutant",                           /*i17 ptre40f3dece4e0 name*/
		"Callback",                         /*i18 ptre40f3deced20 name*/
		"Semaphore",                        /*i19 ptre40f3decee80 name*/
		"Timer",                            /*i20 ptre40f3decff00 name*/
		"IRTimer",                          /*i21 ptre40f3dece0c0 name*/
		"Profile",                          /*i22 ptre40f3dece7a0 name*/
		"KeyedEvent",                       /*i23 ptre40f3dece640 name*/
		"WindowStation",                    /*i24 ptre40f3decea60 name*/
		"Desktop",                          /*i25 ptre40f3decf6c0 name*/
		"Composition",                      /*i26 ptre40f3decf980 name*/
		"RawInputManager",                  /*i27 ptre40f3decfae0 name*/
		"CoreMessaging",                    /*i28 ptre40f3decfc40 name*/
		"TpWorkerFactory",                  /*i29 ptre40f3decfda0 name*/
		"Adapter",                          /*i30 ptre40f3dff8a60 name*/
		"Controller",                       /*i31 ptre40f3dff9820 name*/
		"Device",                           /*i32 ptre40f3dff9da0 name*/
		"Driver",                           /*i33 ptre40f3dff80c0 name*/
		"IoCompletion",                     /*i34 ptre40f3dff96c0 name*/
		"WaitCompletionPacket",             /*i35 ptre40f3dff9ae0 name*/
		"File",                             /*i36 ptre40f3dff9980 name*/
		"TmTm",                             /*i37 ptre40f3dff9c40 name*/
		"TmTx",                             /*i38 ptre40f3dff8380 name*/
		"TmRm",                             /*i39 ptre40f3dff9f00 name*/
		"TmEn",                             /*i40 ptre40f3dff8220 name*/
		"Section",                          /*i41 ptre40f3dff8d20 name*/
		"Session",                          /*i42 ptre40f3dff84e0 name*/
		"Key",                              /*i43 ptre40f3dff8bc0 name*/
		"RegistryTransaction",              /*i44 ptre40f3dff8640 name*/
		"ALPC Port",                        /*i45 ptre40f3dff87a0 name*/
		"PowerRequest",                     /*i46 ptre40f3dff8900 name*/
		"WmiGuid",                          /*i47 ptre40f3dff8e80 name*/
		"EtwRegistration",                  /*i48 ptre40f3f0fbc40 name*/
		"EtwSessionDemuxEntry",             /*i49 ptre40f3f0fbf00 name*/
		"EtwConsumer",                      /*i50 ptre40f3f0fb6c0 name*/
		"CoverageSampler",                  /*i51 ptre40f3f0fb820 name*/
		"DmaAdapter",                       /*i52 ptre40f3f0fa900 name*/
		"PcwObject",                        /*i53 ptre40f3f0fae80 name*/
		"FilterConnectionPort",             /*i54 ptre40f3f0fb980 name*/
		"FilterCommunicationPort",          /*i55 ptre40f3f0fbda0 name*/
		"NdisCmState",                      /*i56 ptre40f47bfe900 name*/
		"DxgkSharedResource",               /*i57 ptre40f47bff400 name*/
		"DxgkSharedKeyedMutexObject",       /*i58 ptre40f47bfe0c0 name*/
		"DxgkSharedSyncObject",             /*i59 ptre40f47bff6c0 name*/
		"DxgkSharedSwapChainObject",        /*i60 ptre40f47bff820 name*/
		"DxgkDisplayManagerObject",         /*i61 ptre40f47bff980 name*/
		"DxgkCurrentDxgProcessObject",      /*i62 ptre40f47bffc40 name*/
		"DxgkSharedProtectedSessionObject", /*i63 ptre40f47bffda0 name*/
		"DxgkSharedBundleObject",           /*i64 ptre40f47bfe380 name*/
		"DxgkCompositionObject",            /*i65 ptre40f47bfff00 name*/
		"VRegConfigurationContext"         /*i66 ptre40f47bfe220 name*/
	};

	size_t size = sizeof pc_type_map / sizeof pc_type_map[0];
	if (type_index < size)
		return pc_type_map[type_index];

	return "";
}

void
windows_free_handles(windows_handles_t *handles)
{
	windows_handles_t *p_next;

	while (handles) {
		p_next = handles->next;
		g_hash_table_destroy(handles->hash);
		free(handles->procname);
		free(handles->ObjectName);
		free(handles->HandleType);
		free(handles);
		handles = p_next;
	}
}


status_t
windows_handles_init_offsets_struct(apsh_instance_t apsh, windows_handles_offsets_struct_t *offsets_struct)
{
	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_OBJECT_HEADER", "TypeIndex",
							 &offsets_struct->object_header_typeindex_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_OBJECT_HEADER", "Body",
							 &offsets_struct->object_header_body_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_EPROCESS", "ImageFileName",
							 &offsets_struct->ImageFileName_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_EPROCESS", "ObjectTable",
							 &offsets_struct->ObjectTable_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_HANDLE_TABLE", "TableCode",
							 &offsets_struct->TableCode_offset) == APSH_F)
		return APSH_F;

	/* get _FILE_OBJECT.FileName, unicode struct */
	if (apsh_get_struct_member_offset_from_apsh_json(
			apsh, "_FILE_OBJECT", "DeviceObject", &offsets_struct->DeviceObject_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_OBJECT_HEADER", "Body",
								&offsets_struct->Body_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_OBJECT_HEADER_NAME_INFO",
							"Name", &offsets_struct->Name_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_FILE_OBJECT", "FileName",
								&offsets_struct->FileName_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(
			apsh, "_EPROCESS", "UniqueProcessId", &offsets_struct->Pid_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_ETHREAD", "Cid",
								&offsets_struct->Cid_offset) == APSH_F)
		return APSH_F;


	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_CM_KEY_BODY",
								"KeyControlBlock",
								&offsets_struct->KeyControlBlock_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_CM_KEY_CONTROL_BLOCK",
								"NameBlock",
								&offsets_struct->NameBlock_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_CM_KEY_CONTROL_BLOCK",
								"Flags", &offsets_struct->Flags_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_CM_KEY_CONTROL_BLOCK",
								"ParentKcb",
								&offsets_struct->ParentKcb_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_CM_NAME_CONTROL_BLOCK",
								"Name", &offsets_struct->ControlBlock_Name_offset) == APSH_F)
		return APSH_F;

	if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_CM_NAME_CONTROL_BLOCK",
								"NameLength",
								&offsets_struct->NameLength_offset) == APSH_F)
		return APSH_F;

	return APSH_S;
}

status_t
windows_get_offsets(apsh_instance_t apsh, windows_handles_offsets_struct_t **offsets)
{
	windows_handles_offsets_struct_t *_offsets = (windows_handles_offsets_struct_t *)plugin_ctx_lookup(apsh->plugin_ctx, PLUGIN_HANDLES);

	if (_offsets == NULL) {
		_offsets = malloc(sizeof(*_offsets));

		if (_offsets == NULL) {
			DOCA_DLOG_ERR("Failed to allocate plugin context memory");
			return APSH_F;
		}
		windows_handles_init_offsets_struct(apsh, _offsets);

		if (!plugin_ctx_insert(apsh->plugin_ctx, PLUGIN_HANDLES, _offsets, free)) {
			free(_offsets);
			return APSH_F;
		}
	}
	*offsets = _offsets;
	return APSH_S;
}

static status_t
find_cookie(apsh_instance_t apsh, uint32_t *cookie)
{
	uint32_t ObHeaderCookie;

	if (apsh_read_32_ksym(apsh, "ObHeaderCookie", &ObHeaderCookie) == APSH_F)
		return APSH_F;

	*cookie = ObHeaderCookie;

	/*
	 * cookie offset: 5498332
	 * cookie: 4195661638
	 */

	return APSH_S;
}

static status_t
get_tablecode_comp(apsh_instance_t apsh, uint32_t pid, addr_t process_address,
		   uint64_t *tablecode_comp, windows_handles_offsets_struct_t *offsets)
{
	*tablecode_comp = 0;

	/* get _EPROCESS.ObjectTable pointer to _HANDLE_TABLE */
	addr_t ObjectTable_va, ObjectTable;

	ObjectTable_va = process_address + offsets->ObjectTable_offset;
	if (apsh_read_64_va(apsh, ObjectTable_va, pid, &ObjectTable) == APSH_F)
		return APSH_F;

	/* get _HANDLE_TABLE.TableCode */
	addr_t TableCode_va;
	uint64_t TableCode = 0;

	TableCode_va = ObjectTable + offsets->TableCode_offset;
	if (apsh_read_64_va(apsh, TableCode_va, pid, &TableCode) == APSH_F)
		return APSH_F;

	*tablecode_comp = TableCode;
	return APSH_S;
}

static char *
strInsert(char *str1, const char *str2, int pos)
{
	size_t l1 = strlen(str1);
	size_t l2 = strlen(str2);

	if (pos < 0)
		pos = 0;
	if (pos > l1)
		pos = l1;

	char *p = str1 + pos;

	memmove(p + l2, p, l1 - pos);
	memcpy(p, str2, l2);
	str1[l1 + l2] = '\0';
	return str1;
}

static char *
get_obj_name(apsh_instance_t apsh, uint32_t pid, char *procname, char *obj_type, addr_t entry,
	     windows_handles_offsets_struct_t *offsets)
{
	char *obj_name = malloc(obj_name_max_length);

	if (obj_name == NULL) {
		DOCA_DLOG_ERR("failed to malloc");
		return NULL;
	}

	memset(obj_name, '\0', obj_name_max_length);
	if (strcmp(obj_type, "File") == 0) {
		/* cast entry to _FILE_OBJECT */

		addr_t DeviceObject_va;
		addr_t DeviceObject;

		/* get _FILE_OBJECT.DeviceObject */
		char devicename[] = "[Unknown]";

		if (apsh_read_64_va(apsh, entry + offsets->DeviceObject_offset, 0, &DeviceObject) ==
		    APSH_F) /* pid */
			goto error_exit;

/* get _OBJECT_HEADER.InfoMask */
#if 0
		addr_t InfoMask_offset, InfoMask_va;
		uint8_t InfoMask;

		if (apsh_get_struct_member_offset_from_apsh_json(apsh, "_OBJECT_HEADER", "InfoMask",
								 &InfoMask_offset) == APSH_F)
			goto error_exit;

		InfoMask_va = DeviceObject + InfoMask_offset;

		if (apsh_read_64_va(apsh, InfoMask_va, 0, &InfoMask) == APSH_F) /* pid */
			goto error_exit;
#endif
		/*  Vol3,  framework\symbols\windows\extension\pool.py NameInfo()  @ line #316
		 */
		/* uint8_t name_info_bit = 0x02;
		 * uint8_t calculated_index = InfoMask & (name_info_bit | (name_info_bit - 1));
		 * calculate header_offset
		 */
		uint8_t header_offset = 32; /* magic */
		addr_t NameInfo = DeviceObject - offsets->Body_offset - header_offset;
		addr_t Name_va;

		Name_va = NameInfo + offsets->Name_offset;

		unicode_string_t *us1 = NULL;
		unicode_string_t out1 = {0};

		us1 = apsh_read_unicode_str_va(apsh, Name_va, 0); /* pid */
		if (us1 && APSH_S == apsh_convert_str_encoding(us1, &out1, "UTF-8")) {
			snprintf(obj_name, obj_name_max_length, "\\Device\\%s", out1.contents);
			free(out1.contents);
		}
		if (us1)
			apsh_free_unicode_str(us1);

		addr_t FileName_va;

		FileName_va = entry + offsets->FileName_offset;

		unicode_string_t *us = NULL;
		unicode_string_t out = {0};
		int name_size = strlen(obj_name);

		us = apsh_read_unicode_str_va(apsh, FileName_va, 0);
		if (name_size < (obj_name_max_length - 1) && us && APSH_S == apsh_convert_str_encoding(us, &out, "UTF-8")) {
			char *p = obj_name + name_size;

			snprintf(p, obj_name_max_length - name_size, "%s", out.contents);
			free(out.contents);
		}
		if (us)
			apsh_free_unicode_str(us);
	} else if (strcmp(obj_type, "Process") == 0) {
		/* cast entry to _EPROCESS */

		addr_t Pid_va;
		uint32_t _pid;

		Pid_va = entry + offsets->Pid_offset;
		apsh_read_32_va(apsh, Pid_va, 0, &_pid); // PID
		snprintf(obj_name, obj_name_max_length, "%s Pid %d", procname, _pid);
	} else if (strcmp(obj_type, "Thread") == 0) {
		/* cast entry to _ETHREAD */
		addr_t Cid_va;

		Cid_va = entry + offsets->Cid_offset;

		struct client_id_t {
			addr_t UniqueProcess;
			addr_t UniqueThread;
		};
		struct client_id_t cid;

		if (apsh_read_va(apsh, Cid_va, 0, sizeof(struct client_id_t), &cid, NULL) ==
		    APSH_F) {
			goto error_exit;
		}
		snprintf(obj_name, obj_name_max_length, "Tid %ld Pid %d", cid.UniqueThread, pid);
	} else if (strcmp(obj_type, "Key") == 0) {
		/* cast entry to _CM_KEY_BODY */

		/* get _CM_KEY_BODY.KeyControlBlock, pointer to _CM_KEY_CONTROL_BLOCK */
		addr_t KeyControlBlock_va;
		addr_t KeyControlBlock;
		addr_t Name_offset;

		KeyControlBlock_va = entry + offsets->KeyControlBlock_offset;
		apsh_read_64_va(apsh, KeyControlBlock_va, 0, &KeyControlBlock); /* pid */

		/*
		 *   vol 3 /frameworke/symbols/windows/extensions/registry.py
		 */
		enum {
			KEY_IS_VOLATILE = 0x01,
			KEY_HIVE_EXIT = 0x02,
			KEY_HIVE_ENTRY = 0x04,
			KEY_NO_DELETE = 0x08,
			KEY_SYM_LINK = 0x10,
			KEY_COMP_NAME = 0x20,
			KEY_PREFEF_HANDLE = 0x40,
			KEY_VIRT_MIRRORED = 0x80,
			KEY_VIRT_TARGET = 0x100,
			KEY_VIRTUAL_STORE = 0x200
		} RegKeyFlags;

		addr_t kcb = KeyControlBlock;

		/* while loop  _CM_KEY_CONTROL_BLOCK. */
		while (kcb) {
			/* get kcb.parant */
			addr_t ParentKcb;

			if (apsh_read_64_va(apsh, kcb + offsets->ParentKcb_offset, 0, &ParentKcb) ==
			    APSH_F) /* pid */
				break;

			if (ParentKcb == 0)
				break;

			/* get _CM_KEY_CONTROL_BLOCK.Flags */
			uint32_t Flags = 0;
			addr_t NameBlock;
			uint16_t NameLength;

			if (apsh_read_32_va(apsh, kcb + offsets->Flags_offset, 0, &Flags) == APSH_F)
				break;

			Flags = (Flags >> 16) & 0xFF;
			if ((Flags & KEY_HIVE_ENTRY) == KEY_HIVE_ENTRY) {
				kcb = ParentKcb;
				if (kcb == 0)
					break;
				apsh_read_64_va(apsh, kcb + offsets->ParentKcb_offset, 0, &ParentKcb);
			}

			/* get _CM_KEY_CONTROL_BLOCK.NameBlock */
			if (apsh_read_64_va(apsh, kcb + offsets->NameBlock_offset, 0, &NameBlock) == APSH_F)
				break;

			/* get _CM_NAME_CONTROL_BLOCK.NameLength */
			if (apsh_read_16_va(apsh, NameBlock + offsets->NameLength_offset, 0, &NameLength) ==
			    APSH_F)
				break;

			size_t length = (NameLength + 1) * sizeof(uint16_t);
			char *wName = malloc(length);
			char *name_utf8 = malloc(length);

			if(wName == NULL || name_utf8 == NULL){
				if(wName == NULL)
					free(name_utf8);
				if(name_utf8 == NULL)
					free(wName);
				DOCA_DLOG_ERR("failed to malloc string");
				break;
			}

			/* get _CM_NAME_CONTROL_BLOCK.Name unicode string */
			if (apsh_read_va(apsh, (NameBlock + offsets->ControlBlock_Name_offset), 0, length, &wName[0],
					 NULL) == APSH_F) {
				free(name_utf8);
				free(wName);
				DOCA_DLOG_ERR("failed to read Name");
				break;
			}

			char *p = name_utf8;
			/* covert unicode string to char[] */
			int i;

			for (i = 0; i < length; i++) {
				if (wName[i] == 0 && wName[i + 1] == 0)
					break;

				if (wName[i])
					*p++ = wName[i];
			}

			*p++ = '\\';
			*p = '\0';

			/* insert name_utf8 into objname at pos 0 */
			strInsert(obj_name, name_utf8, 0);

			free(wName);
			free(name_utf8);

			kcb = ParentKcb;
		}
	} else {
		uint8_t header_offset = 32; // magic
		addr_t NameInfo = entry - offsets->Body_offset - header_offset;
		addr_t Name_va;

		Name_va = NameInfo + offsets->Name_offset;

		unicode_string_t *us = NULL;
		unicode_string_t out = {0};

		us = apsh_read_unicode_str_va(apsh, Name_va, 0); // pid
		if (us && APSH_S == apsh_convert_str_encoding(us, &out, "UTF-8")) {
			snprintf(obj_name, obj_name_max_length, "%s", out.contents);
			free(out.contents);
		}
		if (us)
			apsh_free_unicode_str(us);
	}

error_exit:
	return obj_name;
}

static status_t
_make_handle_array(apsh_instance_t apsh, uint32_t pid, char *procname, uint64_t offset, uint16_t level, uint16_t depth,
		   uint32_t cookie, windows_handles_offsets_struct_t *offsets, windows_handles_t **handles,
		   uint64_t *count_handles)
{
	int limit = GPOINTER_TO_INT(g_hash_table_lookup(apsh->limits_table, GINT_TO_POINTER(DOCA_APSH_HANDLES_LIMIT)));

	if (handles == NULL)
		return APSH_F;

	windows_handles_t *handles_head = NULL;
	windows_handles_t *handle_iter;
	uint32_t count;
	uint32_t subtype_size;
	int i = 0;
	/* load table */
	if (level > 0) {
		subtype_size = sizeof(addr_t); // pointer
		count = 0x1000 / subtype_size;
	} else {
		subtype_size = sizeof(_WINDOWS_HANDLE_TABLE_ENTRY_t); // _WINDOWS_HANDLE_TABLE_ENTRY
		count = 0x1000 / subtype_size;
	}

	// check offset validity
	addr_t paddr;

	if (apsh_translate_uv2p(apsh, offset, pid, &paddr) == APSH_F)
		return APSH_F;

	uint64_t masked_offset = (offset & layer_object_maximum_address);

	/* for each entry in the table, retrieve object info or recursively call _make_handle_array
	 * if level > 0
	 */
	for (i = 0; i < count; i++) {
		/* load a entry from table */
		addr_t entry_offset = offset + i * subtype_size;

		if (level > 0) {
			addr_t entry_va;

			if (apsh_read_va(apsh, entry_offset, 0, subtype_size, &entry_va, NULL) == APSH_F) {
				DOCA_DLOG_ERR("failed to get subtypesize");
				goto error_exit;
			}

			if (*count_handles >= limit)
				goto success_exit;

			if (_make_handle_array(apsh, pid, procname, entry_va, level - 1, depth, cookie, offsets,
					       handles, count_handles) == APSH_F)
				goto error_exit;

			depth += 1;
		} else {
			_WINDOWS_HANDLE_TABLE_ENTRY_t entry;

			if (apsh_read_va(apsh, entry_offset, 0, subtype_size, &entry, NULL) == APSH_F)
				goto error_exit;

			/* calculate handle value */
			uint64_t handle_multiplier = 4;
			uint64_t handle_level_base = depth * count * handle_multiplier;

			uint64_t handle_value = ((entry_offset - masked_offset) /
						 (subtype_size / handle_multiplier)) +
						handle_level_base;

			if (entry.LowValue == 0)
				continue;

			addr_t object_header_offset =
				((entry.LowValue & 0xFFFFFFFFFFFFFFF8) >> magic);

			/* load object header.typeindex */
			uint8_t object_header_typeindex;

			if (apsh_read_8_va(apsh, object_header_offset + offsets->object_header_typeindex_offset, pid,
					   &object_header_typeindex) == APSH_F)
				continue;

			/* decode object type */
			uint16_t type_index = ((object_header_offset >> 8) ^ cookie ^ object_header_typeindex) & 0xFF;

			if (type_index == 0 || type_index > 66)
				continue;

			uint64_t GrantedAccess =
				(entry.HighValue & 0x1FFFFFFFFF); /* see _WINDOWS_HANDLE_TABLE_ENTRY_t */
			addr_t body_va = object_header_offset + offsets->object_header_body_offset;
			addr_t body_entry;

			if (apsh_read_64_va(apsh, body_va, pid, &body_entry) == APSH_F)
				continue;

			char *procname_cpy = strdup(procname);

			if (procname_cpy == NULL) {
				DOCA_DLOG_ERR("failed to allocate memory");
				goto error_exit;
			}

			char *obj_type = strndup(getTypeName(type_index), strlen(getTypeName(type_index)));

			if (obj_type == NULL) {
				DOCA_DLOG_ERR("failed to allocate memory");
				free(procname_cpy);
				goto error_exit;
			}

			/* get object name */
			char *obj_name = get_obj_name(apsh, pid, procname, obj_type, body_va, offsets); /* body_entry */

			if (obj_name == NULL) {
				DOCA_DLOG_ERR("failed to allocate memory");
				free(obj_type);
				free(procname_cpy);
				goto error_exit;
			}

			GHashTable *hash = g_hash_table_new(NULL, NULL);

			if (hash == NULL) {
				DOCA_DLOG_ERR("failed to allocate memory");
				free(procname_cpy);
				free(obj_type);
				free(obj_name);
				goto error_exit;
			}

			windows_handles_t *new_handle = malloc(sizeof(windows_handles_t));

			if (new_handle == NULL) {
				DOCA_DLOG_ERR("failed to allocate memory");
				free(procname_cpy);
				free(obj_type);
				free(obj_name);
				g_hash_table_destroy(hash);
				goto error_exit;
			}

			new_handle->next = NULL;
			new_handle->pid = pid;
			new_handle->procname = procname_cpy;
			new_handle->ObjectName = obj_name;
			new_handle->HandleValue = (uint32_t)(handle_value & 0xFFFFFFFF);
			new_handle->HandleType = obj_type;
			new_handle->GrantedAccess = GrantedAccess;
			new_handle->handle_address = body_va;

			g_hash_table_insert(hash, GINT_TO_POINTER(DOCA_APSH_HANDLE_PID),
					    GINT_TO_POINTER(new_handle->pid));
			g_hash_table_insert(hash, GINT_TO_POINTER(DOCA_APSH_HANDLE_COMM), new_handle->procname);
			g_hash_table_insert(hash, GINT_TO_POINTER(DOCA_APSH_HANDLE_VALUE),
					    GINT_TO_POINTER(new_handle->HandleValue));
			g_hash_table_insert(hash, GINT_TO_POINTER(DOCA_APSH_HANDLE_TYPE), new_handle->HandleType);
			g_hash_table_insert(hash, GINT_TO_POINTER(DOCA_APSH_HANDLE_ACCESS),
					    GINT_TO_POINTER(new_handle->GrantedAccess));
			g_hash_table_insert(hash, GINT_TO_POINTER(DOCA_APSH_HANDLE_NAME), new_handle->ObjectName);
			g_hash_table_insert(hash, GINT_TO_POINTER(DOCA_APSH_HANDLE_TABLE_ENTRY),
					    GINT_TO_POINTER(new_handle->handle_address));

			new_handle->hash = hash;

			/* add to handles list */
			if (handles_head == NULL) { // first handle in a new list
				handles_head = new_handle;
				new_handle->tail = new_handle;
			} else {
				handles_head->tail->next = new_handle;
				handles_head->tail = new_handle;
			}

			if (++(*count_handles) >= limit)
				goto success_exit;
		}
	}
success_exit:
	/* insert new list section at the end of the list */
	if (*handles) {
		handle_iter = *handles;

		while (handle_iter->next)
			handle_iter = handle_iter->next;
		handle_iter->next = handles_head;
	} else
		*handles = handles_head;
	return APSH_S;
error_exit:
	windows_free_handles(handles_head);
	windows_free_handles(*handles);
	*handles = NULL;
	*count_handles = 0;
	return APSH_F;
}

status_t
windows_handles(apsh_instance_t apsh, windows_processes_list_t *process_list, uint32_t pid,
				windows_handles_t **handles)
{
	uint32_t cookie = 0;
	status_t retval = APSH_F;
	windows_processes_list_t *psl, *psl_allocation = NULL;;
	windows_handles_offsets_struct_t *offsets = NULL;

	if (windows_get_offsets(apsh, &offsets) == APSH_F)
		return APSH_F;

	/* find cookie */
	if (find_cookie(apsh, &cookie) == APSH_F) {
		DOCA_DLOG_ERR("failed to find cookie");
		return APSH_F;
	}

	/* get process list */
	if (process_list == NULL) {
		if (windows_pslist_processes_list(apsh, &psl_allocation) == APSH_F)
			return APSH_F;
		psl = psl_allocation;
	} else
		psl = process_list;

	/* for each process, handle objecttable */
	while (psl != NULL) {
		if (pid == 0 || pid == psl->pid) {
			char *procname;

			/* get process name */
			procname = apsh_read_str_va(apsh, psl->address + offsets->ImageFileName_offset, 0);
			if (procname == NULL) {
				DOCA_DLOG_ERR("failed to read procname");
				goto next_process;
			}

			/* get tablecode */
			uint64_t _tablecode_comp;

			if (get_tablecode_comp(apsh, psl->pid, psl->address, &_tablecode_comp, offsets) == APSH_F) {
				DOCA_DLOG_ERR("failed to get table code");
				free(procname);
				goto next_process;
			}

			/* calculate tablecoce, tablelevel */
			uint64_t tablecode = _tablecode_comp & (~7); /* level mask = 7 */
			uint16_t tablelevel = (uint16_t)(_tablecode_comp & 7);
			uint64_t count_handles = 0;
			*handles = NULL;

			/* process handles table */
			_make_handle_array(apsh, psl->pid, procname, tablecode, tablelevel, 0, cookie, offsets,
					   handles, &count_handles); /* depth is 0 */
			free(procname);
		}

next_process:
		if (pid == psl->pid)
			break;

		/* next process */
		psl = psl->next;
	}

	retval = APSH_S;

error_exit:
	if (psl_allocation == NULL)
		windows_free_process_list(psl_allocation);

	return retval;
}
