/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __WINDOWS_HANDLES_H__
#define __WINDOWS_HANDLES_H__

#include "vmm.h"
#include "pslist.h"

typedef struct _WINDOWS_HANDLES
{
    uint32_t pid;
    char *procname;
    char *ObjectName;
    uint16_t HandleValue;
    char *HandleType;
    uint32_t GrantedAccess;
    addr_t handle_address;
    GHashTable *hash;
    struct _WINDOWS_HANDLES *next;
    struct _WINDOWS_HANDLES *tail;
} windows_handles_t;

// windows struct _WINDOWS_HANDLE_TABLE_ENTRY
typedef struct _WINDOWS_HANDLE_TABLE_ENTRY
{
    uint64_t LowValue;  // magic, right shift by 16,
    uint64_t HighValue; // GrantedAccessBits  bit 0 - 24

} _WINDOWS_HANDLE_TABLE_ENTRY_t;

// handles offsets struct
typedef struct _WINDOWS_HANDLES_OFFSETS_struct
{
    addr_t ImageFileName_offset;
    addr_t object_header_body_offset;
    addr_t object_header_typeindex_offset;
    addr_t ObjectTable_offset;
    addr_t TableCode_offset;
    addr_t DeviceObject_offset;
    addr_t Body_offset;
    addr_t Name_offset;
    addr_t FileName_offset;
    addr_t Pid_offset;
    addr_t Cid_offset;
    addr_t KeyControlBlock_offset;
    addr_t NameBlock_offset;
    addr_t Flags_offset;
    addr_t ParentKcb_offset;
    addr_t ControlBlock_Name_offset;
    addr_t NameLength_offset;
} windows_handles_offsets_struct_t;


/* request CAPSTONE library */
#define magic 16
#define layer_object_maximum_address (281474976710655L)
#define obj_name_max_length 1024

/**
 * windows_handles_monitor
 * Lists process handle for a single _EPROCESS
 * @param[in]  apsh
 * @param[in]  process list or if set NULL, then pstree will internally get process list
 * @return APSH status_t defined in APSH
 */
status_t windows_handles_monitor(apsh_instance_t apsh, windows_processes_list_t *process_list);

/**
 * windows_handles
 * Lists process open handles.
 * @param[in]  apsh
 * @param[in]  process_list list or if set NULL, then pstree will internally get process list
 * @param[in]  pid process-id number
 * @param[out] handles results stored in handles struct
 * @return APSH status_t defined in APSH
 */
status_t windows_handles(apsh_instance_t apsh, windows_processes_list_t *process_list, uint32_t pid,
			 windows_handles_t **handles);

/**
 * print out handles output json-string on console
 * @param[in] A string holding pslist output in json format, see pslist
 */
void windows_handles_print_out(char *jsonstr);

void windows_free_handles(windows_handles_t *handles);

#endif /* __WINDOWS_HANDLES_H__ */
