/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __WINDOWS_HASHTEST_H__
#define __WINDOWS_HASHTEST_H__
#include "vmm.h"
#include "pslist.h"
#include "../../../core/APSH/apsh_instance.h"

#define MAX_FILENAME 256

struct file_entry;

typedef struct _WINDOWS_HASHTEST_LIST {
	struct file_entry *file_hashes;
	uint32_t pid;
	char *hash_file;
	GSList *results_list;
} windows_hashtest_t;

/**
 * windows_hashtest
 * Attests a windows process by comparing its memory pages to a pre-computed hash file values
 * @param[in]  apsh apsh instance for host/guest
 * @param[in]  process a process to attest, if set NULL, then internally get process list, and search for pid
 * @param[in]  pid pid of process to attest
 * @param[in]  results_list list to be filled with GHashTables. if set NULL resutls will be printed out
 * @return apsh status_t defined in apsh
 */

status_t windows_hashtest(apsh_instance_t apsh, windows_processes_list_t *process, uint32_t pid, const char *hash_file, windows_hashtest_t **results_list);
status_t windows_hashtest_refresh(apsh_instance_t apsh, windows_processes_list_t *process, windows_hashtest_t **results_list);
void windows_free_hashtest_results(windows_hashtest_t *results_list);


#endif /* __WINDOWS_HASHTEST_H__ */
