/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */
#ifndef __WINDOWS_LDRMODULES_H__
#define __WINDOWS_LDRMODULES_H__

#include <stdbool.h>

#include "vmm.h"
#include "pslist.h"

typedef struct _WINDOWS_LDRMODULES_LIST {
	apsh_pid_t pid;
	char *procname;
	addr_t address;
	addr_t entry;
	bool inload;
	bool inmem;
	bool ininit;
	char *base_dll_name;
	char *full_dll_name;
	GHashTable *hash;
	struct _WINDOWS_LDRMODULES_LIST *next;
} windows_ldrmodules_list_t;


typedef struct _WINDOWS_Dll_LIST
{
    addr_t entry;
    struct _WINDOWS_Dll_LIST *next;
} windows_dll_list_t;

status_t windows_ldrmodules(apsh_instance_t apsh, windows_processes_list_t *process_list, apsh_pid_t pid,
			    windows_ldrmodules_list_t **pldrmodules_list);

status_t windows_ldrmodules_inloadlist(apsh_instance_t apsh, apsh_pid_t pid, addr_t process_address,
				       windows_dll_list_t **dll_list);

status_t windows_ldrmodules_inmemlist(apsh_instance_t apsh, apsh_pid_t pid, addr_t process_address,
				      windows_dll_list_t **dll_list);

status_t windows_ldrmodules_ininitlist(apsh_instance_t apsh, apsh_pid_t pid, addr_t process_address,
				       windows_dll_list_t **dll_list);

void windows_free_ldrmodules_list(windows_ldrmodules_list_t *ldrmodules_list);

void windows_free_dll_list(windows_dll_list_t *dll_list);

#endif /* __WINDOWS_LDRMODULES_H__ */
