/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __WINDOWS_PSLIST_H__
#define __WINDOWS_PSLIST_H__

#include "vmm.h"

typedef struct _WINDOWS_PROCESSES_LIST
{
    uint32_t pid;
    uint32_t ppid;
    uint32_t state;
    char *name;
    addr_t address;
    addr_t exit_time;
    uint32_t threads_count;
    uint64_t cpu_time;
    GHashTable* hash;
    struct _WINDOWS_PROCESSES_LIST *next;
} windows_processes_list_t;

typedef struct _WINDOWS_EPROCESS_struct
{
    addr_t ImageFileName_offset;
    addr_t ActiveThreads_offset;
    addr_t ObjectTable_offset;
    addr_t Session_offset;
    addr_t InheritedFromUniqueProcessId_offset;
    addr_t WoW64Process_offset;
    addr_t CreateTime_offset;
    addr_t ExitTime_offset;
    addr_t HandleCount_offset;
    addr_t Peb_offset;
    bool has_handlecount;

} windows_eprocess_struct_t;

/**
 * windows_pslist
 * Lists the processes present in a particular windows memory image.
 * @param[in]  apsh  apsh instance for host/guest
 * @param[in]  process list or if set NULL, then pslist internally get process list
 * @param[out] json_obj results stored in json_object
 * @param[in]  print out on console
 * @return APSH status_t defined in APSH
 */
status_t windows_pslist(apsh_instance_t apsh, windows_processes_list_t *process_list, json_object **json_obj, bool printout);

/**
 * pslist_process_list
 * get processes list
 * @param[in]  apsh  apsh instance for host/guest
 * @param[out] process list
 * @return APSH status_t defined in APSH
 */
status_t windows_pslist_processes_list(apsh_instance_t apsh, windows_processes_list_t **pprocess_list);

/**
 * free process list
 */
void windows_free_process_list(windows_processes_list_t *process_list);

/**
 * get process from process list by given pid
 * @param[in]  process list
 * @param[in]  process id
 * @return pointer to demanded process, or NULL
*/
windows_processes_list_t *windows_pslist_get_process(windows_processes_list_t *process_list, uint32_t pid);

/**
 * init _EPROCESS struct
 * @param[in]  apsh  apsh instance for host/guest
 * @param[in]  eprocess structure
 * @return status_t
*/
status_t windows_pslist_init_eprocess_struct(apsh_instance_t apsh, windows_eprocess_struct_t *eprocess_struct);


/**
 * lookup _EPROCESS struct, create and initialize one if needed
 * @param[in]  apsh  apsh instance for host/guest
 * @param[out]  eprocess structure
 * @return status_t
*/
status_t windows_get_eprocess(apsh_instance_t apsh, windows_eprocess_struct_t **eps);




#endif /* __WINDOWS_PSLIST_H__ */
