/*
 * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __WINDOWS_VADINFO_H__
#define __WINDOWS_VADINFO_H__

#include "vmm.h"
#include "pslist.h"

#define MAX_PATH 260
#define BIT_MASK(__TYPE__, __ONE_COUNT__) \
    ((__TYPE__)(-((__ONE_COUNT__) != 0))) & (((__TYPE__)-1) >> ((sizeof(__TYPE__) * CHAR_BIT) - (__ONE_COUNT__)))

struct EPROCESS
{
    addr_t ActiveProcessLinks;
    addr_t ImageFileName;
    addr_t UniqueProcessId;
};

struct windows_PsList
{
    addr_t PsActiveProcessHead;
    addr_t current_entry;
    addr_t next_entry;

};

struct _FILE_OBJECT
{
    addr_t FileName_rva;
};

typedef struct _RTL_BALANCED_NODE rtl_balanced_node_t, *rtl_balanced_node_p;

typedef struct _RTL_BALANCED_NODE
{
    addr_t Left;          // pointer tp rtl_balanced_node_t
    addr_t Right;         // pointer to rtl_balanced_node_t
    uint64_t ParentValue; // offset: 16, see Balance
    /*
       Balance
            "bit_length": 2,
            "bit_position": 0,
    */

} rtl_balanced_node_t;

typedef struct _MMVAD_SHORT
{

    rtl_balanced_node_t VadNode; // offset : 0  size : 24
    uint32_t StartingVpn;        // offset: 24, size : 4
    uint32_t EndingVpn;          // offset: 28, size : 4
    uint8_t StartingVpnHigh;     // offset: 32, size : 1
    uint8_t EndingVpnHigh;       // offset: 33, size : 1
    uint8_t CommitChargeHigh;    // offset: 34  size : 1
    uint8_t SpareNT64VadUChar;
    uint32_t ReferenceCount; // offset : 36  size : 4
    uint64_t PushLock;       // offset: 40  size : 8  ,  see _EX_PUSH_LOCK
    uint32_t u;              // offset : 48  , see union __unnamed_17b2
    uint32_t u1;             // offset : 52, see union __unnamed_17b5
    uint64_t EventList;      // offset : 56, see _MI_VAD_EVENT_BLOCK

} mmvad_short_t;

struct _MMVAD_FLAG
{
    addr_t protection_offset;
    size_t protection_start_bit;
    size_t protection_end_bit;
    addr_t commit_charge_offset;
    size_t commit_charge_start_bit;
    size_t commit_charge_end_bit;
    addr_t private_memory_offset;
    size_t private_memory_start_bit;
    size_t private_memory_end_bit;
};

typedef struct _VADINFO
{

    char tag[8];
    char *procname;
    addr_t procaddr;
    uint32_t pid;
    addr_t startvpn;
    addr_t endvpn;
    char filepath[MAX_PATH + 1];
    char protection[MAX_PATH];

} vadinfo_t;

struct _MMVAD
{
    addr_t subsection_rva;
};

struct _SUBSECTION
{
    addr_t control_area_rva;
};

struct _CONTROL_AREA
{
    addr_t file_pointer_rva;
};


#define TARGET_PID 9148

typedef struct
{
    char name[32];
    uint32_t mask;
} protection_t;

#define MmProtectToValueSIZE 32

struct vadinfo_struct {
	addr_t ImageFileName_offset;
	addr_t vadroot_offset;
	struct _FILE_OBJECT file_object;
	struct _MMVAD_FLAG mmvad_flag;
	struct _MMVAD mmvad;
	struct _SUBSECTION subsection;
	struct _CONTROL_AREA control_area;
	int MmProtectToValue[MmProtectToValueSIZE];
};

typedef struct WINDOWS_VAD
{
    vadinfo_t vadinfo;
    struct _MMVAD_FLAG mmvad_flag;
    GHashTable* hash;
    struct WINDOWS_VAD *next;
    struct WINDOWS_VAD *tail;
} windows_vad_t;

/**
 * windows_vadinfo
 * Lists vad information for process
 * @param[in]  apsh
 * @param[in]  process list or if set NULL, then pstree will internally get process list
 * @param[out] json_obj results stored in json_object
 * @param[in]  print out on console
 * @return APSH status_t defined in APSH
 */
status_t windows_vadinfo(apsh_instance_t apsh, windows_processes_list_t *process_list, uint32_t pid, json_object **json_obj, bool printout);

status_t windows_get_vadlist(apsh_instance_t apsh, windows_processes_list_t *process_list, uint32_t pid, windows_vad_t **vad_list);

void windows_free_vad_list(windows_vad_t *vad_list);

status_t vadinfo_struct_init(apsh_instance_t apsh, struct vadinfo_struct *vad_struct);

#endif /* __WINDOWS_VADINFO_H__ */
