/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __CC_DEVX_H__
#define __CC_DEVX_H__

#include <stdint.h>
#include <math.h>
#include <doca_dev.h>
#include <priv_doca_dev.h>
#include <priv_doca_os_abstraction.hpp>
#include <multi_os_prm.h>

struct qp_params_reg_mr {
	size_t size;
	uint32_t pd : 24;
	uint32_t mkey_7_0 : 8;
	uint32_t qpn : 24;
	uint32_t access_mode_1_0 : 2;
	uint32_t lw : 1;
	uint32_t lr : 1;
	uint32_t pg_access : 1;
};
struct qp_params_init {
	uint32_t qp : 24;
	uint32_t vhca_port_num : 8;
};

struct qp_params_rtr {
	uint32_t qp : 24;
	uint32_t mtu : 3;
	uint32_t log_msg_max : 5;
	uint32_t dctn : 24;
	uint32_t hop_limit : 8;
};

struct qp_params_rts {
	uint32_t qp : 24;
	uint32_t log_ack_req_freq : 4;
	uint32_t retry_count : 3;
	uint32_t rnr_retry : 3;
	uint32_t ack_timeout : 5;
};

struct qp_params_set_ud_addr {
	uint32_t dest_qpn : 24;
	uint32_t src_addr_index : 8;
	uint32_t rlid : 16;
	uint32_t ext : 1;
};

struct cc_qp_cap {
	uint32_t log_max_msg : 5;
	uint32_t log_max_queue_sz : 8;
	uint32_t num_ports : 8;
	uint32_t num_vhca_ports : 8;
	uint32_t port_type : 2;
	uint32_t log_max_connections : 5;
};

struct devx_context {
	struct ibv_context *ctx;
	struct doca_dev *doca_device;
};

struct cc_q_obj {
	struct doca_devx_object *q_object;
	struct doca_umem_object *wq_mem;
	struct doca_umem_object *dbrm;
	os_abstraction::aligned_buffer<uint8_t> *wq_buff;
	os_abstraction::aligned_buffer<uint32_t> *dbr;
};

struct cc_cq_obj {
	struct priv_doca_cq *cq_q;
	uint32_t cqn;
	bool is_armed;
	struct priv_doca_comp_events_channel *events_channel; /** Need this to arm the CQ */
};

struct channel_params {
	uint32_t dci_dct_num;
	uint32_t qkey;
	uint32_t vhca_id : 16;
	uint32_t flag : 3;
	uint32_t transport_type : 3;
	uint32_t max_num_connected_client : 16;
	const char *service_name;
};

static inline int log2_by_bits(uint32_t num)
{
	return (int)ceil(log2(num));
}

void arm_cq(struct cc_cq_obj *cq);

void force_arm(struct cc_cq_obj *cq);

int devx_reg_mr(struct devx_context *devx_ctx, void *buff, const struct qp_params_reg_mr *params,
		struct doca_devx_object **mr_obj, struct doca_umem_object **umem);

int cc_q_obj_destroy(struct cc_q_obj *q_obj);

int create_dci(struct devx_context *devx_ctx, struct priv_doca_uar *uar, int cqn, int pd, size_t log_sq_size,
	       struct cc_q_obj *q);

int create_rmp(struct devx_context *devx_ctx, int pd, int log_rq_size, struct cc_q_obj *rmp_obj);

int create_dct(struct devx_context *devx_ctx, int cqn, int pd, uint16_t port_num, int srqn, struct cc_q_obj *dct_obj);

int get_issi(struct devx_context *devx_ctx);

int devx_to_init(struct doca_devx_object *obj, const struct qp_params_init *params);

int devx_to_rtr(struct doca_devx_object *obj, const struct qp_params_rtr *params);

int devx_to_rts(struct doca_devx_object *obj, const struct qp_params_rts *params);

int to_rst(struct doca_devx_object *obj, int qpn);

void devx_set_ud_addr(uint8_t *ud_addr, const struct qp_params_set_ud_addr *params);

void update_wqe_ctrl_index(uint8_t *wqe_ptr, uint32_t value);

/*
 * cap - output
 * Return < 0 on error
 */
int devx_query_capabilities(const struct devx_context *devx_ctx, struct cc_qp_cap *cap);

struct doca_devx_object *register_channel_connection(struct devx_context *devx_ctx,
						     const struct channel_params *params);

struct doca_devx_object *register_channel_service(struct devx_context *devx_ctx,
						  const struct channel_params *params);

#endif /* __CC_DEVX_H__ */
