/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <doca_log.h>

#include "doca_dpi_grpc_handler.hpp"

/* Declares a proceed method for a handler.
 * "ACTION" should be the handler prefix before "RpcHandler".
 */
#define DECLARE_PROCEED(ACTION) \
	void ACTION##RpcHandler::Proceed()				\
	{								\
		Status stat;						\
		switch (status_) {					\
		case PROCESS:						\
			DOCA_DLOG_DBG(#ACTION" request received");					\
			new ACTION##RpcHandler(static_cast<DocaDpiRpcHandler&>(*this));			\
			stat = this->dpi_adapter_->DocaDpi##ACTION(&this->ctx_, &request_, &reply_);	\
			/* If you allocate more than 1 thread/lcore per completion queue:		\
			 * Don't uncomment the GPR_ASSERT line and don't replace the line order of the	\
			 * call to "responder_.Finish" with the line that sets "status_ = FINISH".	\
			 * The call to "responder_.Finish" releases a gRPC lock on this handler and	\
			 * if the status doesn't change another thread might try to process this	\
			 * handler as well, which enters gRPC to an invalid state and then gRPC		\
			 * aborts the server.								\
			 * Using the commentted out GPR_ASSERT will fail because a different thread	\
			 * might deletes the handler by the time the "responder_.Finish" finishes	\
			 * and "status_" will contains garbage. */	\
			status_ = FINISH;				\
			responder_.Finish(reply_, stat, this);		\
			/* GPR_ASSERT(status_ == FINISH); */		\
			break;						\
		case FINISH:						\
			DOCA_DLOG_DBG(#ACTION" request processed & finished");				\
			delete this;									\
			/* Should not get here the destructor is called. */				\
			break;										\
		default:										\
			DOCA_LOG_WARN("Got default state for "#ACTION"RpcHandler");			\
			break;						\
		}							\
	}								\

DOCA_LOG_REGISTER(DOCA_DPI_SERVER_RPC_HANDLER);

using grpc::Status;

void DocaDpiRpcHandler::Proceed()
{
	DOCA_DLOG_CRIT("This class is only used to init the handlers and should not be proceeded.");
}

DECLARE_PROCEED(Init);
DECLARE_PROCEED(Destroy);
DECLARE_PROCEED(LoadSignatures);
DECLARE_PROCEED(Enqueue);
DECLARE_PROCEED(Dequeue);
DECLARE_PROCEED(FlowCreate);
DECLARE_PROCEED(FlowDestroy);
DECLARE_PROCEED(FlowMatchGet);
DECLARE_PROCEED(SignatureGet);
DECLARE_PROCEED(SignaturesGet);
DECLARE_PROCEED(StatGet);
