/*
 * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef SERVER_CORE_H_
#define SERVER_CORE_H_

#include <grpcpp/grpcpp.h>

#include <doca_error.h>

#include "doca_dpi.grpc.pb.h"
#include "doca_dpi.pb.h"
#include "sync_map.hpp"

void run_server(uint16_t num_of_dpdk_queues, const char *raw_server_address);

class DocaDpiImpl : public DocaDpi::Service
{
	private:
		struct sync_map doca_dpi_flow_ctx_pool;
		uint16_t nb_microservices;

		DocaDpiErrorInfo *generate_error_info(int error_code, DocaDpiErrorInfo *inplace_buf = NULL);

	public:
		static doca_error_t InitServerEnv(void);
		DocaDpiImpl(uint16_t nb_microservices) : nb_microservices(nb_microservices){};
		~DocaDpiImpl();
		grpc::Status DocaDpiInit(grpc::ServerContext *context, const DocaDpiInitParams *request,
			       DocaDpiInitResponse *response) override;
		grpc::Status DocaDpiDestroy(grpc::ServerContext *context, const DocaDpiDestroyParams *request,
			       DocaDpiDestroyResponse *response) override;
		grpc::Status DocaDpiLoadSignatures(grpc::ServerContext *context, const DocaDpiLoadSignaturesParams *request,
				      DocaDpiErrorInfo *response) override;
		grpc::Status DocaDpiEnqueue(grpc::ServerContext *context, const DocaDpiEnqueueParams *request,
			       DocaDpiErrorInfo *response) override;
		grpc::Status DocaDpiDequeue(grpc::ServerContext *context, const DocaDpiDequeueParams *request,
			       DocaDpiActionResponse *response) override;
		grpc::Status DocaDpiFlowCreate(grpc::ServerContext *context, const DocaDpiFlowCreateParams *request,
				  DocaDpiFlowCreateResponse *response) override;
		grpc::Status DocaDpiFlowDestroy(grpc::ServerContext *context, const DocaDpiFlowDestroyParams *request,
				   DocaDpiFlowDestroyResponse *response) override;
		grpc::Status DocaDpiFlowMatchGet(grpc::ServerContext *context, const DocaDpiFlowMatchGetParams *request,
				    DocaDpiActionResponse *response) override;
		grpc::Status DocaDpiSignatureGet(grpc::ServerContext *context, const DocaDpiSignatureGetParams *request,
				    DocaDpiSignatureGetResponse *response) override;
		grpc::Status DocaDpiSignaturesGet(grpc::ServerContext *context, const DocaDpiSignaturesGetParams *request,
				     DocaDpiSignaturesGetResponse *response) override;
		grpc::Status DocaDpiStatGet(grpc::ServerContext *context, const DocaDpiStatGetParams *request,
			       DocaDpiStatInfo *response) override;
};


#endif /* SERVER_CORE_H_ */
