/*
 * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include "sync_map.hpp"

void
sync_map_add(struct sync_map *obj, uint64_t key, void *value)
{
	obj->lock.lock();
	obj->hash_table[key] = value;
	obj->lock.unlock();
}

void *
sync_map_remove(struct sync_map *obj, uint64_t key)
{
	void *value;
	obj->lock.lock();
	if (obj->hash_table.find(key) == obj->hash_table.end()) {
		obj->lock.unlock();
		return NULL;
	}
	value = obj->hash_table[key];
	obj->hash_table.erase(key);
	obj->lock.unlock();
	return value;
}

void *
sync_map_get(struct sync_map *obj, uint64_t key)
{
	void *value;
	obj->lock.lock();
	if (obj->hash_table.find(key) == obj->hash_table.end()) {
		obj->lock.unlock();
		return NULL;
	}
	value = obj->hash_table[key];
	obj->lock.unlock();
	return value;
}
