/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPI_REGEX_H_
#define DPI_REGEX_H_

#include <rte_regexdev.h>

struct dpi_regex_dev_ctx;
struct dpi_regex_queue_ctx;

struct dpi_regex_result {
	const struct rte_regex_ops *op;
	void *user_ptr;
};

enum {
	DPI_REGEX_TYPE_HS,
	DPI_REGEX_TYPE_RXP
};

struct dpi_sw_queue_entry;

struct dpi_regex_dev_ctx *dpi_regex_dev_alloc(int dpdk_dev_id, int num_queues, int *error);
void dpi_regex_dev_dealloc(struct dpi_regex_dev_ctx *ctx);

int dpi_regex_load_sig(struct dpi_regex_dev_ctx *ctx, int db_type, void *db, size_t len);

struct dpi_regex_queue_ctx *
dpi_regex_queue_alloc(struct dpi_regex_dev_ctx *ctx, int hw_queue_id, uint16_t queue_sz);
void dpi_regex_queue_dealloc(struct dpi_regex_queue_ctx *ctx);
int dpi_regex_queue_set_hs_db(struct dpi_regex_queue_ctx *ctx, void *hs_db);
int dpi_regex_enqueue_mbuf(struct dpi_regex_queue_ctx *ctx, struct rte_mbuf *mbuf,
			   int groups[4], void *user_ptr, char *meta);
int dpi_regex_enqueue_buf(struct dpi_regex_queue_ctx *ctx, const void *buf, uint16_t len,
			  int groups[4], void *user_ptr, char *meta);
int dpi_regex_dequeue(struct dpi_regex_queue_ctx *ctx, struct dpi_regex_result *result);

#endif /* DPI_REGEX_H_ */
