/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef BASE_PARSER_H_
#define BASE_PARSER_H_

#include <rte_mbuf.h>

#include "dpi_parser_types.h"

#ifdef __cplusplus
extern "C" {
#endif

struct dpi_parser {
	int (*ctx_alloc)(void **ctx);
	void (*ctx_dealloc)(void *ctx);

	/*
	 * Parse new flow, or continue parsing prev flow.
	 * Parser should be able to store the parsing state on the ctx.
	 * Parser should store the mbuf pointer for field retrival in the
	 * future.
	 * returns the parsing state on success.
	 * Offset represents the L4 payload start.
	 */

	int (*parse)(void *ctx, struct rte_mbuf *mbuf,
		     field_cb_t field_cb,
		     void *user_data,
		     uint32_t *consumed);
	/**
	 * Set peer parser
	 */
	void (*peer_set)(void *ctx, void *peer_ctx);

	/**
	 * Support Gather list.
	 * Means the parser can accept scattered input in mbuf format
	 */
	bool (*support_mbuf)(void *ctx);
};

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* BASE_PARSER_H_ */
