/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPI_PARSER_H_
#define DPI_PARSER_H_

#include <rte_mbuf.h>

#include "dpi_parser_types.h"

#ifdef __cplusplus
extern "C" {
#endif

struct dpi_parser_ctx;
struct dns_cache;
struct ftp_cache;

struct dpi_parser_main_ctx;

struct dpi_parser_main_ctx *dpi_parser_init(uint32_t max_sig_len);
void dpi_parser_destroy(struct dpi_parser_main_ctx *ctx);

int dpi_parser_ctx_alloc(struct dpi_parser_main_ctx *main_ctx, struct dpi_parser_ctx **pctx1,
			 struct dpi_parser_ctx **pctx2, uint16_t port, int l4_type);
void dpi_parser_ctx_dealloc(struct dpi_parser_ctx *ctx);

/**
 * Parse new flow, or continue parsing existing flow.
 * Parser should be able to store the parsing state on the ctx.
 * Parser should store the mbuf pointer for field retrival in the future.
 * returns the parsing state on success.
 * Offset represents the L4 payload start in the current packet.
 */
uint32_t dpi_parser_parse(int dpi_q,
			  struct dpi_parser_ctx *ctx, struct rte_mbuf *mbuf_chain,
			  uint32_t first_mbuf_offset, field_cb_t field_cb, void *user_data);

int dpi_parser_update_dns_cache(struct rte_mbuf *mbuf, uint32_t payload_offset,
				struct dns_cache *dns_cache, int sig_id);

int dpi_parser_update_ftp_cache(struct rte_mbuf *mbuf, uint32_t payload_offset,
				struct ftp_cache *ftp_cache, int sig_id);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPI_PARSER_H_ */
