/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPI_PARSER_TYPES_H_
#define DPI_PARSER_TYPES_H_

#include <stdbool.h>

#include <private/common_defines.h>

#ifdef __cplusplus
extern "C" {
#endif

#define EMPTY_FIELD 24

enum iana_table_t {
	DPI_IANA_TCP = 6,
	DPI_IANA_UDP = 17,
	DPI_IANA_ICMP = 1
};

enum dpi_parser_protocol {
	DPI_PARSER_ICMP = DPI_IANA_ICMP,
	DPI_PARSER_TCP = DPI_IANA_TCP,
	DPI_PARSER_UDP = DPI_IANA_UDP,
	DPI_PARSER_HTTP_1,
	DPI_PARSER_HTTP_2,
	DPI_PARSER_RTMP,
	DPI_PARSER_FTP,
	DPI_PARSER_SSL,
	DPI_PARSER_SIP,
	DPI_PARSER_QUIC,
	DPI_PARSER_STUN,
	DPI_PARSER_DNS,
	DPI_PARSER_IP,
	DPI_PARSER_IP6,
	DPI_PARSER_OTHER_L4,
	DPI_PARSER_MAX_PROTOCOL, /* add new protocols above here */
	DPI_PARSER_PROTOCOL_NOT_READY, /* adding certain protocols only above */
	DPI_PARSER_PROTOCOL_MISS
};

enum context_http_fields {
	HTTP_URI,
	HTTP_URI_NORM,
	HTTP_SERVER,
	HTTP_MIME_TYPE,
	HTTP_USER_AGENT,
	HTTP_REFERER,
	HTTP_SERVER_AGENT,
	HTTP_LOCATION,
	HTTP_COOKIE,
	HTTP_CONTENT,
	HTTP_CONTENT_LENGTH,
	HTTP_MAX_ENTRIES /* keep this entry last */
};

enum context_ssl_fields {
	SSL_SERVER_NAME,
	SSL_COMMON_NAME,
	SSL_SUBJECT_ALT_NAME,
	SSL_CERT_SUBJECT,
	SSL_MAX_ENTRIES /* keep this entry last */
};

enum context_rtmp_fields {
	RTMP_PAGE_URL,
	RTMP_MAX_ENTRIES /* keep this entry last */
};

enum context_tcp_fields {
	TCP_STREAM,
	TCP_DPORT,
	TCP_MAX_ENTRIES /* keep this entry last */
};

enum context_udp_fields {
	UDP_STREAM,
	UDP_DPORT,
	UDP_MAX_ENTRIES /* keep this entry last */
};

enum context_other_l4_fields {
	OTHER_L4_STREAM,
	OTHER_L4_MAX_ENTRIES /* keep this entry last */
};

enum context_ip_fields {
	IP_DEST,
	IP_MAX_ENTRIES /* keep this entry last */
};

enum context_quic_fields {
	QUIC_SERVER_NAME,
	QUIC_USER_AGENT,
	QUIC_MAX_ENTRIES /* keep this entry last */
};

enum context_dns_fields {
	DNS_QUERY,
	DNS_RESOLVE_NAME,
	DNS_MAX_ENTRIES /* keep this entry last */
};

enum context_stun_fields {
	STUN_REALM,
	STUN_SOFTWARE,
	STUN_MAX_ENTRIES /* keep this entry last */
};

enum context_sip_fields {
	SIP_USER_AGENT,
	SIP_MAX_ENTRIES /* keep this entry last */
};

enum dpi_parser_state {
	PARSER_STATE_PART, /* Some fields may be already parsed. */
	PARSER_STATE_PROT_VERIFIED,
	PARSER_STATE_DONE, /* TBD, when are we done? */
	PARSER_STATE_ERROR /* Did not recognize protocol. */
};

struct dpi_parser_status {
	int protocol; /* -1 means invalid */
	int field; /* -1 means invalid */
	uint32_t start;
	uint32_t end; /* is invalid */
	uint32_t session_id_start;
	uint32_t session_id_end; /* is invalid */
	uint32_t payload_offset; /* position in current mbuf */
};

struct dpi_parser_cb_metadata {
	int field;
	int prot;
	bool stream_start;
	bool mbuf;
};

typedef void (*field_cb_t)(void *user_data, void *start_or_mbuf, uint16_t len,
			   const struct dpi_parser_cb_metadata *metadata);

#ifdef __cplusplus
}
#endif

#endif /* DPI_PARSER_TYPES_H_ */
