/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <thread>
#include <rte_lcore.h>

#include <doca_log.h>

#include "flow_server_core.hpp"

DOCA_LOG_REGISTER(doca_flow::GRPC::Core);

void DocaFlowMicroService::InitMicroService(std::string server_address_)
{
        ServerBuilder builder;

        builder.AddListeningPort(server_address_, grpc::InsecureServerCredentials());
        builder.RegisterService(&service_);
        builder.RegisterService(&orchestration_service);

        cq_ = builder.AddCompletionQueue();
        server_ = builder.BuildAndStart();
        server_address = std::string(server_address_);
}

void DocaFlowMicroService::RequestStartReceiving()
{
        new InitRpcHandler(&service_, cq_.get(), server_params);
        new DestroyRpcHandler(&service_, cq_.get(), server_params);
        new PortStartRpcHandler(&service_, cq_.get(), server_params);
        new PortStopRpcHandler(&service_, cq_.get(), server_params);
        new PortPairRpcHandler(&service_, cq_.get(), server_params);
        new PortPrivDataRpcHandler(&service_, cq_.get(), server_params);
        new SharedResourceCfgRpcHandler(&service_, cq_.get(), server_params);
        new SharedResourcesBindRpcHandler(&service_, cq_.get(), server_params);
        new PipeCreateRpcHandler(&service_, cq_.get(), server_params);
        new SharedResourcesQueryRpcHandler(&service_, cq_.get(), server_params);
        new PipeAddEntryRpcHandler(&service_, cq_.get(), server_params);
        new PipeControlAddEntryRpcHandler(&service_, cq_.get(), server_params);
        new PipeLpmAddEntryRpcHandler(&service_, cq_.get(), server_params);
        new PipeRmEntryRpcHandler(&service_, cq_.get(), server_params);
        new PipeDestroyRpcHandler(&service_, cq_.get(), server_params);
        new PortPipesFlushRpcHandler(&service_, cq_.get(), server_params);
        new PortDestroyRpcHandler(&service_, cq_.get(), server_params);
        new PortPipesDumpRpcHandler(&service_, cq_.get(), server_params);
        new QueryRpcHandler(&service_, cq_.get(), server_params);
        new AgingHandleRpcHandler(&service_, cq_.get(), server_params);
        new EntriesProcessRpcHandler(&service_, cq_.get(), server_params);
        new PipeEntryGetStatusRpcHandler(&service_, cq_.get(), server_params);
        new PortSwitchGetRpcHandler(&service_, cq_.get(), server_params);
}

int DocaFlowMicroService::HandleRpcs()
{
        DOCA_LOG_DBG("Doca Flow MicroServer %d is listening on address %s", server_id, server_address.c_str());
        RequestStartReceiving();
        void* tag;
        bool ok = false;

        while (cq_->Next(&tag, &ok)) {
                if (ok) {
                        DocaFlowRpcHandler* call = static_cast<DocaFlowRpcHandler*>(tag);
                        call->Proceed();
                }
        }

        DOCA_LOG_DBG("Server finished handling Doca Flow RPCs");
        return 1;
}

void DocaFlowMicroService::Shutdown()
{
        /* Always shutdown the completion queues after the server. */
        server_->Shutdown();
        cq_->Shutdown();
}
