/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */
#ifndef FLOW_SERVER_CORE_H_
#define FLOW_SERVER_CORE_H_
#include <common.pb.h>

#include "flow_grpc_handler.hpp"
#include "orchestration.hpp"

using namespace std;
using grpc::Server;
using grpc::ServerBuilder;
using grpc::ServerCompletionQueue;
using grpc::ServerContext;
using grpc::Status;
using grpc::Channel;

class MicroService {
        protected:
                int server_id;
                std::string server_address;
                DocaOrchestrationImpl orchestration_service;
                std::unique_ptr<Server> server_;
                std::unique_ptr<ServerCompletionQueue> cq_;
                DocaFlowServerParams *server_params;

        public:
                MicroService (unsigned int id, DocaFlowServerParams *params) : server_id(id) , server_params(params) {};
                virtual void InitMicroService (std::string server_address_) = 0;
                virtual void RequestStartReceiving() = 0;
                virtual int HandleRpcs() = 0;
                virtual void Shutdown() = 0;
                virtual ~MicroService() {};
};

class DocaFlowMicroService : public MicroService{
	DocaFlow::AsyncService service_;

	public:

		DocaFlowMicroService (unsigned int id, DocaFlowServerParams *params) : MicroService(id, params) {}

		virtual void InitMicroService(std::string server_address_) override;

		virtual void RequestStartReceiving() override;

		virtual int HandleRpcs() override;

		virtual void Shutdown() override;

                virtual ~DocaFlowMicroService() {};
};

#endif /* FLOW_SERVER_CORE_H_ */
