/*
 * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */
#ifndef FLOW_SERVER_PIPE_MANAGER_H_
#define FLOW_SERVER_PIPE_MANAGER_H_

#include <unordered_map>
#include <utility>

#include <doca_error.h>
#include <doca_flow.h>

using namespace std;

class PipesManager {

	unordered_map<uint64_t, tuple<struct doca_flow_pipe *, unordered_map<uint64_t, uint64_t> &,
						unordered_map<uint64_t, struct doca_flow_pipe_entry *>>> pipe_id_to_tuple_map;

	unordered_map <uint64_t , uint64_t > entry_id_to_pipe_id_map;

	unordered_map<uint16_t, unordered_map<uint64_t, uint64_t>> port_id_to_pipes_id_map;

	uint64_t private_key;

	/* generates a unique id for each pointer */
	uint64_t generateID(void *ptr) const;
public:
	PipesManager();

	/* prepare the pipes manager for ID generation */
	doca_error_t init();

	/* saves the given pipe that is attached to the given port and returns pipe's id or 0 if failed */
	uint64_t addPipe(struct doca_flow_pipe *pipe, uint16_t port_id);

	/* saves the given entry in the hash table of the given pipe id and returns entry's id or 0 if failed */
	uint64_t addPipeEntry(struct doca_flow_pipe* pipe, struct doca_flow_pipe_entry *entry);

	/* get a pointer of the given pipe id  */
	struct doca_flow_pipe *getPipe(uint64_t pipe_id);

	/* get a pointer of the given entry id that is attached to the given port id */
	struct doca_flow_pipe_entry *getPipeEntry(uint64_t entry_id);

	/* removes pipe and all its entries */
	bool removePipe(uint64_t pipe_id);

	/* removes entry of the given pipe id */
	bool removePipeEntry(uint64_t entry_id);

	/* removes all pipes and entries for a single port id. */
	bool portPipesFlush(uint16_t port_id);

	/* returns entry id */
	uint64_t getEntryId(struct doca_flow_pipe_entry *entry);
};

#endif /* DOCA_FLOW_SERVER_PIPE_MANAGER_H_ */
