/*
 * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef LIBS_DOCA_REGEX_IMPL_REGEX_DEFINES_H_
#define LIBS_DOCA_REGEX_IMPL_REGEX_DEFINES_H_


#include <stdlib.h>

/** define the max entries per QP - for now until we have a better way - should be a multiple of 2
 */
#define REGEX_MAX_PER_QP_NUM_DESCS 1024

/* Opaque flag values */
/** Job processing exceeded the maximum number of primary threads. */
#define REGEX_DEVICE_FLAG_MAX_PRI_THREADS (1ULL << 3)
/** Job processing exceeded the maximum number of secondary threads. */
#define REGEX_DEVICE_FLAG_MAX_SEC_THREADS (1ULL << 4)
/** Job processing exceeded the maximum time limit. */
#define REGEX_DEVICE_FLAG_MAX_LATENCY (1ULL << 5)
/** Job processing exceeded the maximum number of matches. */
#define REGEX_DEVICE_FLAG_MAX_MATCH (1ULL << 6)
/** Job processing exceeded the maximum number of prefixes. */
#define REGEX_DEVICE_FLAG_MAX_PREFIX (1ULL << 7)
/** Job was requested to be a high priority job. */
#define REGEX_DEVICE_FLAG_HPM (1ULL << 8)
/** Job was requested to stop on the first detected match. */
#define REGEX_DEVICE_FLAG_ANYMATCH (1ULL << 9)
/**
 * Leading bytes partially matched a rule which searched the job backwards, but there was not enough data to complete
 * the search. Submitting this job again with additional preceding bytes may lead to a match being found.
 */
#define REGEX_DEVICE_FLAG_PMI_SOJ (1ULL << 13)
/**
 * Trailing bytes partially matched a rule which searched the job forwards, but there was not enough data to complete
 * the search. Submitting this job again with additional following bytes may lead to a match being found.
 */
#define REGEX_DEVICE_FLAG_PMI_EOJ (1ULL << 14)

#define REGEX_DEVICE_FALLBACK_FLAGS \
	(REGEX_DEVICE_FLAG_MAX_PRI_THREADS | REGEX_DEVICE_FLAG_MAX_SEC_THREADS | REGEX_DEVICE_FLAG_MAX_LATENCY | \
	 REGEX_DEVICE_FLAG_MAX_MATCH | REGEX_DEVICE_FLAG_MAX_PREFIX)

static inline uint32_t
make_aligned_size(size_t alignment, size_t size)
{
	return (alignment * ((size / alignment) + (size % alignment == 0 ? 0 : 1)));
}

static inline void *
aligned_calloc(size_t const alignment, size_t const size)
{
	void *const ptr = aligned_alloc(alignment, make_aligned_size(alignment, size));

	if (ptr != NULL)
		memset(ptr, 0, size);

	return ptr;
}

#endif /* LIBS_DOCA_REGEX_IMPL_REGEX_DEFINES_H_ */
