/*
 * Copyright (c) 2011-2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef LIB_DOCA_SHA_BF2_PRM_H_
#define LIB_DOCA_SHA_BF2_PRM_H_

#include <stdio.h>
#include <stdint.h>

/* Helper Macros to parse and edit structs defined in terms of bits */
#define __doca_mlx5_nullp(typ) ((struct doca_mlx5_ifc_##typ##_bits *)0)
#define __doca_mlx5_bit_sz(typ, fld) sizeof(__doca_mlx5_nullp(typ)->fld)
#define __doca_mlx5_bit_off(typ, fld) ((unsigned int)(uintptr_t)(&(__doca_mlx5_nullp(typ)->fld)))
#define __doca_mlx5_dw_bit_off(typ, fld) \
	(32 - __doca_mlx5_bit_sz(typ, fld) - (__doca_mlx5_bit_off(typ, fld) & 0x1f))
#define __doca_mlx5_dw_mask(typ, fld) \
	(__doca_mlx5_mask(typ, fld) << __doca_mlx5_dw_bit_off(typ, fld))
#define __doca_mlx5_dw_off(typ, fld) (__doca_mlx5_bit_off(typ, fld) / 32)
#define __doca_mlx5_64_off(typ, fld) (__doca_mlx5_bit_off(typ, fld) / 64)
#define __doca_mlx5_mask(typ, fld) ((uint32_t)((1ull << __doca_mlx5_bit_sz(typ, fld)) - 1))
#define DOCA_MLX5_BYTE_OFF(typ, fld) (__doca_mlx5_bit_off(typ, fld) / 8)
#define DOCA_MLX5_ADDR_OF(typ, p, fld) ((char *)(p) + DOCA_MLX5_BYTE_OFF(typ, fld))

#define DOCA_MLX5_GET_VOLATILE(typ, p, fld) \
	((be32toh(*((volatile uint32_t *)(p) + __doca_mlx5_dw_off(typ, fld))) >> \
	  __doca_mlx5_dw_bit_off(typ, fld)) & \
	 __doca_mlx5_mask(typ, fld))
#define DOCA_MLX5_GET(typ, p, fld) \
	((be32toh(*((uint32_t *)(p) + __doca_mlx5_dw_off(typ, fld))) >> \
	  __doca_mlx5_dw_bit_off(typ, fld)) & \
	 __doca_mlx5_mask(typ, fld))

#define DOCA_MLX5_ST_SZ_DW(typ) (sizeof(struct doca_mlx5_ifc_##typ##_bits) / 32)
#define DOCA_MLX5_ST_SZ_BYTES(typ) (sizeof(struct doca_mlx5_ifc_##typ##_bits) / 8)

/* insert a value to a struct */
#define DOCA_MLX5_SET(typ, p, fld, v) \
	do { \
		uint32_t _v = v; \
		*((uint32_t *)(p) + __doca_mlx5_dw_off(typ, fld)) = htobe32( \
			(be32toh(*((uint32_t *)(p) + __doca_mlx5_dw_off(typ, fld))) & \
			 (~__doca_mlx5_dw_mask(typ, fld))) | \
			(((_v)&__doca_mlx5_mask(typ, fld)) << __doca_mlx5_dw_bit_off(typ, fld))); \
	} while (0)

#define DOCA_MLX5_ASSERT(exp) \
	do { \
		if (!(exp)) { \
			DOCA_DLOG_ERR("failed assert\n"); \
			exit(0); \
		} \
	} while (0)

#define DOCA_MLX5_SET64(typ, p, fld, v) \
	do { \
		DOCA_MLX5_ASSERT(__doca_mlx5_bit_sz(typ, fld) == 64); \
		*((uint64_t *)(p) + __doca_mlx5_64_off(typ, fld)) = htobe64(v); \
	} while (0)

#define DOCA_MLX5_ALIGN(v, mul) \
	((((v) + (typeof(v))(mul)-1) / ((typeof(v))(mul))) * (typeof(v))(mul))

/* DOCA_MLX5 Identifiers. */
#define DOCA_MLX5_RXP_BF2_IDENTIFIER 0x0

/* CMESG types/defines. */
enum doca_mlx5_cmd_op {
	DOCA_MLX5_CMD_OP_QUERY_HCA_CAP = 0x100,
	DOCA_MLX5_CMD_OP_CREATE_MKEY = 0x200,
	DOCA_MLX5_CMD_OP_CREATE_CQ = 0x400,
	DOCA_MLX5_CMD_OP_CREATE_QP = 0x500,
	DOCA_MLX5_CMD_OP_RST2INIT_QP = 0x502,
	DOCA_MLX5_CMD_OP_INIT2RTR_QP = 0x503,
	DOCA_MLX5_CMD_OP_RTR2RTS_QP = 0x504,
	DOCA_MLX5_CMD_SET_REGEX_PARAMS = 0xb04,
};

#define DOCA_MLX5_ADAPTER_PAGE_SHIFT 12
#define DOCA_MLX5_LOG_RQ_STRIDE_SHIFT 4

enum { DOCA_MLX5_QP_ST_RC = 0x0,
};

enum { DOCA_MLX5_QP_PM_MIGRATED = 0x3,
};

enum {
	DOCA_MLX5_NON_ZERO_RQ = 0x0,
	DOCA_MLX5_SRQ_RQ = 0x1,
	DOCA_MLX5_CRQ_RQ = 0x2,
	DOCA_MLX5_ZERO_LEN_RQ = 0x3,
};

enum doca_mlx5_get_hca_cap_op_mod {
	DOCA_MLX5_GET_HCA_CAP_OP_MOD_GENERAL_DEVICE = 0x0 << 1,
	DOCA_MLX5_GET_HCA_CAP_OP_MOD_ETHERNET_OFFLOAD_CAPS = 0x1 << 1,
	DOCA_MLX5_GET_HCA_CAP_OP_MOD_QOS_CAP = 0xc << 1,
	DOCA_MLX5_GET_HCA_CAP_OP_MOD_ROCE = 0x4 << 1,
	DOCA_MLX5_GET_HCA_CAP_OP_MOD_NIC_FLOW_TABLE = 0x7 << 1,
	DOCA_MLX5_GET_HCA_CAP_OP_MOD_VDPA_EMULATION = 0x13 << 1,
	DOCA_MLX5_GET_HCA_CAP_OP_MOD_GENERAL_DEVICE_2 = 0x20 << 1,
};

enum doca_mlx5_hca_cap_opmod {
	DOCA_MLX5_HCA_CAP_OPMOD_GET_MAX = 0,
	DOCA_MLX5_HCA_CAP_OPMOD_GET_CUR = 1,
};

enum doca_mlx5_mkc_access_mode {
	DOCA_MLX5_MKC_ACCESS_MODE_MTT = 0x1,
	DOCA_MLX5_MKC_ACCESS_MODE_KLM = 0x2,
	DOCA_MLX5_MKC_ACCESS_MODE_KLM_FBS = 0x3,
};

enum doca_mlx5_sqc_state {
	DOCA_MLX5_SQC_STATE_RST = 0x0,
	DOCA_MLX5_SQC_STATE_RDY = 0x1,
	DOCA_MLX5_SQC_STATE_ERR = 0x3,
};

#define DOCA_MLX5_ADAPTER_PAGE_SHIFT 12

/* The supported timestamp formats reported in HCA attributes. */
enum doca_mlx5_hca_cap_timestamp_format {
	DOCA_MLX5_HCA_CAP_TIMESTAMP_FORMAT_FR = 0x0,
	DOCA_MLX5_HCA_CAP_TIMESTAMP_FORMAT_RT = 0x1,
	DOCA_MLX5_HCA_CAP_TIMESTAMP_FORMAT_FR_RT = 0x2,
};

/* The timestamp format attributes to configure queues (RQ/SQ/QP). */
enum doca_mlx5_qpc_timestamp_format {
	DOCA_MLX5_QPC_TIMESTAMP_FORMAT_FREE_RUNNING = 0x0,
	DOCA_MLX5_QPC_TIMESTAMP_FORMAT_DEFAULT = 0x1,
	DOCA_MLX5_QPC_TIMESTAMP_FORMAT_REAL_TIME = 0x2,
};

enum doca_mlx5_wq_type {
	DOCA_MLX5_WQ_TYPE_LINKED_LIST = 0x0,
	DOCA_MLX5_WQ_TYPE_CYCLIC = 0x1,
	DOCA_MLX5_WQ_TYPE_LINKED_LIST_STRIDING_RQ = 0x2,
	DOCA_MLX5_WQ_TYPE_CYCLIC_STRIDING_RQ = 0x3,
};

enum doca_mlx5_cqe_size {
	DOCA_MLX5_CQE_SIZE_64B = 0x0,
	DOCA_MLX5_CQE_SIZE_128B = 0x1,
};

/* Query messages. */
struct doca_mlx5_ifc_cmd_hca_cap_bits {
	uint8_t reserved_at_0[0x13];
	uint8_t log_sha_mmo_max_size[0x5];
	uint8_t reserved_at_18[0x18];
	uint8_t vhca_id[0x10];
	uint8_t reserved_at_40[0x20];
	uint8_t reserved_at_60[0x3];
	uint8_t log_regexp_scatter_gather_size[0x5];
	uint8_t reserved_at_68[0x3];
	uint8_t log_dma_mmo_size[0x5];
	uint8_t reserved_at_70[0x3];
	uint8_t log_compress_mmo_size[0x5];
	uint8_t reserved_at_78[0x3];
	uint8_t log_decompress_mmo_size[0x5];
	uint8_t log_max_srq_sz[0x8];
	uint8_t log_max_qp_sz[0x8];
	uint8_t reserved_at_90[0x9];
	uint8_t wqe_index_ignore_cap[0x1];
	uint8_t dynamic_qp_allocation[0x1];
	uint8_t log_max_qp[0x5];
	uint8_t reserved_at_a0[0x4];
	uint8_t regexp_num_of_engines[0x4];
	uint8_t reserved_at_a8[0x1];
	uint8_t reg_c_preserve[0x1];
	uint8_t reserved_at_aa[0x1];
	uint8_t log_max_srq[0x5];
	uint8_t reserved_at_b0[0xb];
	uint8_t scatter_fcs_w_decap_disable[0x1];
	uint8_t reserved_at_bc[0x4];
	uint8_t reserved_at_c0[0x8];
	uint8_t log_max_cq_sz[0x8];
	uint8_t reserved_at_d0[0xb];
	uint8_t log_max_cq[0x5];
	uint8_t log_max_eq_sz[0x8];
	uint8_t relaxed_ordering_write[0x1];
	uint8_t relaxed_ordering_read[0x1];
	uint8_t access_register_user[0x1];
	uint8_t log_max_mkey[0x5];
	uint8_t reserved_at_f0[0x8];
	uint8_t dump_fill_mkey[0x1];
	uint8_t reserved_at_f9[0x3];
	uint8_t log_max_eq[0x4];
	uint8_t max_indirection[0x8];
	uint8_t fixed_buffer_size[0x1];
	uint8_t log_max_mrw_sz[0x7];
	uint8_t force_teardown[0x1];
	uint8_t reserved_at_111[0x1];
	uint8_t log_max_bsf_list_size[0x6];
	uint8_t umr_extended_translation_offset[0x1];
	uint8_t null_mkey[0x1];
	uint8_t log_max_klm_list_size[0x6];
	uint8_t non_wire_sq[0x1];
	uint8_t reserved_at_121[0x9];
	uint8_t log_max_ra_req_dc[0x6];
	uint8_t reserved_at_130[0x3];
	uint8_t log_max_static_sq_wq[0x5];
	uint8_t reserved_at_138[0x2];
	uint8_t log_max_ra_res_dc[0x6];
	uint8_t reserved_at_140[0xa];
	uint8_t log_max_ra_req_qp[0x6];
	uint8_t rtr2rts_qp_counters_set_id[0x1];
	uint8_t rts2rts_udp_sport[0x1];
	uint8_t rts2rts_lag_tx_port_affinity[0x1];
	uint8_t dma_mmo_sq[0x1];
	uint8_t compress_min_block_size[0x4];
	uint8_t compress_mmo_sq[0x1];
	uint8_t decompress_mmo_sq[0x1];
	uint8_t log_max_ra_res_qp[0x6];
	uint8_t end_pad[0x1];
	uint8_t cc_query_allowed[0x1];
	uint8_t cc_modify_allowed[0x1];
	uint8_t start_pad[0x1];
	uint8_t cache_line_128byte[0x1];
	uint8_t reserved_at_165[0xa];
	uint8_t qcam_reg[0x1];
	uint8_t gid_table_size[0x10];
	uint8_t out_of_seq_cnt[0x1];
	uint8_t vport_counters[0x1];
	uint8_t retransmission_q_counters[0x1];
	uint8_t debug[0x1];
	uint8_t modify_rq_counter_set_id[0x1];
	uint8_t rq_delay_drop[0x1];
	uint8_t max_qp_cnt[0xa];
	uint8_t pkey_table_size[0x10];
	uint8_t vport_group_manager[0x1];
	uint8_t vhca_group_manager[0x1];
	uint8_t ib_virt[0x1];
	uint8_t eth_virt[0x1];
	uint8_t vnic_env_queue_counters[0x1];
	uint8_t ets[0x1];
	uint8_t nic_flow_table[0x1];
	uint8_t eswitch_manager[0x1];
	uint8_t device_memory[0x1];
	uint8_t mcam_reg[0x1];
	uint8_t pcam_reg[0x1];
	uint8_t local_ca_ack_delay[0x5];
	uint8_t port_module_event[0x1];
	uint8_t enhanced_error_q_counters[0x1];
	uint8_t ports_check[0x1];
	uint8_t reserved_at_1b3[0x1];
	uint8_t disable_link_up[0x1];
	uint8_t beacon_led[0x1];
	uint8_t port_type[0x2];
	uint8_t num_ports[0x8];
	uint8_t reserved_at_1c0[0x1];
	uint8_t pps[0x1];
	uint8_t pps_modify[0x1];
	uint8_t log_max_msg[0x5];
	uint8_t reserved_at_1c8[0x4];
	uint8_t max_tc[0x4];
	uint8_t temp_warn_event[0x1];
	uint8_t dcbx[0x1];
	uint8_t general_notification_event[0x1];
	uint8_t reserved_at_1d3[0x2];
	uint8_t fpga[0x1];
	uint8_t rol_s[0x1];
	uint8_t rol_g[0x1];
	uint8_t reserved_at_1d8[0x1];
	uint8_t wol_s[0x1];
	uint8_t wol_g[0x1];
	uint8_t wol_a[0x1];
	uint8_t wol_b[0x1];
	uint8_t wol_m[0x1];
	uint8_t wol_u[0x1];
	uint8_t wol_p[0x1];
	uint8_t stat_rate_support[0x10];
	uint8_t reserved_at_1f0[0xc];
	uint8_t cqe_version[0x4];
	uint8_t compact_address_vector[0x1];
	uint8_t striding_rq[0x1];
	uint8_t reserved_at_202[0x1];
	uint8_t ipoib_enhanced_offloads[0x1];
	uint8_t ipoib_basic_offloads[0x1];
	uint8_t reserved_at_205[0x1];
	uint8_t repeated_block_disabled[0x1];
	uint8_t umr_modify_entity_size_disabled[0x1];
	uint8_t umr_modify_atomic_disabled[0x1];
	uint8_t umr_indirect_mkey_disabled[0x1];
	uint8_t umr_fence[0x2];
	uint8_t reserved_at_20c[0x3];
	uint8_t drain_sigerr[0x1];
	uint8_t cmdif_checksum[0x2];
	uint8_t sigerr_cqe[0x1];
	uint8_t reserved_at_213[0x1];
	uint8_t wq_signature[0x1];
	uint8_t sctr_data_cqe[0x1];
	uint8_t reserved_at_216[0x1];
	uint8_t sho[0x1];
	uint8_t tph[0x1];
	uint8_t rf[0x1];
	uint8_t dct[0x1];
	uint8_t qos[0x1];
	uint8_t eth_net_offloads[0x1];
	uint8_t roce[0x1];
	uint8_t atomic[0x1];
	uint8_t reserved_at_21f[0x1];
	uint8_t cq_oi[0x1];
	uint8_t cq_resize[0x1];
	uint8_t cq_moderation[0x1];
	uint8_t reserved_at_223[0x3];
	uint8_t cq_eq_remap[0x1];
	uint8_t pg[0x1];
	uint8_t block_lb_mc[0x1];
	uint8_t reserved_at_229[0x1];
	uint8_t scqe_break_moderation[0x1];
	uint8_t cq_period_start_from_cqe[0x1];
	uint8_t cd[0x1];
	uint8_t reserved_at_22d[0x1];
	uint8_t apm[0x1];
	uint8_t vector_calc[0x1];
	uint8_t umr_ptr_rlky[0x1];
	uint8_t imaicl[0x1];
	uint8_t reserved_at_232[0x4];
	uint8_t qkv[0x1];
	uint8_t pkv[0x1];
	uint8_t set_deth_sqpn[0x1];
	uint8_t reserved_at_239[0x3];
	uint8_t xrc[0x1];
	uint8_t ud[0x1];
	uint8_t uc[0x1];
	uint8_t rc[0x1];
	uint8_t uar_4k[0x1];
	uint8_t reserved_at_241[0x8];
	uint8_t regexp_params[0x1];
	uint8_t uar_sz[0x6];
	uint8_t reserved_at_250[0x8];
	uint8_t log_pg_sz[0x8];
	uint8_t bf[0x1];
	uint8_t driver_version[0x1];
	uint8_t pad_tx_eth_packet[0x1];
	uint8_t reserved_at_263[0x8];
	uint8_t log_bf_reg_size[0x5];
	uint8_t reserved_at_270[0xb];
	uint8_t lag_master[0x1];
	uint8_t num_lag_ports[0x4];
	uint8_t reserved_at_280[0x10];
	uint8_t max_wqe_sz_sq[0x10];
	uint8_t reserved_at_2a0[0xc];
	uint8_t regexp_mmo_sq[0x1];
	uint8_t regexp_version[0x3];
	uint8_t max_wqe_sz_rq[0x10];
	uint8_t max_flow_counter_31_16[0x10];
	uint8_t max_wqe_sz_sq_dc[0x10];
	uint8_t reserved_at_2e0[0x7];
	uint8_t max_qp_mcg[0x19];
	uint8_t reserved_at_300[0x10];
	uint8_t flow_counter_bulk_alloc[0x08];
	uint8_t log_max_mcg[0x8];
	uint8_t reserved_at_320[0x3];
	uint8_t log_max_transport_domain[0x5];
	uint8_t reserved_at_328[0x3];
	uint8_t log_max_pd[0x5];
	uint8_t reserved_at_330[0xb];
	uint8_t log_max_xrcd[0x5];
	uint8_t nic_receive_steering_discard[0x1];
	uint8_t receive_discard_vport_down[0x1];
	uint8_t transmit_discard_vport_down[0x1];
	uint8_t reserved_at_343[0x5];
	uint8_t log_max_flow_counter_bulk[0x8];
	uint8_t max_flow_counter_15_0[0x10];
	uint8_t modify_tis[0x1];
	uint8_t flow_counters_dump[0x1];
	uint8_t reserved_at_360[0x1];
	uint8_t log_max_rq[0x5];
	uint8_t reserved_at_368[0x3];
	uint8_t log_max_sq[0x5];
	uint8_t reserved_at_370[0x3];
	uint8_t log_max_tir[0x5];
	uint8_t reserved_at_378[0x3];
	uint8_t log_max_tis[0x5];
	uint8_t basic_cyclic_rcv_wqe[0x1];
	uint8_t reserved_at_381[0x1];
	uint8_t mem_rq_rmp[0x1];
	uint8_t log_max_rmp[0x5];
	uint8_t reserved_at_388[0x3];
	uint8_t log_max_rqt[0x5];
	uint8_t reserved_at_390[0x3];
	uint8_t log_max_rqt_size[0x5];
	uint8_t reserved_at_398[0x3];
	uint8_t log_max_tis_per_sq[0x5];
	uint8_t ext_stride_num_range[0x1];
	uint8_t reserved_at_3a1[0x2];
	uint8_t log_max_stride_sz_rq[0x5];
	uint8_t reserved_at_3a8[0x3];
	uint8_t log_min_stride_sz_rq[0x5];
	uint8_t reserved_at_3b0[0x3];
	uint8_t log_max_stride_sz_sq[0x5];
	uint8_t reserved_at_3b8[0x3];
	uint8_t log_min_stride_sz_sq[0x5];
	uint8_t hairpin[0x1];
	uint8_t reserved_at_3c1[0x2];
	uint8_t log_max_hairpin_queues[0x5];
	uint8_t reserved_at_3c8[0x3];
	uint8_t log_max_hairpin_wq_data_sz[0x5];
	uint8_t reserved_at_3d0[0x3];
	uint8_t log_max_hairpin_num_packets[0x5];
	uint8_t reserved_at_3d8[0x3];
	uint8_t log_max_wq_sz[0x5];
	uint8_t nic_vport_change_event[0x1];
	uint8_t disable_local_lb_uc[0x1];
	uint8_t disable_local_lb_mc[0x1];
	uint8_t log_min_hairpin_wq_data_sz[0x5];
	uint8_t reserved_at_3e8[0x3];
	uint8_t log_max_vlan_list[0x5];
	uint8_t reserved_at_3f0[0x3];
	uint8_t log_max_current_mc_list[0x5];
	uint8_t reserved_at_3f8[0x3];
	uint8_t log_max_current_uc_list[0x5];
	uint8_t general_obj_types[0x40];
	uint8_t sq_ts_format[0x2];
	uint8_t rq_ts_format[0x2];
	uint8_t steering_format_version[0x4];
	uint8_t reserved_at_448[0x18];
	uint8_t reserved_at_460[0x10];
	uint8_t max_num_eqs[0x10];
	uint8_t reserved_at_480[0x3];
	uint8_t log_max_l2_table[0x5];
	uint8_t reserved_at_488[0x8];
	uint8_t log_uar_page_sz[0x10];
	uint8_t reserved_at_4a0[0x20];
	uint8_t device_frequency_mhz[0x20];
	uint8_t device_frequency_khz[0x20];
	uint8_t reserved_at_500[0x20];
	uint8_t num_of_uars_per_page[0x20];
	uint8_t flex_parser_protocols[0x20];
	uint8_t max_geneve_tlv_options[0x8];
	uint8_t reserved_at_568[0x3];
	uint8_t max_geneve_tlv_option_data_len[0x5];
	uint8_t reserved_at_570[0x49];
	uint8_t mini_cqe_resp_l3_l4_tag[0x1];
	uint8_t mini_cqe_resp_flow_tag[0x1];
	uint8_t enhanced_cqe_compression[0x1];
	uint8_t mini_cqe_resp_stride_index[0x1];
	uint8_t cqe_128_always[0x1];
	uint8_t cqe_compression_128[0x1];
	uint8_t cqe_compression[0x1];
	uint8_t cqe_compression_timeout[0x10];
	uint8_t cqe_compression_max_num[0x10];
	uint8_t reserved_at_5e0[0x10];
	uint8_t tag_matching[0x1];
	uint8_t rndv_offload_rc[0x1];
	uint8_t rndv_offload_dc[0x1];
	uint8_t log_tag_matching_list_sz[0x5];
	uint8_t reserved_at_5f8[0x3];
	uint8_t log_max_xrq[0x5];
	uint8_t affiliate_nic_vport_criteria[0x8];
	uint8_t native_port_num[0x8];
	uint8_t num_vhca_ports[0x8];
	uint8_t reserved_at_618[0x6];
	uint8_t sw_owner_id[0x1];
	uint8_t reserved_at_61f[0x121];
	uint8_t sha_mmo_qp[0x1];
	uint8_t reserved_at_741[0x7];
	uint8_t dma_mmo_qp[0x1];
	uint8_t regexp_mmo_qp[0x1];
	uint8_t compress_mmo_qp[0x1];
	uint8_t decompress_mmo_qp[0x1];
	uint8_t reserved_at_74c[0xb4];
};

struct doca_mlx5_ifc_roce_caps_bits {
	uint8_t reserved_0[0x1e];
	uint8_t qp_ts_format[0x2];
	uint8_t reserved_at_20[0x7e0];
};

union doca_mlx5_ifc_hca_cap_union_bits {
	struct doca_mlx5_ifc_cmd_hca_cap_bits cmd_hca_cap;
	struct doca_mlx5_ifc_roce_caps_bits roce_caps;
	/*
	 * Currently unused capability calls:
	 *
	 *	struct doca_mlx5_ifc_per_protocol_networking_offload_caps_bits;
	 *	struct doca_mlx5_ifc_qos_cap_bits qos_cap;
	 *	struct doca_mlx5_ifc_virtio_emulation_cap_bits vdpa_caps;
	 *	struct doca_mlx5_ifc_flow_table_nic_cap_bits flow_table_nic_cap;
	 */
	uint8_t reserved_at_0[0x8000];
};
struct doca_mlx5_ifc_query_hca_cap_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];
	uint8_t syndrome[0x20];
	uint8_t reserved_at_40[0x40];
	union doca_mlx5_ifc_hca_cap_union_bits capability;
};

struct doca_mlx5_ifc_query_hca_cap_in_bits {
	uint8_t opcode[0x10];
	uint8_t reserved_at_10[0x10];
	uint8_t reserved_at_20[0x10];
	uint8_t op_mod[0x10];
	uint8_t reserved_at_40[0x40];
};

/* Mkey commands. */
struct doca_mlx5_ifc_klm_bits {
	uint8_t byte_count[0x20];
	uint8_t mkey[0x20];
	uint8_t address[0x40];
};

struct doca_mlx5_ifc_mkc_bits {
	uint8_t reserved_at_0[0x1];
	uint8_t free[0x1];
	uint8_t reserved_at_2[0x1];
	uint8_t access_mode_4_2[0x3];
	uint8_t reserved_at_6[0x7];
	uint8_t relaxed_ordering_write[0x1];
	uint8_t reserved_at_e[0x1];
	uint8_t small_fence_on_rdma_read_response[0x1];
	uint8_t umr_en[0x1];
	uint8_t a[0x1];
	uint8_t rw[0x1];
	uint8_t rr[0x1];
	uint8_t lw[0x1];
	uint8_t lr[0x1];
	uint8_t access_mode_1_0[0x2];
	uint8_t reserved_at_18[0x8];
	uint8_t qpn[0x18];
	uint8_t mkey_7_0[0x8];
	uint8_t reserved_at_40[0x20];
	uint8_t length64[0x1];
	uint8_t bsf_en[0x1];
	uint8_t sync_umr[0x1];
	uint8_t reserved_at_63[0x2];
	uint8_t expected_sigerr_count[0x1];
	uint8_t reserved_at_66[0x1];
	uint8_t en_rinval[0x1];
	uint8_t pd[0x18];
	uint8_t start_addr[0x40];
	uint8_t len[0x40];
	uint8_t bsf_octword_size[0x20];
	uint8_t reserved_at_120[0x80];
	uint8_t translations_octword_size[0x20];
	uint8_t reserved_at_1c0[0x19];
	uint8_t relaxed_ordering_read[0x1];
	uint8_t reserved_at_1da[0x1];
	uint8_t log_page_size[0x5];
	uint8_t reserved_at_1e0[0x20];
};

struct doca_mlx5_ifc_create_mkey_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];
	uint8_t syndrome[0x20];
	uint8_t reserved_at_40[0x8];
	uint8_t mkey_index[0x18];
	uint8_t reserved_at_60[0x20];
};

struct doca_mlx5_ifc_create_mkey_in_bits {
	uint8_t opcode[0x10];
	uint8_t reserved_at_10[0x10];
	uint8_t reserved_at_20[0x10];
	uint8_t op_mod[0x10];
	uint8_t reserved_at_40[0x20];
	uint8_t pg_access[0x1];
	uint8_t reserved_at_61[0x1f];
	struct doca_mlx5_ifc_mkc_bits memory_key_mkey_entry;
	uint8_t reserved_at_280[0x80];
	uint8_t translations_octword_actual_size[0x20];
	uint8_t mkey_umem_id[0x20];
	uint8_t mkey_umem_offset[0x40];
	uint8_t reserved_at_380[0x500];
	uint8_t klm_pas_mtt[][0x20];
};

/* sha mmo engine */
struct doca_mlx5_ifc_sha_mmo_control_bits {
	uint8_t reserved_at_31[0x1];
	uint8_t re[0x1];
	uint8_t le[0x1];
	uint8_t reserved_at_28[0x5];
	uint8_t key_size[0x8];
	uint8_t reserved_at_15[0x4];
	uint8_t sha_func[0x4];
	uint8_t reserved_at_7[0x3];
	uint8_t pe[0x1];
	uint8_t reserved_at_3[0x3];
	uint8_t ty[0x1];
};

/* Regex specific commands. */
struct regexp_params_field_select_bits {
	uint8_t reserved_at_0[0x1d];
	uint8_t rof_mkey[0x1];
	uint8_t stop_engine[0x1];
	uint8_t reserved_at_1f[0x1];
};

struct doca_mlx5_ifc_regexp_params_bits {
	uint8_t reserved_at_0[0x1f];
	uint8_t stop_engine[0x1];
	uint8_t reserved_at_20[0x60];
	uint8_t rof_mkey[0x20];
	uint8_t rof_size[0x20];
	uint8_t rof_mkey_va[0x40];
	uint8_t reserved_at_100[0x80];
};

struct doca_mlx5_ifc_set_regexp_params_in_bits {
	uint8_t opcode[0x10];
	uint8_t uid[0x10];
	uint8_t reserved_at_20[0x10];
	uint8_t op_mod[0x10];
	uint8_t reserved_at_40[0x18];
	uint8_t engine_id[0x8];
	struct regexp_params_field_select_bits field_select;
	struct doca_mlx5_ifc_regexp_params_bits regexp_params;
};

struct doca_mlx5_ifc_set_regexp_params_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];
	uint8_t syndrome[0x20];
	uint8_t reserved_at_18[0x40];
};

struct doca_mlx5_ifc_regexp_mmo_control_bits {
	uint8_t reserved_at_31[0x2];
	uint8_t le[0x1];
	uint8_t reserved_at_28[0x1];
	uint8_t subset_id_0[0xc];
	uint8_t reserved_at_16[0x4];
	uint8_t subset_id_1[0xc];
	uint8_t ctrl[0x4];
	uint8_t subset_id_2[0xc];
	uint8_t reserved_at_16_1[0x4];
	uint8_t subset_id_3[0xc];
};

struct doca_mlx5_ifc_compress_mmo_control_bits {
	uint8_t reserved_at_31[0x2];
	uint8_t le[0x1];
	uint8_t reserved_at_28_24[0x5];
	uint8_t dynamic_size[0x4];
	uint8_t block_size[0x4];
	uint8_t window_size[0x4];
	uint8_t reserved_at_15_0[0xc];
};

struct doca_mlx5_ifc_regexp_metadata_bits {
	uint8_t rof_version[0x10];
	uint8_t latency_count[0x10];
	uint8_t instruction_count[0x10];
	uint8_t primary_thread_count[0x10];
	uint8_t match_count[0x8];
	uint8_t detected_match_count[0x8];
	uint8_t status[0x10];
	uint8_t job_id[0x20];
	uint8_t reserved[0x80];
};

struct doca_mlx5_ifc_regexp_match_tuple_bits {
	uint8_t length[0x10];
	uint8_t start_ptr[0x10];
	uint8_t rule_id[0x20];
};

/* QP cmesgs. */
struct doca_mlx5_cqe {
	uint8_t pkt_info;
	uint8_t rsvd0;
	uint16_t wqe_id;
	uint8_t lro_tcppsh_abort_dupack;
	uint8_t lro_min_ttl;
	uint16_t lro_tcp_win;
	uint32_t lro_ack_seq_num;
	uint32_t rx_hash_res;
	uint8_t rx_hash_type;
	uint8_t rsvd1[3];
	uint16_t csum;
	uint8_t rsvd2[6];
	uint16_t hdr_type_etc;
	uint16_t vlan_info;
	uint8_t lro_num_seg;
	union {
		uint8_t user_index_bytes[3];
		struct __attribute__((__packed__)) {
			uint8_t user_index_hi;
			uint16_t user_index_low;
		};
	};
	uint32_t flow_table_metadata;
	uint8_t rsvd4[4];
	uint32_t byte_cnt;
	uint64_t timestamp;
	uint32_t sop_drop_qpn;
	uint16_t wqe_counter;
	uint8_t rsvd5;
	uint8_t op_own;
};

#define DOCA_MLX5_ESEG_EXTRA_DATA_SIZE 32u
#define DOCA_MLX5_DSEG_MIN_INLINE_SIZE 12u

struct __attribute__((__packed__)) doca_mlx5_wqe_dseg {
	uint32_t bcount;
	union {
		uint8_t inline_data[DOCA_MLX5_DSEG_MIN_INLINE_SIZE];
		struct __attribute__((__packed__)) {
			uint32_t lkey;
			uint64_t pbuf;
		};
	};
};

struct __attribute__((__packed__)) doca_mlx5_wqe_eseg {
	union {
		struct __attribute__((__packed__)) {
			uint32_t swp_offs;
			uint8_t cs_flags;
			uint8_t swp_flags;
			uint16_t mss;
			uint32_t metadata;
			uint16_t inline_hdr_sz;
			union {
				uint16_t inline_data;
				uint16_t vlan_tag;
			};
		};
		struct __attribute__((__packed__)) {
			uint32_t offsets;
			uint32_t flags;
			uint32_t flow_metadata;
			uint32_t inline_hdr;
		};
	};
};

struct __attribute__((__packed__)) doca_mlx5_wqe_cseg {
	uint32_t opcode;
	uint32_t sq_ds;
	uint32_t flags;
	uint32_t misc;
};

struct __attribute__((__packed__)) doca_mlx5_wqe {
	union {
		struct doca_mlx5_wqe_cseg cseg;
		uint32_t ctrl[4];
	};
	struct doca_mlx5_wqe_eseg eseg;
	union {
		struct doca_mlx5_wqe_dseg dseg[2];
		uint8_t data[DOCA_MLX5_ESEG_EXTRA_DATA_SIZE];
	};
};

struct doca_mlx5_ifc_ads_bits {
	uint8_t fl[0x1];
	uint8_t free_ar[0x1];
	uint8_t reserved_at_2[0xe];
	uint8_t pkey_index[0x10];
	uint8_t reserved_at_20[0x8];
	uint8_t grh[0x1];
	uint8_t mlid[0x7];
	uint8_t rlid[0x10];
	uint8_t ack_timeout[0x5];
	uint8_t reserved_at_45[0x3];
	uint8_t src_addr_index[0x8];
	uint8_t reserved_at_50[0x4];
	uint8_t stat_rate[0x4];
	uint8_t hop_limit[0x8];
	uint8_t reserved_at_60[0x4];
	uint8_t tclass[0x8];
	uint8_t flow_label[0x14];
	uint8_t rgid_rip[16][0x8];
	uint8_t reserved_at_100[0x4];
	uint8_t f_dscp[0x1];
	uint8_t f_ecn[0x1];
	uint8_t reserved_at_106[0x1];
	uint8_t f_eth_prio[0x1];
	uint8_t ecn[0x2];
	uint8_t dscp[0x6];
	uint8_t udp_sport[0x10];
	uint8_t dei_cfi[0x1];
	uint8_t eth_prio[0x3];
	uint8_t sl[0x4];
	uint8_t vhca_port_num[0x8];
	uint8_t rmac_47_32[0x10];
	uint8_t rmac_31_0[0x20];
};

struct doca_mlx5_ifc_qpc_bits {
	uint8_t state[0x4];
	uint8_t lag_tx_port_affinity[0x4];
	uint8_t st[0x8];
	uint8_t reserved_at_10[0x3];
	uint8_t pm_state[0x2];
	uint8_t reserved_at_15[0x1];
	uint8_t req_e2e_credit_mode[0x2];
	uint8_t offload_type[0x4];
	uint8_t end_padding_mode[0x2];
	uint8_t reserved_at_1e[0x2];
	uint8_t wq_signature[0x1];
	uint8_t block_lb_mc[0x1];
	uint8_t atomic_like_write_en[0x1];
	uint8_t latency_sensitive[0x1];
	uint8_t reserved_at_24[0x1];
	uint8_t drain_sigerr[0x1];
	uint8_t reserved_at_26[0x2];
	uint8_t pd[0x18];
	uint8_t mtu[0x3];
	uint8_t log_msg_max[0x5];
	uint8_t reserved_at_48[0x1];
	uint8_t log_rq_size[0x4];
	uint8_t log_rq_stride[0x3];
	uint8_t no_sq[0x1];
	uint8_t log_sq_size[0x4];
	uint8_t reserved_at_55[0x3];
	uint8_t ts_format[0x2];
	uint8_t reserved_at_5a[0x1];
	uint8_t rlky[0x1];
	uint8_t ulp_stateless_offload_mode[0x4];
	uint8_t counter_set_id[0x8];
	uint8_t uar_page[0x18];
	uint8_t reserved_at_80[0x8];
	uint8_t user_index[0x18];
	uint8_t reserved_at_a0[0x3];
	uint8_t log_page_size[0x5];
	uint8_t remote_qpn[0x18];
	struct doca_mlx5_ifc_ads_bits primary_address_path;
	struct doca_mlx5_ifc_ads_bits secondary_address_path;
	uint8_t log_ack_req_freq[0x4];
	uint8_t reserved_at_384[0x4];
	uint8_t log_sra_max[0x3];
	uint8_t reserved_at_38b[0x2];
	uint8_t retry_count[0x3];
	uint8_t rnr_retry[0x3];
	uint8_t reserved_at_393[0x1];
	uint8_t fre[0x1];
	uint8_t cur_rnr_retry[0x3];
	uint8_t cur_retry_count[0x3];
	uint8_t reserved_at_39b[0x5];
	uint8_t reserved_at_3a0[0x20];
	uint8_t reserved_at_3c0[0x8];
	uint8_t next_send_psn[0x18];
	uint8_t reserved_at_3e0[0x8];
	uint8_t cqn_snd[0x18];
	uint8_t reserved_at_400[0x8];
	uint8_t deth_sqpn[0x18];
	uint8_t reserved_at_420[0x20];
	uint8_t reserved_at_440[0x8];
	uint8_t last_acked_psn[0x18];
	uint8_t reserved_at_460[0x8];
	uint8_t ssn[0x18];
	uint8_t reserved_at_480[0x8];
	uint8_t log_rra_max[0x3];
	uint8_t reserved_at_48b[0x1];
	uint8_t atomic_mode[0x4];
	uint8_t rre[0x1];
	uint8_t rwe[0x1];
	uint8_t rae[0x1];
	uint8_t reserved_at_493[0x1];
	uint8_t page_offset[0x6];
	uint8_t reserved_at_49a[0x3];
	uint8_t cd_slave_receive[0x1];
	uint8_t cd_slave_send[0x1];
	uint8_t cd_master[0x1];
	uint8_t reserved_at_4a0[0x3];
	uint8_t min_rnr_nak[0x5];
	uint8_t next_rcv_psn[0x18];
	uint8_t reserved_at_4c0[0x8];
	uint8_t xrcd[0x18];
	uint8_t reserved_at_4e0[0x8];
	uint8_t cqn_rcv[0x18];
	uint8_t dbr_addr[0x40];
	uint8_t q_key[0x20];
	uint8_t reserved_at_560[0x5];
	uint8_t rq_type[0x3];
	uint8_t srqn_rmpn_xrqn[0x18];
	uint8_t reserved_at_580[0x8];
	uint8_t rmsn[0x18];
	uint8_t hw_sq_wqebb_counter[0x10];
	uint8_t sw_sq_wqebb_counter[0x10];
	uint8_t hw_rq_counter[0x20];
	uint8_t sw_rq_counter[0x20];
	uint8_t reserved_at_600[0x20];
	uint8_t reserved_at_620[0xf];
	uint8_t cgs[0x1];
	uint8_t cs_req[0x8];
	uint8_t cs_res[0x8];
	uint8_t dc_access_key[0x40];
	uint8_t reserved_at_680[0x3];
	uint8_t dbr_umem_valid[0x1];
	uint8_t reserved_at_684[0x9c];
	uint8_t dbr_umem_id[0x20];
};

struct doca_mlx5_ifc_rtr2rts_qp_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];
	uint8_t syndrome[0x20];
	uint8_t reserved_at_40[0x40];
};

struct doca_mlx5_ifc_rtr2rts_qp_in_bits {
	uint8_t opcode[0x10];
	uint8_t uid[0x10];
	uint8_t reserved_at_20[0x10];
	uint8_t op_mod[0x10];
	uint8_t reserved_at_40[0x8];
	uint8_t qpn[0x18];
	uint8_t reserved_at_60[0x20];
	uint8_t opt_param_mask[0x20];
	uint8_t reserved_at_a0[0x20];
	struct doca_mlx5_ifc_qpc_bits qpc;
	uint8_t reserved_at_800[0x80];
};

struct doca_mlx5_ifc_rst2init_qp_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];
	uint8_t syndrome[0x20];
	uint8_t reserved_at_40[0x40];
};

struct doca_mlx5_ifc_rst2init_qp_in_bits {
	uint8_t opcode[0x10];
	uint8_t uid[0x10];
	uint8_t reserved_at_20[0x10];
	uint8_t op_mod[0x10];
	uint8_t reserved_at_40[0x8];
	uint8_t qpn[0x18];
	uint8_t reserved_at_60[0x20];
	uint8_t opt_param_mask[0x20];
	uint8_t reserved_at_a0[0x20];
	struct doca_mlx5_ifc_qpc_bits qpc;
	uint8_t reserved_at_800[0x80];
};

struct doca_mlx5_ifc_init2rtr_qp_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];
	uint8_t syndrome[0x20];
	uint8_t reserved_at_40[0x40];
};

struct doca_mlx5_ifc_init2rtr_qp_in_bits {
	uint8_t opcode[0x10];
	uint8_t uid[0x10];
	uint8_t reserved_at_20[0x10];
	uint8_t op_mod[0x10];
	uint8_t reserved_at_40[0x8];
	uint8_t qpn[0x18];
	uint8_t reserved_at_60[0x20];
	uint8_t opt_param_mask[0x20];
	uint8_t reserved_at_a0[0x20];
	struct doca_mlx5_ifc_qpc_bits qpc;
	uint8_t reserved_at_800[0x80];
};

struct doca_mlx5_ifc_create_qp_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];
	uint8_t syndrome[0x20];
	uint8_t reserved_at_40[0x8];
	uint8_t qpn[0x18];
	uint8_t reserved_at_60[0x20];
};

struct doca_mlx5_ifc_qpc_extension_bits {
	uint8_t reserved_at_0[0x2];
	uint8_t mmo[0x1];
	uint8_t reserved_at_3[0x5fd];
};

struct doca_mlx5_ifc_qpc_pas_list_bits {
	uint8_t pas[0][0x40];
};

struct doca_mlx5_ifc_qpc_extension_and_pas_list_bits {
	struct doca_mlx5_ifc_qpc_extension_bits qpc_data_extension;
	uint8_t pas[0][0x40];
};

struct doca_mlx5_ifc_create_qp_in_bits {
	uint8_t opcode[0x10];
	uint8_t uid[0x10];
	uint8_t reserved_at_20[0x10];
	uint8_t op_mod[0x10];
	uint8_t qpc_ext[0x1];
	uint8_t reserved_at_41[0x3f];
	uint8_t opt_param_mask[0x20];
	uint8_t reserved_at_a0[0x20];
	struct doca_mlx5_ifc_qpc_bits qpc;
	uint8_t wq_umem_offset[0x40];
	uint8_t wq_umem_id[0x20];
	uint8_t wq_umem_valid[0x1];
	uint8_t reserved_at_861[0x1f];
	union {
		struct doca_mlx5_ifc_qpc_pas_list_bits qpc_pas_list;
		struct doca_mlx5_ifc_qpc_extension_and_pas_list_bits qpc_extension_and_pas_list;
	};
};

struct doca_mlx5_ifc_cqc_bits {
	uint8_t status[0x4];
	uint8_t as_notify[0x1];
	uint8_t initiator_src_dct[0x1];
	uint8_t dbr_umem_valid[0x1];
	uint8_t reserved_at_7[0x1];
	uint8_t cqe_sz[0x3];
	uint8_t cc[0x1];
	uint8_t reserved_at_c[0x1];
	uint8_t scqe_break_moderation_en[0x1];
	uint8_t oi[0x1];
	uint8_t cq_period_mode[0x2];
	uint8_t cqe_comp_en[0x1];
	uint8_t mini_cqe_res_format[0x2];
	uint8_t st[0x4];
	uint8_t reserved_at_18[0x1];
	uint8_t cqe_comp_layout[0x7];
	uint8_t dbr_umem_id[0x20];
	uint8_t reserved_at_40[0x14];
	uint8_t page_offset[0x6];
	uint8_t reserved_at_5a[0x2];
	uint8_t mini_cqe_res_format_ext[0x2];
	uint8_t cq_timestamp_format[0x2];
	uint8_t reserved_at_60[0x3];
	uint8_t log_cq_size[0x5];
	uint8_t uar_page[0x18];
	uint8_t reserved_at_80[0x4];
	uint8_t cq_period[0xc];
	uint8_t cq_max_count[0x10];
	uint8_t reserved_at_a0[0x18];
	uint8_t c_eqn[0x8];
	uint8_t reserved_at_c0[0x3];
	uint8_t log_page_size[0x5];
	uint8_t reserved_at_c8[0x18];
	uint8_t reserved_at_e0[0x20];
	uint8_t reserved_at_100[0x8];
	uint8_t last_notified_index[0x18];
	uint8_t reserved_at_120[0x8];
	uint8_t last_solicit_index[0x18];
	uint8_t reserved_at_140[0x8];
	uint8_t consumer_counter[0x18];
	uint8_t reserved_at_160[0x8];
	uint8_t producer_counter[0x18];
	uint8_t local_partition_id[0xc];
	uint8_t process_id[0x14];
	uint8_t reserved_at_1A0[0x20];
	uint8_t dbr_addr[0x40];
};

struct doca_mlx5_ifc_create_cq_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];
	uint8_t syndrome[0x20];
	uint8_t reserved_at_40[0x8];
	uint8_t cqn[0x18];
	uint8_t reserved_at_60[0x20];
};

struct doca_mlx5_ifc_create_cq_in_bits {
	uint8_t opcode[0x10];
	uint8_t uid[0x10];
	uint8_t reserved_at_20[0x10];
	uint8_t op_mod[0x10];
	uint8_t reserved_at_40[0x40];
	struct doca_mlx5_ifc_cqc_bits cq_context;
	uint8_t cq_umem_offset[0x40];
	uint8_t cq_umem_id[0x20];
	uint8_t cq_umem_valid[0x1];
	uint8_t reserved_at_2e1[0x1f];
	uint8_t reserved_at_300[0x580];
	uint8_t pas[];
};

struct doca_mlx5_ifc_wq_bits {
	uint8_t wq_type[0x4];
	uint8_t wq_signature[0x1];
	uint8_t end_padding_mode[0x2];
	uint8_t cd_slave[0x1];
	uint8_t reserved_at_8[0x18];
	uint8_t hds_skip_first_sge[0x1];
	uint8_t log2_hds_buf_size[0x3];
	uint8_t reserved_at_24[0x7];
	uint8_t page_offset[0x5];
	uint8_t lwm[0x10];
	uint8_t reserved_at_40[0x8];
	uint8_t pd[0x18];
	uint8_t reserved_at_60[0x8];
	uint8_t uar_page[0x18];
	uint8_t dbr_addr[0x40];
	uint8_t hw_counter[0x20];
	uint8_t sw_counter[0x20];
	uint8_t reserved_at_100[0xc];
	uint8_t log_wq_stride[0x4];
	uint8_t reserved_at_110[0x3];
	uint8_t log_wq_pg_sz[0x5];
	uint8_t reserved_at_118[0x3];
	uint8_t log_wq_sz[0x5];
	uint8_t dbr_umem_valid[0x1];
	uint8_t wq_umem_valid[0x1];
	uint8_t reserved_at_122[0x1];
	uint8_t log_hairpin_num_packets[0x5];
	uint8_t reserved_at_128[0x3];
	uint8_t log_hairpin_data_sz[0x5];
	uint8_t reserved_at_130[0x4];
	uint8_t single_wqe_log_num_of_strides[0x4];
	uint8_t two_byte_shift_en[0x1];
	uint8_t reserved_at_139[0x4];
	uint8_t single_stride_log_num_of_bytes[0x3];
	uint8_t dbr_umem_id[0x20];
	uint8_t wq_umem_id[0x20];
	uint8_t wq_umem_offset[0x40];
	uint8_t reserved_at_1c0[0x440];
};

struct doca_mlx5_ifc_sqc_bits {
	uint8_t rlky[0x1];
	uint8_t cd_master[0x1];
	uint8_t fre[0x1];
	uint8_t flush_in_error_en[0x1];
	uint8_t allow_multi_pkt_send_wqe[0x1];
	uint8_t min_wqe_inline_mode[0x3];
	uint8_t state[0x4];
	uint8_t reg_umr[0x1];
	uint8_t allow_swp[0x1];
	uint8_t hairpin[0x1];
	uint8_t non_wire[0x1];
	uint8_t static_sq_wq[0x1];
	uint8_t regexp_en[0x1];
	uint8_t reserved_at_12[0x8];
	uint8_t ts_format[0x02];
	uint8_t reserved_at_1c[0x4];
	uint8_t reserved_at_20[0x8];
	uint8_t user_index[0x18];
	uint8_t reserved_at_40[0x8];
	uint8_t cqn[0x18];
	uint8_t reserved_at_60[0x8];
	uint8_t hairpin_peer_rq[0x18];
	uint8_t reserved_at_80[0x10];
	uint8_t hairpin_peer_vhca[0x10];
	uint8_t reserved_at_a0[0x50];
	uint8_t packet_pacing_rate_limit_index[0x10];
	uint8_t tis_lst_sz[0x10];
	uint8_t reserved_at_110[0x10];
	uint8_t reserved_at_120[0x40];
	uint8_t reserved_at_160[0x8];
	uint8_t tis_num_0[0x18];
	struct doca_mlx5_ifc_wq_bits wq;
};

struct doca_mlx5_ifc_modify_sq_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];
	uint8_t syndrome[0x20];
	uint8_t reserved_at_40[0x40];
};

struct doca_mlx5_ifc_modify_sq_in_bits {
	uint8_t opcode[0x10];
	uint8_t uid[0x10];
	uint8_t reserved_at_20[0x10];
	uint8_t op_mod[0x10];
	uint8_t sq_state[0x4];
	uint8_t reserved_at_44[0x4];
	uint8_t sqn[0x18];
	uint8_t reserved_at_60[0x20];
	uint8_t modify_bitmask[0x40];
	uint8_t reserved_at_c0[0x40];
	struct doca_mlx5_ifc_sqc_bits ctx;
};

struct doca_mlx5_ifc_create_sq_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];
	uint8_t syndrome[0x20];
	uint8_t reserved_at_40[0x8];
	uint8_t sqn[0x18];
	uint8_t reserved_at_60[0x20];
};

struct doca_mlx5_ifc_create_sq_in_bits {
	uint8_t opcode[0x10];
	uint8_t uid[0x10];
	uint8_t reserved_at_20[0x10];
	uint8_t op_mod[0x10];
	uint8_t reserved_at_40[0xc0];
	struct doca_mlx5_ifc_sqc_bits ctx;
};

#endif /* LIB_DOCA_SHA_BF2_PRM_H_ */
