/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef SHA_BUFFER_POOL_H_
#define SHA_BUFFER_POOL_H_

#include <stddef.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <doca_error.h>
#include <doca_log.h>
#include <id_allocator.h>

#ifdef __cplusplus
extern "C" {
#endif

struct sha_buffer_pool {
	void *addr;			   /**< buffer pool region. */
	uint32_t unit_buf_size;		   /**< single buffer byte count. */
	uint32_t nb_buf;		   /**< Number of buffers in this pool. */
	struct id_allocator *id_alloc;	   /**< id allocator, equivalent to buf index. */
};

doca_error_t sha_buffer_pool_create(struct sha_buffer_pool **pool,
				const uint32_t unit_buf_size, const uint32_t nb_buf,
				uint32_t align_size);

void* get_buf_from_pool(struct sha_buffer_pool *pool);

bool is_buf_valid(const struct sha_buffer_pool *pool, const void *addr, uint32_t *id);

doca_error_t put_buf_to_pool(struct sha_buffer_pool *pool, const void *addr);

void sha_buffer_pool_destroy(struct sha_buffer_pool *pool);

#ifdef __cplusplus
}
#endif

#endif /* SHA_BUFFER_POOL_H_ */
