/*
 * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include "dpi_qosmos_compiler.hpp"
#include "dpi_suricata_compiler.hpp"
#include "dpi_compiler_common.hpp"

int
main(int argc, char **argv)
{
	doca_error_t ret;
	struct in_args dpi_compiler_cfg = {0};

	ret = doca_argp_init("dpi_compiler", &dpi_compiler_cfg);
	if (ret != DOCA_SUCCESS) {
		printf("Failed to init ARGP resources: %s\n", doca_get_error_string(ret));
		return -1;
	}
	ret = register_dpi_compiler_params();
	if (ret != DOCA_SUCCESS)
		return -1;
	ret = doca_argp_start(argc, argv);
	if (ret != DOCA_SUCCESS) {
		printf("Failed to parse ARGP application input: %s\n", doca_get_error_string(ret));
		return -1;
	}

	if (dpi_compiler_cfg.rules_format == SURICATA) {
		if (suricata_main(&dpi_compiler_cfg)) {
			printf("Error: failed to run DPI compiler with Suricata rules\n");
			return -1;
		}
	} else if (dpi_compiler_cfg.rules_format == QOSMOS) {
		if (qosmos_main(&dpi_compiler_cfg)) {
			printf("Error: failed to run DPI compiler with PDD rules\n");
			return -1;
		}
	}
	return 0;
}
