/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef __SURICATA_PARSER_HPP__
#define __SURICATA_PARSER_HPP__

#include "dpi_compiler.hpp"
#include <netinet/in.h>
#include <string.h>

enum input_parse_ret {
	BREAK,
	SUCCESS,
	ERROR
};

enum l3_l4_protocols {
	DOCA_DPI_UDP = 17,
	DOCA_DPI_TCP = 6,
	DOCA_DPI_ICMP = 1,
	DOCA_DPI_IP = 4,
	DOCA_DPI_IP6 = 41
};

struct doca_dpi_suricata_parser_in_arg {
	const char *input_file; /* name for the suricata rules file */
	const char *cdo_file_name; /* name for the CDO file */
	const char *classtype_file; /* name for the input classtype file */
};

struct port {
	bool valid;
	uint16_t start;
	uint16_t end;
};

struct suricata_hdr {
	char *action;
	int l7_protocol;
	int l4_protocol;
	int l3_protocol;
	char *src_ip;
	int src_netmask;
	char *dst_ip;
	int dst_netmask;
	struct port src_port;
	struct port dst_port;
	int direction;
};

struct sig_global_options {
	int rule_count;
	int flow_dir;
	int ip_proto;
	int itype;
	int icode;
	int sid;
	int gid;
	char *msg;
	int sticky_field;
	int sticky_prot;
	int priority;
	char *classtype;
};

enum doca_dpi_rule_mask {
	DOCA_DPI_L7_PROTOCOL_MASK	= 1 << 0,
	DOCA_DPI_L4_PROTOCOL_MASK	= 1 << 1,
	DOCA_DPI_L3_PROTOCOL_MASK	= 1 << 2,
	DOCA_DPI_FIELD_MASK		= 1 << 3,
	DOCA_DPI_PORT_MASK		= 1 << 4,
	DOCA_DPI_REGEX_MASK		= 1 << 5,
	DOCA_DPI_DIRECTION_MASK		= 1 << 6,
	DOCA_DPI_OFFSET_MASK		= 1 << 7,
	DOCA_DPI_IP_MASK_MASK		= 1 << 8,
	DOCA_DPI_DST_IP_MASK		= 1 << 9,
	DOCA_DPI_DST_IP6_MASK		= 1 << 10,
	DOCA_DPI_ANCHOR_MASK		= 1 << 11,
	DOCA_DPI_DOLLAR_MASK		= 1 << 12,
	DOCA_DPI_NOCASE_MASK		= 1 << 13,
	DOCA_DPI_IP_PROTO_MASK		= 1 << 14,
	DOCA_DPI_PRIORITY_MASK		= 1 << 15,
};

struct doca_dpi_rule {
	int l3_protocol;
	int l4_protocol;
	int l7_protocol;
	int field;
	char *regex; /* NULL if not in use */
	int direction;
	int offset;
	int bitmask;
	bool anchor;
	bool dollar;
	bool nocase;
};

#endif /* __SURICATA_PARSER_HPP__ */
