%{
	#include "dpi_suricata_parser.tab.hpp"
	#include "dpi_compiler.hpp"
	#include "dpi_suricata_parser.hpp"
%}
%option noyywrap
%option nounput
%option noinput
%x STRING_VAL IGNORE_LINE IGNORE_STR CLASSTYPE_STR CLASSTYPE_NUM
%%

#					{ BEGIN(IGNORE_LINE); }
[ \t]|;					{;} /* ignore all whitespaces and ; */
"->"					{ return CTS_DIR; }
"<>"					{ return BOTH_DIR; }
alert|pass|drop|reject|rejectsrc	{ yylval.sval = strdup(yytext);
					  return ACTION;}
rejectdst|rejectboth			{ yylval.sval = strdup(yytext);
					  return ACTION;}
dns					{ yylval.ival = DOCA_DPI_DNS;
					  return LAYER7; }
tls					{ yylval.ival = DOCA_DPI_TLS;
					  return LAYER7; }
ssl					{ yylval.ival = DOCA_DPI_SSL;
					  return LAYER7; }
sip					{ yylval.ival = DOCA_DPI_SIP;
					  return LAYER7; }
http					{ yylval.ival = DOCA_DPI_HTTP;
					  return LAYER7; }
tcp					{ yylval.ival = DOCA_DPI_TCP;
					  return LAYER4; }
udp					{ yylval.ival = DOCA_DPI_UDP;
					  return LAYER4; }
ip					{ yylval.ival = DOCA_DPI_IP;
					  return LAYER3; }
ip6					{ yylval.ival = DOCA_DPI_IP6;
					  return LAYER3; }
dns\.					{ yylval.ival = DOCA_DPI_DNS;
					  return LAYER7_BEFORE; }
tls\.					{ yylval.ival = DOCA_DPI_TLS;
					  return LAYER7_BEFORE; }
ssl\.					{ yylval.ival = DOCA_DPI_SSL;
					  return LAYER7_BEFORE; }
sip\.					{ yylval.ival = DOCA_DPI_SIP;
					  return LAYER7_BEFORE; }
http\.					{ yylval.ival = DOCA_DPI_HTTP;
					  return LAYER7_BEFORE; }
tcp\.					{ yylval.ival = DOCA_DPI_TCP;
					  return LAYER4_BEFORE; }
udp\.					{ yylval.ival = DOCA_DPI_UDP;
					  return LAYER4_BEFORE; }
dns_					{ yylval.ival = DOCA_DPI_DNS;
					  return LAYER7_AFTER; }
tls_					{ yylval.ival = DOCA_DPI_TLS;
					  return LAYER7_AFTER; }
sip_					{ yylval.ival = DOCA_DPI_SIP;
					  return LAYER7_AFTER; }
http_					{ yylval.ival = DOCA_DPI_HTTP;
					  return LAYER7_AFTER; }
tcp_					{ yylval.ival = DOCA_DPI_TCP;
					  return LAYER4_AFTER; }
udp_					{ yylval.ival = DOCA_DPI_UDP;
					  return LAYER4_AFTER; }
ip_proto				{ return IP_PROT_OPT;}
:					{ return COLON;}
([0-9a-f]{1,4}[:\.]){3,7}[0-9a-f]{1,4}	{ yylval.sval = strdup(yytext);
					  return ADDR;}
\/					{ return BACKSLESH;}
any|\$[A-Z_]+				{ return IGNORE;}
msg[ \t]*:[ \t]*\"			{ BEGIN(STRING_VAL);
					  return MSG_OPT; }
flow					{ return FLOW_OPT; }
to_server|from_client			{ return CTS_DIR; }
to_client|from_server			{ return STC_DIR; }
[0-9]+					{ yylval.ival = atoi(yytext);
					  return INT;}
-					{ return MINUS;}
\(					{ return ROUND_BRACKET;}
\[|\]					{ return SQUARE_BRACKET;}
startswith				{ return ANCHOR; }
endswith				{ return DOLLAR; }
nocase					{ return NOCASE; }
offset					{ return OFFSET; }
priority				{ return PRIORITY; }
classtype:				{ BEGIN(CLASSTYPE_STR);
					  return CLASSTYPE; }
rev|metadata				{ BEGIN(IGNORE_STR); }
uri					{ yylval.ival = DOCA_DPI_HTTP_URI;
					  return OPT_STRING; }
cookie					{ yylval.ival = DOCA_DPI_HTTP_COOKIE;
					  return OPT_STRING; }
user_agent				{ yylval.ival = DOCA_DPI_HTTP_USER_AGENT;
					  return OPT_STRING; }
host					{ yylval.ival = DOCA_DPI_HTTP_HOST;
					  return OPT_STRING; }
server_body				{ yylval.ival = DOCA_DPI_HTTP_CONTENT;
					  return OPT_STRING; }
client_body				{ yylval.ival = DOCA_DPI_HTTP_CONTENT;
					  return OPT_STRING; }
referer					{ yylval.ival = DOCA_DPI_HTTP_REFERER;
					  return OPT_STRING; }
content_type				{ yylval.ival = DOCA_DPI_HTTP_CONTENT_TYPE;
					  return OPT_STRING; }
server					{ yylval.ival = DOCA_DPI_HTTP_SERVER;
					  return OPT_STRING; }
location				{ yylval.ival = DOCA_DPI_HTTP_LOCATION;
					  return OPT_STRING; }
request_body				{ yylval.ival = DOCA_DPI_HTTP_CONTENT;
					  return OPT_STRING; }
response_body				{ yylval.ival = DOCA_DPI_HTTP_CONTENT;
					  return OPT_STRING; }
hdr					{ yylval.ival = DOCA_DPI_L4_STREAM;
					  return OPT_STRING; }
query					{ yylval.ival = DOCA_DPI_DNS_QUERY;
					  return OPT_STRING; }
sni					{ yylval.ival = DOCA_DPI_SSL_COMMON_NAME;
					  return OPT_STRING; }
ssl_version				{ return SSL_RECORD_VERSION; }
tls\.version				{ return SSL_HANDSHAKE_VERSION; }
sslv3					{ yylval.sval = strdup("\\x03\\x00");
					  return VERSION_STR;}
(1\.0)|(tls1\.0)			{ yylval.sval = strdup("\\x03\\x01");
					  return VERSION_STR;}
(1\.1)|(tls1\.1)			{ yylval.sval = strdup("\\x03\\x02");
					  return VERSION_STR;}
(1\.2)|(tls1\.2)			{ yylval.sval = strdup("\\x03\\x03");
					  return VERSION_STR;}
(1\.3)|(tls1\.3)			{ yylval.sval = strdup("\\x03\\x04");
					  return VERSION_STR;}
subject[ \t]*:[ \t]*\"			{ BEGIN(STRING_VAL);
					  yylval.ival = DOCA_DPI_SSL_CERT_SUBJECT;
					  return OPT_STRING; }
cert_subject				{ yylval.ival = DOCA_DPI_SSL_CERT_SUBJECT;
					  return OPT_STRING; }
content[ \t]*:[ \t]*\"			{ BEGIN(STRING_VAL);
					  return CONTENT_OPT; }
pcre[ \t]*:[ \t]*\"			{ BEGIN(STRING_VAL);
					  return PCRE_OPT; }
sid					{ return SID; }
gid					{ return GID; }

\)[ \t]*[\n]*				{ return END_OF_RULE; }

<STRING_VAL>([^\"]|\\\")+		{ yylval.sval = strdup(yytext); }
<STRING_VAL>\"				{ return PARSE_ERROR; }
<STRING_VAL>\"[ \t]*;			{ BEGIN(INITIAL);
					  return VAL_STRING;}

<IGNORE_STR>[^;]+			{ return IGNORED_OPT; }
<IGNORE_STR>;				{ BEGIN(INITIAL); }

<IGNORE_LINE>[^\n]			{;} /* ignore all characters except for new-line */
<IGNORE_LINE>\n				{ BEGIN(INITIAL); }

"config classification: "		{ BEGIN(CLASSTYPE_STR);
					  return CONFIG_CLASSTYPE; } /* classtype priority parse */
<CLASSTYPE_STR>[ \t]			{;}
<CLASSTYPE_STR>,			{ return COMMA; }
<CLASSTYPE_STR>[0-9a-z-]+		{ yylval.sval = strdup(yytext);
					  return PRIORITY_STR; }
<CLASSTYPE_STR>;			{ BEGIN(INITIAL); }
<CLASSTYPE_STR>[A-Z][^,]+		{ BEGIN(CLASSTYPE_NUM);
					  return ALERT_STR; }
<CLASSTYPE_NUM>,			{ return COMMA; }
<CLASSTYPE_NUM>[0-9]+			{ yylval.ival = atoi(yytext);
					  return INT;}
<CLASSTYPE_NUM>[\n]+			{ BEGIN(INITIAL);
					  return FINISH_CLASSTYPE; }

%%
/* ((\\\")/(\\\")|[^\"])+      example for including \" that dont work at the moment  */
