/*
 * @file    rxp_qosmos_class.cpp
 * @author  Mark McMullan <mmcmullan@mellanox.com>
 *
 * @section LICENSE
 *
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */


#ifndef SRC_RXP_QOSMOS_CLASS_HPP
#define SRC_RXP_QOSMOS_CLASS_HPP
#include <stdint.h>
#include <vector>
#include <string>
#include <bitset>

enum rxpc_number_base {
	BASE_OCTAL = 8,
	BASE_DECIMAL = 10,
	BASE_HEX = 16
};
class rxp_qosmos_class {
public:
	rxp_qosmos_class();
	virtual ~rxp_qosmos_class();

	bool convert_qosmos_rule_to_rxp(std::string & in_rule, std::string & out_rule,
					std::string & err_text);

private:
	void scan_and_convert_qosmos_rule(std::string & in_rule, std::string & out_rule,
					  bool & data_found, bool & rule_start_anchor,
					  bool & end_dot_star_found, int & end_dot_star_index,
					  bool & rule_is_group);
	//get a character and increment the offset
	unsigned char get_c(int & char_pointer, std::string & current_rule);

	unsigned char peek_c(int & char_pointer, std::string & current_rule);

	void  unget_c(int & char_pointer);

	unsigned char compile_numeric_value(int & char_pointer, std::string & current_rule,
					    rxpc_number_base base, bool bracket_delimited,
					    int override_number_of_characters);
	void capture_until_limit(int & char_pointer, std::string & current_rule, bool & is_quoted,
				 char name_delimiter, std::string & current_literal);

	void compile_hex_string(const std::string & hex_string, std::string & escaped_string);

	std::string message_specific_error(std::string message, int & position);
	std::string message_unexpected_char(std::string message, unsigned char & c, int & position);
};

#endif /* SRC_RXP_QOSMOS_CLASS_HPP */
