/**
 * @file    rxp_qosmos_utility.cpp
 * @author  Mark McMullan <mmcmullan@mellanox.com>
 *
 * @section LICENSE
 *
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <stdio.h>
#include <string.h>
#include <sstream>
#include <sys/types.h>
#include <dirent.h>
#include <vector>
#include <iostream>
#include <algorithm>
#include <iomanip>
#include <map>

#include "rxp_qosmos_general_utils.hpp"

using namespace std;

general_utilities::general_utilities()
{

}

general_utilities::~general_utilities()
{

}

string general_utilities::int2hexstr(uint64_t number)
{
   stringstream ss;
   ss << hex << number;
   return ss.str();
}

uint64_t general_utilities::hex2int (const string &str)
{
    // If the string is "-" this is the hex file representation of no data or 0
    if (str == "-")
    {
        return 0;
    }
    uint64_t value;
    stringstream ss;
    ss << hex << str;
    if((ss >> value).fail())
    {
        return -1;
    }
    else
    {
        return value;
    }
}

uint64_t general_utilities::hex2int (unsigned char &c)
{
    uint64_t value;
    stringstream ss;
    ss << hex << c;
    if((ss >> value).fail())
    {
        return -1;
    }
    else
    {
        return value;
    }
}

int general_utilities::str2int (const string &str)
{
    // If the string is "-" this is the hex file representation of no data or 0
    if (str == "-")
    {
        return 0;
    }
    stringstream ss(str);
    int value;
    if((ss >> value).fail())
    {
        return -1;
    }
    else
    {
        return value;
    }
}

int general_utilities::str2int (char * &str)
{
    stringstream ss(str);
    int value;
    if((ss >> value).fail())
    {
        return -1;
    }
    else
    {
        return value;
    }
}

unsigned int general_utilities::str2uint (const string &str)
{
    stringstream ss(str);
    unsigned int value;
    if((ss >> value).fail())
    {
        return -1;
    }
    else
    {
        return value;
    }
}

unsigned int general_utilities::str2uint (char * &str)
{
    stringstream ss(str);
    unsigned int value;
    if((ss >> value).fail())
    {
        return -1;
    }
    else
    {
        return value;
    }
}

void general_utilities::trim(string & str)
{
    const std::string& whitespace = " \t\n\r\f\v";
    size_t found;

    found = str.find_first_not_of(whitespace);
    // all characters are white-spaces
    if (found == std::string::npos) {
        str = "";
        return;
    }

    str.erase(0, found);

    found = str.find_last_not_of(whitespace);
    if (found != std::string::npos)
        str.erase(found + 1);
}

vector<string> general_utilities::split(const string &s, char delim)
{
    stringstream ss(s);
    string item;
    vector<string> tokens;
    while (getline(ss, item, delim))
    {
        tokens.push_back(item);
    }
    return tokens;
}



uint8_t general_utilities::extract_escaped_hex_byte(string str)
{

    if (str[0] == '\\' && str[1] == 'x' && str.length() == 4 )
    {
        uint8_t value = general_utilities::hex2int(str.substr(2, 2));
        return value;
    }
    throw "Unable to interpret escaped hex value: " + str;
}
