/**
 * @file    rxp_file_utilities.h
 * @author  Gareth Douglas <gdouglas@mellanox.com>
 *
 * @section LICENSE
 *
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef GENERAL_UTILITIES_HPP_
#define GENERAL_UTILITIES_HPP_

#include <stdint.h>
#include <vector>
#include <string>

enum rxp_status_code {
	STATUS_OK,
	FILE_NOT_FOUND,
	RULE_FAILED
};

class general_utilities
{
public:
	general_utilities();
	virtual ~general_utilities();
	//Returns -1 if it fails to convert
	//otherwise the int value extracted
	//from the hex number is returned
	static uint64_t hex2int (const std::string & str);
	//or the version that accepts an unsigned char
	static uint64_t hex2int (unsigned char & c);
	//Takes in an integer and converts
	//to a string in hex format, returning the string
	static std::string int2hexstr(uint64_t number);

	//Returns -1 if it fails to convert
	//otherwise the new int value
	//will be returned.
	static int str2int (const std::string & str);
	static int str2int (char * &str);
	static unsigned int str2uint (const std::string & str);
	static unsigned int str2uint (char * &str);

	//Remove spaces and new lines
	static void trim(std::string & str);

	static std::vector < std::string > split(const std::string & s, char delim);

	// Return the op code value for the desired instruction

	static uint8_t extract_escaped_hex_byte(std::string str);
};

#endif /* GENERAL_UTILITIES_HPP_ */
