/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <stdlib.h>

#include <doca_argp.h>
#include <doca_log.h>

#include "socket_relay_core.h"

DOCA_LOG_REGISTER(SOCKET_RELAY);

/*
 * Socket Relay tool main function
 *
 * @argc [in]: command line arguments size
 * @argv [in]: array of command line arguments
 * @return: EXIT_SUCCESS on success and EXIT_FAILURE otherwise
 */
int
main(int argc, char *argv[])
{
	doca_error_t result;
	struct sr_cfg sr_cfg = {0};
	struct cc_listener_ctx cc_listener_ctx = {0};
	struct cc_sender_ctx cc_sender_ctx = {0};
	struct dpu_clients_listener_ctx dpu_listen_to_uds_clients_ctx = {0};
	struct host_clients_acceptor_ctx host_accept_uds_clients_ctx = {0};
	struct host_clients_listener_ctx host_listen_to_uds_clients_ctx = {0};

#ifdef DOCA_ARCH_DPU
	sr_cfg.mode = SR_MODE_DPU;
#endif

	result = doca_argp_init("socket_relay", &sr_cfg);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to init ARGP resources: %s", doca_get_error_string(result));
		return EXIT_FAILURE;
	}
	result = register_sr_params();
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to register application params: %s", doca_get_error_string(result));
		doca_argp_destroy();
		return EXIT_FAILURE;
	}
	result = doca_argp_start(argc, argv);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to parse application input: %s", doca_get_error_string(result));
		doca_argp_destroy();
		return EXIT_FAILURE;
	}

	result = sr_start(&sr_cfg, &cc_listener_ctx, &cc_sender_ctx, &host_accept_uds_clients_ctx,
			  &host_listen_to_uds_clients_ctx, &dpu_listen_to_uds_clients_ctx);
	if (result != DOCA_SUCCESS) {
		doca_argp_destroy();
		return EXIT_FAILURE;
	}

	doca_argp_destroy();

	return EXIT_SUCCESS;
}
